@@ -25,7 +25,7 @@ import (
 	"strings"
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"gopkg.in/yaml.v2"
 )
@@ -734,7 +734,7 @@ partitions:
       - name: root
     placementrules:
       - name: fixed
-        value: default
+        value: root
       - name: tag
         value: Just Any value
 `
@@ -844,6 +844,7 @@ partitions:
       - name: user
         parent:
           name: fixed
+          value: test
           parent:
             name: provided
 `
@@ -994,6 +995,194 @@ partitions:
 	}
 }
 
+//nolint:funlen
+func TestSameLevelQueueLimitsResourceFail(t *testing.T) {
+	// Make sure limit max apps exceed queue max apps will failed
+	data := `
+partitions:
+  - name: default
+    queues:
+      - name: root
+        limits:
+          - limit:
+            users: 
+            - user1
+            maxresources: {memory: 10000, vcore: 10}
+            maxapplications: 5
+          - limit:
+            users:
+            - user2
+            groups:
+            - prod
+            maxapplications: 10
+        queues:
+          - name: level1
+            maxapplications: 100
+            resources:
+              guaranteed:
+                {memory: 1000, vcore: 10}
+              max:
+                {memory: 10000, vcore: 10}
+            limits:
+              - limit:
+                users: 
+                - test
+                maxapplications: 1000
+                maxresources: {memory: 10000, vcore: 10}
+`
+	// validate the config and check after the update
+	_, err := CreateConfig(data)
+	assert.ErrorContains(t, err, "invalid MaxApplications settings for limit")
+
+	// Make sure limit max apps not set, but queue limit set, will fail.
+	data = `
+partitions:
+  - name: default
+    queues:
+      - name: root
+        limits:
+          - limit:
+            users: 
+            - user1
+            maxresources: {memory: 10000, vcore: 10}
+            maxapplications: 5
+          - limit:
+            users:
+            - user2
+            groups:
+            - prod
+            maxapplications: 10
+        queues:
+          - name: level1
+            maxapplications: 100
+            resources:
+              guaranteed:
+                {memory: 1000, vcore: 10}
+              max:
+                {memory: 10000, vcore: 10}
+            limits:
+              - limit:
+                users: 
+                - test
+                maxresources: {memory: 10000, vcore: 10}
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "MaxApplications is 0 in limit name")
+
+	// Make sure limit max apps set, but queue limit not set, will not fail.
+	data = `
+partitions:
+  - name: default
+    queues:
+      - name: root
+        limits:
+          - limit:
+            users: 
+            - user1
+            maxresources: {memory: 10000, vcore: 10}
+            maxapplications: 5
+          - limit:
+            users:
+            - user2
+            groups:
+            - prod
+            maxapplications: 10
+        queues:
+          - name: level1
+            resources:
+              guaranteed:
+                {memory: 1000, vcore: 10}
+              max:
+                {memory: 10000, vcore: 10}
+            limits:
+              - limit:
+                users: 
+                - test
+                maxresources: {memory: 10000, vcore: 10}
+                maxapplications: 100
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.NilError(t, err)
+
+	// Make sure limit max resources exceed queue max resources will failed
+	data = `
+partitions:
+  - name: default
+    queues:
+      - name: root
+        maxapplications: 50
+        limits:
+          - limit:
+            users: 
+            - user1
+            maxresources: {memory: 10000, vcore: 10}
+            maxapplications: 5
+          - limit:
+            users:
+            - user2
+            groups:
+            - prod
+            maxapplications: 3
+        queues:
+          - name: level1
+            maxapplications: 100
+            resources:
+              guaranteed:
+                {memory: 1000, vcore: 10}
+              max:
+                {memory: 10000, vcore: 10}
+            limits:
+              - limit:
+                users: 
+                - test
+                maxapplications: 5
+                maxresources: {memory: 100000, vcore: 100}
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "invalid MaxResources settings for limit")
+
+	// Make sure queue max resource parse fail will return err
+	data = `
+partitions:
+  - name: default
+    queues:
+      - name: root
+        maxapplications: 50
+        limits:
+          - limit:
+            users: 
+            - user1
+            maxresources: {memory: 10000, vcore: 10}
+            maxapplications: 5
+          - limit:
+            users:
+            - user2
+            groups:
+            - prod
+            maxapplications: 3
+        queues:
+          - name: level1
+            maxapplications: 100
+            resources:
+              guaranteed:
+                {memory: 1000, vcore: 10}
+              max:
+                {memory: parseFailed, vcore: 10}
+            limits:
+              - limit:
+                users: 
+                - test
+                maxapplications: 5
+                maxresources: {memory: 100000, vcore: 100}
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "parse queue level1 max resource failed")
+}
+
 func TestComplexUsers(t *testing.T) {
 	data := `
 partitions:
@@ -1011,6 +1200,10 @@ partitions:
         users:
         - "*"
         maxapplications: 1
+      - limit: no wildcard group
+        groups:
+        - "test"
+        maxapplications: 1
       - limit: wildcard group
         groups:
         - "*"
@@ -1025,7 +1218,7 @@ partitions:
 	if len(conf.Partitions[0].Queues) != 1 && len(conf.Partitions[0].Queues[0].Limits) != 0 {
 		t.Errorf("failed to load queues from config: %v", conf)
 	}
-	if len(conf.Partitions[0].Limits) != 4 {
+	if len(conf.Partitions[0].Limits) != 5 {
 		t.Errorf("failed to load partition limits from config: %v", conf)
 	}
 
@@ -1175,6 +1368,283 @@ partitions:
 	if err == nil {
 		t.Errorf("limit parsing should have failed group @: %v", conf)
 	}
+
+	data = `
+partitions:
+  - name: default
+    limits:
+      - limit: dot user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: "@ user"
+        users:
+        - user@domain
+        maxapplications: 1
+      - limit: wildcard user
+        users:
+        - "*"
+        maxapplications: 1
+      - limit: no wildcard group
+        groups:
+        - "test"
+        maxapplications: 1
+      - limit: wildcard group
+        groups:
+        - "*"
+        maxapplications: 1
+      - limit: error no wildcard user after wildcard user
+        users:
+        - "user1"
+        maxapplications: 1
+    queues:
+      - name: root
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "should not set no wildcard user user1 after wildcard user limit")
+
+	data = `
+partitions:
+  - name: default
+    limits:
+      - limit: dot user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: "@ user"
+        users:
+        - user@domain
+        maxapplications: 1
+      - limit: wildcard user
+        users:
+        - "*"
+        maxapplications: 1
+      - limit: no wildcard group
+        groups:
+        - "test"
+        maxapplications: 1
+      - limit: wildcard group
+        groups:
+        - "*"
+        maxapplications: 1
+      - limit: error no wildcard group after wildcard user
+        groups:
+        - "group1"
+        maxapplications: 1
+    queues:
+      - name: root
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "should not set no wildcard group group1 after wildcard group limit")
+
+	data = `
+partitions:
+  - name: default
+    limits:
+      - limit: dot user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: "@ user"
+        users:
+        - user@domain
+        maxapplications: 1
+      - limit: wildcard user
+        users:
+        - "*"
+        maxapplications: 1
+      - limit: no wildcard group
+        groups:
+        - "test"
+        maxapplications: 1
+      - limit: wildcard group
+        groups:
+        - "*"
+        maxapplications: 1
+      - limit: more than one wildcard user
+        users:
+        - "*"
+        maxapplications: 2
+    queues:
+      - name: root
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "duplicated user name *")
+
+	data = `
+partitions:
+  - name: default
+    limits:
+      - limit: dot user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: "@ user"
+        users:
+        - user@domain
+        maxapplications: 1
+      - limit: wildcard user
+        users:
+        - "*"
+        maxapplications: 1
+      - limit: no wildcard group
+        groups:
+        - "test"
+        maxapplications: 1
+      - limit: wildcard group
+        groups:
+        - "*"
+        maxapplications: 1
+      - limit: more than one wildcard group
+        groups:
+        - "*"
+        maxapplications: 2
+    queues:
+      - name: root
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "duplicated group name *")
+
+	data = `
+partitions:
+  - name: default
+    limits:
+      - limit: dot user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: "@ user"
+        users:
+        - user@domain
+        maxapplications: 1
+      - limit: wildcard user
+        users:
+        - "*"
+        maxapplications: 1
+      - limit: wildcard group
+        groups:
+        - "*"
+        maxapplications: 1
+    queues:
+      - name: root
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "should not specify only one group limit that is using the wildcard")
+
+	data = `
+partitions:
+  - name: default
+    limits:
+      - limit: dot user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: duplicated user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: "@ user"
+        users:
+        - user@domain
+        maxapplications: 1
+      - limit: wildcard user
+        users:
+        - "*"
+        maxapplications: 1
+      - limit: no wildcard group
+        groups:
+        - "test"
+        maxapplications: 1
+      - limit: wildcard group
+        groups:
+        - "*"
+        maxapplications: 1
+    queues:
+      - name: root
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "duplicated user name user.lastname")
+
+	data = `
+partitions:
+  - name: default
+    limits:
+      - limit: dot user
+        users:
+        - user.lastname
+        maxapplications: 1
+      - limit: "@ user"
+        users:
+        - user@domain
+        maxapplications: 1
+      - limit: wildcard user
+        users:
+        - "*"
+        maxapplications: 1
+      - limit: no wildcard group
+        groups:
+        - "test"
+        maxapplications: 1
+      - limit: duplicated group
+        groups:
+        - "test"
+        maxapplications: 1
+      - limit: wildcard group
+        groups:
+        - "*"
+        maxapplications: 1
+    queues:
+      - name: root
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.ErrorContains(t, err, "duplicated group name test")
+
+	// Make sure different queues can support same username or groupname
+	data = `
+partitions:
+  - name: default
+    queues:
+      - name: root
+        maxapplications: 500
+        limits:
+          - limit:
+            users: 
+            - user1
+            maxresources: {memory: 10000, vcore: 10}
+            maxapplications: 5
+          - limit:
+            users:
+            - user2
+            groups:
+            - prod
+            maxapplications: 3
+        queues:
+          - name: level1
+            maxapplications: 100
+            resources:
+              guaranteed:
+                {memory: 1000, vcore: 10}
+              max:
+                {memory: 10000, vcore: 10}
+            limits:
+              - limit:
+                users: 
+                - user1
+                groups:
+                - prod
+                maxapplications: 5
+                maxresources: {memory: 10000, vcore: 10}
+`
+	// validate the config and check after the update
+	_, err = CreateConfig(data)
+	assert.NilError(t, err, "different queues should support same username or groupname")
 }
 
 func TestLoadSchedulerConfigFromByteArray(t *testing.T) {
