@@ -32,6 +32,7 @@ import (
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/common/security"
 	"github.com/apache/yunikorn-core/pkg/log"
+	"github.com/apache/yunikorn-core/pkg/scheduler/placement/types"
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
 )
 
@@ -65,10 +66,10 @@ var QueueNameRegExp = regexp.MustCompile(`^[a-zA-Z0-9_-]{1,64}$`)
 
 // User and group name check: systems allow different things POSIX is the base but we need to be lenient and allow more.
 // allow upper and lower case, add the @ and . (dot) and officially no length.
-var UserRegExp = regexp.MustCompile(`^[_a-zA-Z][a-zA-Z0-9_.@-]*[$]?$`)
+var UserRegExp = regexp.MustCompile(`^[_a-zA-Z][a-zA-Z0-9:_.@-]*[$]?$`)
 
 // Groups should have a slightly more restrictive regexp (no @ . or $ at the end)
-var GroupRegExp = regexp.MustCompile(`^[_a-zA-Z][a-zA-Z0-9_-]*$`)
+var GroupRegExp = regexp.MustCompile(`^[_a-zA-Z][a-zA-Z0-9:_.-]*$`)
 
 // all characters that make a name different from a regexp
 var SpecialRegExp = regexp.MustCompile(`[\^$*+?()\[{}|]`)
@@ -76,6 +77,21 @@ var SpecialRegExp = regexp.MustCompile(`[\^$*+?()\[{}|]`)
 // The rule maps to a go identifier check that regexp only
 var RuleNameRegExp = regexp.MustCompile(`^[_a-zA-Z][a-zA-Z0-9_]*$`)
 
+type placementStaticPath struct {
+	path      string
+	ruleChain string
+	create    bool
+	ruleNo    int
+}
+
+type placementPathCheckResult int
+
+const (
+	checkOK placementPathCheckResult = iota
+	nonExistingQueue
+	queueNotParent
+)
+
 // Check the ACL
 func checkACL(acl string) error {
 	// trim any white space
@@ -169,9 +185,65 @@ func checkPlacementRules(partition *PartitionConfig) error {
 			return err
 		}
 	}
+
+	placementStaticPaths := getLongestPlacementPaths(partition.PlacementRules)
+	for _, staticPath := range placementStaticPaths {
+		queuePath := staticPath.path
+		parts := strings.Split(strings.ToLower(queuePath), DOT)
+		result := checkQueueHierarchyForPlacement(parts, staticPath.create, partition.Queues)
+		if result == queueNotParent {
+			return fmt.Errorf("placement rule no. #%d (%s) references a queue (%s) which is a leaf",
+				staticPath.ruleNo, staticPath.ruleChain, queuePath)
+		}
+		if result == nonExistingQueue {
+			return fmt.Errorf("placement rule no. #%d (%s) references non-existing queues (%s) and create is 'false'",
+				staticPath.ruleNo, staticPath.ruleChain, queuePath)
+		}
+	}
+
 	return nil
 }
 
+func checkQueueHierarchyForPlacement(path []string, create bool, conf []QueueConfig) placementPathCheckResult {
+	queueName := path[0]
+
+	// no more queues in the configuration
+	if len(conf) == 0 {
+		if !create {
+			return nonExistingQueue
+		}
+		return checkOK
+	}
+
+	var queueConf *QueueConfig
+	for _, queue := range conf {
+		queue := queue
+		if queue.Name == queueName {
+			queueConf = &queue
+			break
+		}
+	}
+
+	// queue not found on this level
+	if queueConf == nil {
+		if !create {
+			return nonExistingQueue
+		}
+		return checkOK
+	}
+
+	if !queueConf.Parent {
+		return queueNotParent
+	}
+
+	if len(path) == 1 {
+		return checkOK
+	}
+
+	path = path[1:]
+	return checkQueueHierarchyForPlacement(path, create, queueConf.Queues)
+}
+
 // Check the specific rule for syntax.
 // The create flag is checked automatically by the config parser and is not checked.
 func checkPlacementRule(rule PlacementRule) error {
@@ -212,21 +284,23 @@ func checkPlacementFilter(filter Filter) error {
 	// anything that does not parse in a list of users is ignored (like ACL list)
 	if len(filter.Users) == 1 {
 		// for a length of 1 we could either have regexp or username
-		isUser := UserRegExp.MatchString(filter.Users[0])
+		user := filter.Users[0]
+		isUser := UserRegExp.MatchString(user)
 		// if it is not a user name it must be a regexp
 		// two step check: first compile if that fails it is
 		if !isUser {
-			if _, err := regexp.Compile(filter.Users[0]); err != nil || !SpecialRegExp.MatchString(filter.Users[0]) {
+			if _, err := regexp.Compile(user); err != nil || !SpecialRegExp.MatchString(user) {
 				return fmt.Errorf("invalid rule filter user list is not a proper list or regexp: %v", filter.Users)
 			}
 		}
 	}
 	if len(filter.Groups) == 1 {
 		// for a length of 1 we could either have regexp or groupname
-		isGroup := GroupRegExp.MatchString(filter.Groups[0])
+		group := filter.Groups[0]
+		isGroup := GroupRegExp.MatchString(group)
 		// if it is not a group name it must be a regexp
 		if !isGroup {
-			if _, err := regexp.Compile(filter.Groups[0]); err != nil || !SpecialRegExp.MatchString(filter.Groups[0]) {
+			if _, err := regexp.Compile(group); err != nil || !SpecialRegExp.MatchString(group) {
 				return fmt.Errorf("invalid rule filter group list is not a proper list or regexp: %v", filter.Groups)
 			}
 		}
@@ -235,20 +309,55 @@ func checkPlacementFilter(filter Filter) error {
 }
 
 // Check a single limit entry
-func checkLimit(limit Limit) error {
+func checkLimit(limit Limit, existedUserName map[string]bool, existedGroupName map[string]bool, queue *QueueConfig) error {
 	if len(limit.Users) == 0 && len(limit.Groups) == 0 {
 		return fmt.Errorf("empty user and group lists defined in limit '%v'", limit)
 	}
+
 	for _, name := range limit.Users {
 		if name != "*" && !UserRegExp.MatchString(name) {
 			return fmt.Errorf("invalid limit user name '%s' in limit definition", name)
 		}
+
+		if existedUserName[name] {
+			return fmt.Errorf("duplicated user name %s , already existed", name)
+		}
+		existedUserName[name] = true
+
+		// The user without wildcard should not happen after the wildcard user
+		// It means the wildcard for user should be the last item for limits object list which including the username,
+		// and we should only set one wildcard user for all limits
+		if existedUserName["*"] && name != "*" {
+			return fmt.Errorf("should not set no wildcard user %s after wildcard user limit", name)
+		}
 	}
 	for _, name := range limit.Groups {
 		if name != "*" && !GroupRegExp.MatchString(name) {
 			return fmt.Errorf("invalid limit group name '%s' in limit definition", name)
 		}
+
+		if existedGroupName[name] {
+			return fmt.Errorf("duplicated group name %s , already existed", name)
+		}
+		existedGroupName[name] = true
+
+		// The group without wildcard should not happen after the wildcard group
+		// It means the wildcard for group should be the last item for limits object list which including the group name,
+		// and we should only set one wildcard group for all limits
+		if existedGroupName["*"] && name != "*" {
+			return fmt.Errorf("should not set no wildcard group %s after wildcard group limit", name)
+		}
 	}
+
+	// Specifying a wildcard for the group limit sets a cumulative limit for all users in that queue.
+	// If there is no specific group mentioned the wildcard group limit would thus be the same as the queue limit.
+	// For that reason we do not allow specifying only one group limit that is using the wildcard.
+	// There must be at least one limit with a group name defined.
+	if existedGroupName["*"] && len(existedGroupName) == 1 {
+		return fmt.Errorf("should not specify only one group limit that is using the wildcard. " +
+			"There must be at least one limit with a group name defined ")
+	}
+
 	var limitResource = resources.NewResource()
 	var err error
 	// check the resource (if defined)
@@ -262,13 +371,37 @@ func checkLimit(limit Limit) error {
 	}
 	// at least some resource should be not null
 	if limit.MaxApplications == 0 && resources.IsZero(limitResource) {
-		return fmt.Errorf("invalid resource combination for limit names '%s' all resource limits are null", limit.Users)
+		return fmt.Errorf("invalid resource combination for limit %s all resource limits are null", limit.Limit)
 	}
+
+	if queue.MaxApplications != 0 {
+		if limit.MaxApplications > queue.MaxApplications {
+			return fmt.Errorf("invalid MaxApplications settings for limit %s exeecd current the queue MaxApplications", limit.Limit)
+		}
+		if limit.MaxApplications == 0 {
+			return fmt.Errorf("MaxApplications is 0 in limit name %s, it should be 1 ~ %d", limit.Limit, queue.MaxApplications)
+		}
+	}
+
+	// If queue is RootQueue, the queue.Resources.Max will be null, we don't need to check for root queue
+	// But we may need to check the root resource during loading the config and after partition resource loading when update node
+	if queue.Name != RootQueue {
+		queueMaxResource, err := resources.NewResourceFromConf(queue.Resources.Max)
+		if err != nil {
+			log.Logger().Debug("resource parsing failed",
+				zap.Error(err))
+			return fmt.Errorf("parse queue %s max resource failed: %s", queue.Name, err.Error())
+		}
+		if !queueMaxResource.FitInMaxUndef(limitResource) {
+			return fmt.Errorf("invalid MaxResources settings for limit %s exeecd current the queue MaxResources", limit.Limit)
+		}
+	}
+
 	return nil
 }
 
 // Check the defined limits list
-func checkLimits(limits []Limit, obj string) error {
+func checkLimits(limits []Limit, obj string, queue *QueueConfig) error {
 	// return if nothing defined
 	if len(limits) == 0 {
 		return nil
@@ -277,8 +410,22 @@ func checkLimits(limits []Limit, obj string) error {
 	log.Logger().Debug("checking limits configs",
 		zap.String("objName", obj),
 		zap.Int("limitsLength", len(limits)))
+
+	existedUserName := make(map[string]bool)
+	existedGroupName := make(map[string]bool)
+
+	defer func() {
+		for k := range existedUserName {
+			delete(existedUserName, k)
+		}
+
+		for k := range existedGroupName {
+			delete(existedGroupName, k)
+		}
+	}()
+
 	for _, limit := range limits {
-		if err := checkLimit(limit); err != nil {
+		if err := checkLimit(limit, existedUserName, existedGroupName, queue); err != nil {
 			return err
 		}
 	}
@@ -321,7 +468,7 @@ func checkQueues(queue *QueueConfig, level int) error {
 	}
 
 	// check the limits for this child (if defined)
-	err = checkLimits(queue.Limits, queue.Name)
+	err = checkLimits(queue.Limits, queue.Name, queue)
 	if err != nil {
 		return err
 	}
@@ -452,7 +599,7 @@ func Validate(newConfig *SchedulerConfig) error {
 		if err != nil {
 			return err
 		}
-		err = checkLimits(partition.Limits, partition.Name)
+		err = checkLimits(partition.Limits, partition.Name, &partition.Queues[0])
 		if err != nil {
 			return err
 		}
@@ -473,3 +620,53 @@ func Validate(newConfig *SchedulerConfig) error {
 	}
 	return nil
 }
+
+// returns the longest fixed queue path defined by the placement rule chain
+// e.g. the chain is user->tag->fixed, returns something like "root.users.<tag>.<user>",
+// the longest static part is "root.users"
+func getLongestPlacementPaths(rules []PlacementRule) []placementStaticPath {
+	paths := make([]placementStaticPath, 0)
+
+	for i, rule := range rules {
+		path, ruleChain, _ := getLongestStaticPath(rule)
+		placementPath := placementStaticPath{
+			path:      path,
+			create:    rule.Create,
+			ruleChain: ruleChain,
+			ruleNo:    i,
+		}
+		paths = append(paths, placementPath)
+	}
+
+	return paths
+}
+
+func getLongestStaticPath(rule PlacementRule) (string, string, bool) {
+	var parentPath string
+	var ruleChain string
+	dynamicParent := false
+
+	if rule.Parent != nil {
+		var chain string
+		parentPath, chain, dynamicParent = getLongestStaticPath(*rule.Parent)
+		ruleChain = rule.Name + "->" + chain
+	} else {
+		ruleChain = rule.Name
+		parentPath = RootQueue
+	}
+
+	if rule.Name == types.Fixed {
+		queueName := strings.ToLower(rule.Value)
+		qualified := strings.HasPrefix(queueName, RootQueue)
+		if qualified {
+			return queueName, ruleChain, false
+		}
+		// there is a parent rule other than "fixed", we can't do anything about that
+		if dynamicParent {
+			return RootQueue, ruleChain, true
+		}
+		return parentPath + "." + queueName, ruleChain, false
+	}
+
+	return parentPath, ruleChain, true
+}
