@@ -25,6 +25,8 @@ import (
 	"sort"
 	"strconv"
 	"strings"
+	"sync"
+	"time"
 
 	"go.uber.org/zap"
 
@@ -44,6 +46,58 @@ func (q Quantity) string() string {
 	return strconv.FormatInt(int64(q), 10)
 }
 
+// Util struct to keep track of application resource usage
+type UsedResource struct {
+	// Two level map for aggregated resource usage
+	// With instance type being the top level key, the mapped value is a map:
+	//   resource type (CPU, memory etc) -> the aggregated used time (in seconds) of the resource type
+	//
+	UsedResourceMap map[string]map[string]int64
+
+	sync.RWMutex
+}
+
+func (ur *UsedResource) Clone() *UsedResource {
+	if ur == nil {
+		return nil
+	}
+	ret := NewUsedResource()
+	ur.RLock()
+	defer ur.RUnlock()
+	for k, v := range ur.UsedResourceMap {
+		dest := make(map[string]int64)
+		for key, element := range v {
+			dest[key] = element
+		}
+		ret.UsedResourceMap[k] = dest
+	}
+	return ret
+}
+
+// Aggregate the resource usage to UsedResourceMap[instType]
+// The time the given resource used is the delta between the resource createTime and currentTime
+func (ur *UsedResource) AggregateUsedResource(instType string,
+	resource *Resource, bindTime time.Time) {
+	ur.Lock()
+	defer ur.Unlock()
+
+	releaseTime := time.Now()
+	timeDiff := int64(releaseTime.Sub(bindTime).Seconds())
+	aggregatedResourceTime, ok := ur.UsedResourceMap[instType]
+	if !ok {
+		aggregatedResourceTime = map[string]int64{}
+	}
+	for key, element := range resource.Resources {
+		curUsage, ok := aggregatedResourceTime[key]
+		if !ok {
+			curUsage = 0
+		}
+		curUsage += int64(element) * timeDiff // resource size times timeDiff
+		aggregatedResourceTime[key] = curUsage
+	}
+	ur.UsedResourceMap[instType] = aggregatedResourceTime
+}
+
 // Never update value of Zero
 var Zero = NewResource()
 
@@ -103,6 +157,10 @@ func NewResourceFromConf(configMap map[string]string) (*Resource, error) {
 	return res, nil
 }
 
+func NewUsedResource() *UsedResource {
+	return &UsedResource{UsedResourceMap: make(map[string]map[string]int64)}
+}
+
 func (r *Resource) String() string {
 	if r == nil {
 		return "nil resource"
@@ -353,7 +411,29 @@ func Sub(left, right *Resource) *Resource {
 	return out
 }
 
-// Subtract resource returning a new resource with the result. A nil resource is considered
+// SubOnlyExisting subtract delta from defined resource.
+// Ignore any type not defined in the base resource (ie receiver).
+// Used as part of the headroom updates as undefined resources are unlimited
+func (r *Resource) SubOnlyExisting(delta *Resource) {
+	// check nil inputs and shortcut
+	if r == nil || delta == nil {
+		return
+	}
+	// neither are nil, subtract the delta
+	for k := range r.Resources {
+		r.Resources[k] = subVal(r.Resources[k], delta.Resources[k])
+	}
+}
+
+// SubEliminateNegative subtracts resource returning a new resource with the result
+// A nil resource is considered an empty resource
+// This will return 0 values for negative values
+func SubEliminateNegative(left, right *Resource) *Resource {
+	res, _ := subNonNegative(left, right)
+	return res
+}
+
+// SubErrorNegative subtracts resource returning a new resource with the result. A nil resource is considered
 // an empty resource. This will return an error if any value in the result is negative.
 // The caller should at least log the error.
 // The returned resource is valid and has all negative values reset to 0
@@ -366,14 +446,6 @@ func SubErrorNegative(left, right *Resource) (*Resource, error) {
 	return res, err
 }
 
-// Subtract resource returning a new resource with the result
-// A nil resource is considered an empty resource
-// This will return 0 values for negative values
-func SubEliminateNegative(left, right *Resource) *Resource {
-	res, _ := subNonNegative(left, right)
-	return res
-}
-
 // Internal subtract resource returning a new resource with the result and an error message when a
 // quantity in the result was less than zero. All negative values are reset to 0.
 func subNonNegative(left, right *Resource) (*Resource, string) {
@@ -870,7 +942,7 @@ func CalculateAbsUsedCapacity(capacity, used *Resource) *Resource {
 		log.Logger().Debug("Cannot calculate absolute capacity because of missing capacity or usage")
 		return absResource
 	}
-	var missingResources strings.Builder
+	missingResources := &strings.Builder{}
 	for resourceName, availableResource := range capacity.Resources {
 		var absResValue int64
 		if usedResource, ok := used.Resources[resourceName]; ok {
@@ -909,7 +981,7 @@ func CalculateAbsUsedCapacity(capacity, used *Resource) *Resource {
 	}
 	if missingResources.Len() != 0 {
 		log.Logger().Debug("Absolute usage result is missing resource information",
-			zap.String("missing resource(s)", missingResources.String()))
+			zap.Stringer("missing resource(s)", missingResources))
 	}
 	return absResource
 }
