@@ -57,12 +57,12 @@ func enqueueAndCheckFull(queue chan interface{}, ev interface{}) {
 	select {
 	case queue <- ev:
 		log.Logger().Debug("enqueue event",
-			zap.String("eventType", reflect.TypeOf(ev).String()),
+			zap.Stringer("eventType", reflect.TypeOf(ev)),
 			zap.Any("event", ev),
 			zap.Int("currentQueueSize", len(queue)))
 	default:
 		log.Logger().DPanic("failed to enqueue event",
-			zap.String("event", reflect.TypeOf(ev).String()))
+			zap.Stringer("event", reflect.TypeOf(ev)))
 	}
 }
 
