@@ -264,7 +264,7 @@ func (cc *ClusterContext) processNodes(request *si.NodeRequest) {
 			}
 			switch nodeInfo.Action {
 			case si.NodeInfo_CREATE:
-				err := cc.addNode(nodeInfo, nodeCount)
+				err := cc.addNode(nodeInfo)
 				if err == nil {
 					acceptedNodes = append(acceptedNodes, &si.AcceptedNode{
 						NodeID: nodeInfo.NodeID,
@@ -602,7 +602,7 @@ func (cc *ClusterContext) removePartition(partitionName string) {
 
 // addNode adds a new node to the cluster enforcing just one unlimited node in the cluster.
 // nil nodeInfo objects must be filtered out before calling this function
-func (cc *ClusterContext) addNode(nodeInfo *si.NodeInfo, nodeCount int) error {
+func (cc *ClusterContext) addNode(nodeInfo *si.NodeInfo) error {
 	sn := objects.NewNode(nodeInfo)
 	if !sn.IsReady() {
 		metrics.GetSchedulerMetrics().IncUnhealthyNodes()
@@ -619,7 +619,7 @@ func (cc *ClusterContext) addNode(nodeInfo *si.NodeInfo, nodeCount int) error {
 		return err
 	}
 
-	existingAllocations := cc.convertAllocations(nodeInfo.ExistingAllocations)
+	existingAllocations := cc.convertAllocations(sn, nodeInfo.ExistingAllocations)
 	err := partition.AddNode(sn, existingAllocations)
 	if err != nil {
 		wrapped := fmt.Errorf("failure while adding new node, node rejected with error: %w", err)
@@ -644,7 +644,7 @@ func (cc *ClusterContext) updateNode(nodeInfo *si.NodeInfo) {
 	} else {
 		log.Logger().Error("node partition not specified",
 			zap.String("nodeID", nodeInfo.NodeID),
-			zap.String("nodeAction", nodeInfo.Action.String()))
+			zap.Stringer("nodeAction", nodeInfo.Action))
 		return
 	}
 
@@ -652,7 +652,7 @@ func (cc *ClusterContext) updateNode(nodeInfo *si.NodeInfo) {
 		log.Logger().Error("Failed to update node on non existing partition",
 			zap.String("nodeID", nodeInfo.NodeID),
 			zap.String("partitionName", nodeInfo.Attributes[siCommon.NodePartition]),
-			zap.String("nodeAction", nodeInfo.Action.String()))
+			zap.Stringer("nodeAction", nodeInfo.Action))
 		return
 	}
 
@@ -661,7 +661,7 @@ func (cc *ClusterContext) updateNode(nodeInfo *si.NodeInfo) {
 		log.Logger().Error("Failed to update non existing node",
 			zap.String("nodeID", nodeInfo.NodeID),
 			zap.String("partitionName", nodeInfo.Attributes[siCommon.NodePartition]),
-			zap.String("nodeAction", nodeInfo.Action.String()))
+			zap.Stringer("nodeAction", nodeInfo.Action))
 		return
 	}
 
@@ -725,7 +725,7 @@ func (cc *ClusterContext) updateNode(nodeInfo *si.NodeInfo) {
 		log.Logger().Debug("unknown action for node update",
 			zap.String("nodeID", nodeInfo.NodeID),
 			zap.String("partitionName", nodeInfo.Attributes[siCommon.NodePartition]),
-			zap.String("nodeAction", nodeInfo.Action.String()))
+			zap.Stringer("nodeAction", nodeInfo.Action))
 	}
 }
 
@@ -827,10 +827,11 @@ func (cc *ClusterContext) processAllocationReleases(releases []*si.AllocationRel
 }
 
 // Convert the si allocation to a proposal to add to the node
-func (cc *ClusterContext) convertAllocations(allocations []*si.Allocation) []*objects.Allocation {
+func (cc *ClusterContext) convertAllocations(node *objects.Node, allocations []*si.Allocation) []*objects.Allocation {
+	instType := node.GetInstanceType()
 	convert := make([]*objects.Allocation, len(allocations))
 	for current, allocation := range allocations {
-		convert[current] = objects.NewAllocationFromSI(allocation)
+		convert[current] = objects.NewAllocationFromSI(allocation, instType)
 	}
 
 	return convert
