@@ -19,6 +19,7 @@
 package objects
 
 import (
+	"context"
 	"fmt"
 	"math"
 	"strings"
@@ -74,15 +75,18 @@ type Application struct {
 	SubmissionTime time.Time
 
 	// Private fields need protection
-	queuePath            string
-	queue                *Queue                    // queue the application is running in
-	pending              *resources.Resource       // pending resources from asks for the app
-	reservations         map[string]*reservation   // a map of reservations
-	requests             map[string]*AllocationAsk // a map of asks
-	sortedRequests       []*AllocationAsk
-	user                 security.UserGroup          // owner of the application
-	tags                 map[string]string           // application tags used in scheduling
-	allocatedResource    *resources.Resource         // total allocated resources
+	queuePath         string
+	queue             *Queue                    // queue the application is running in
+	pending           *resources.Resource       // pending resources from asks for the app
+	reservations      map[string]*reservation   // a map of reservations
+	requests          map[string]*AllocationAsk // a map of asks
+	sortedRequests    []*AllocationAsk
+	user              security.UserGroup  // owner of the application
+	tags              map[string]string   // application tags used in scheduling
+	allocatedResource *resources.Resource // total allocated resources
+
+	usedResource *resources.UsedResource // keep track of resource usage of the application
+
 	maxAllocatedResource *resources.Resource         // max allocated resources
 	allocatedPlaceholder *resources.Resource         // total allocated placeholder resources
 	allocations          map[string]*Allocation      // list of all allocations
@@ -93,12 +97,12 @@ type Application struct {
 	execTimeout          time.Duration               // execTimeout for the application run
 	placeholderTimer     *time.Timer                 // placeholder replace timer
 	gangSchedulingStyle  string                      // gang scheduling style can be hard (after timeout we fail the application), or soft (after timeeout we schedule it as a normal application)
+	startTime            time.Time                   // the time that the application starts running. Default is zero.
 	finishedTime         time.Time                   // the time of finishing this application. the default value is zero time
 	rejectedMessage      string                      // If the application is rejected, save the rejected message
 	stateLog             []*StateLogEntry            // state log for this application
 	placeholderData      map[string]*PlaceholderData // track placeholder and gang related info
 	askMaxPriority       int32                       // highest priority value of outstanding asks
-	allocMinPriority     int32                       // lowest priority value of allocations
 
 	rmEventHandler     handler.EventHandler
 	rmID               string
@@ -107,6 +111,50 @@ type Application struct {
 	sync.RWMutex
 }
 
+type ApplicationSummary struct {
+	ApplicationID  string
+	SubmissionTime time.Time
+	StartTime      time.Time
+	FinishTime     time.Time
+	User           string
+	Queue          string
+	State          string
+	RmID           string
+	ResourceUsage  *resources.UsedResource
+}
+
+func (as *ApplicationSummary) DoLogging() {
+	log.Logger().Info("YK_APP_SUMMARY:",
+		zap.String("appID", as.ApplicationID),
+		zap.Int64("submissionTime", as.SubmissionTime.UnixMilli()),
+		zap.Int64("startTime", as.StartTime.UnixMilli()),
+		zap.Int64("finishTime", as.FinishTime.UnixMilli()),
+		zap.String("user", as.User),
+		zap.String("queue", as.Queue),
+		zap.String("state", as.State),
+		zap.String("rmID", as.RmID),
+		zap.Any("resourceUsage", as.ResourceUsage.UsedResourceMap))
+}
+
+func (sa *Application) GetApplicationSummary(rmID string) *ApplicationSummary {
+	state := sa.stateMachine.Current()
+	ru := sa.usedResource.Clone()
+	sa.RLock()
+	defer sa.RUnlock()
+	appSummary := &ApplicationSummary{
+		ApplicationID:  sa.ApplicationID,
+		SubmissionTime: sa.SubmissionTime,
+		StartTime:      sa.startTime,
+		FinishTime:     sa.finishedTime,
+		User:           sa.user.User,
+		Queue:          sa.queuePath,
+		State:          state,
+		RmID:           rmID,
+		ResourceUsage:  ru,
+	}
+	return appSummary
+}
+
 func NewApplication(siApp *si.AddApplicationRequest, ugi security.UserGroup, eventHandler handler.EventHandler, rmID string) *Application {
 	app := &Application{
 		ApplicationID:        siApp.ApplicationID,
@@ -116,6 +164,7 @@ func NewApplication(siApp *si.AddApplicationRequest, ugi security.UserGroup, eve
 		tags:                 siApp.Tags,
 		pending:              resources.NewResource(),
 		allocatedResource:    resources.NewResource(),
+		usedResource:         resources.NewUsedResource(),
 		maxAllocatedResource: resources.NewResource(),
 		allocatedPlaceholder: resources.NewResource(),
 		requests:             make(map[string]*AllocationAsk),
@@ -123,11 +172,11 @@ func NewApplication(siApp *si.AddApplicationRequest, ugi security.UserGroup, eve
 		allocations:          make(map[string]*Allocation),
 		stateMachine:         NewAppState(),
 		placeholderAsk:       resources.NewResourceFromProto(siApp.PlaceholderAsk),
+		startTime:            time.Time{},
 		finishedTime:         time.Time{},
 		rejectedMessage:      "",
 		stateLog:             make([]*StateLogEntry, 0),
 		askMaxPriority:       configs.MinPriority,
-		allocMinPriority:     configs.MaxPriority,
 	}
 	placeholderTimeout := common.ConvertSITimeoutWithAdjustment(siApp, defaultPlaceholderTimeout)
 	gangSchedStyle := siApp.GetGangSchedulingStyle()
@@ -236,7 +285,7 @@ func (sa *Application) IsResuming() bool {
 // HandleApplicationEvent handles the state event for the application.
 // The application lock is expected to be held.
 func (sa *Application) HandleApplicationEvent(event applicationEvent) error {
-	err := sa.stateMachine.Event(event.String(), sa)
+	err := sa.stateMachine.Event(context.Background(), event.String(), sa)
 	// handle the same state transition not nil error (limit of fsm).
 	if err != nil && err.Error() == noTransition {
 		return nil
@@ -247,7 +296,7 @@ func (sa *Application) HandleApplicationEvent(event applicationEvent) error {
 // HandleApplicationEventWithInfo handles the state event for the application with associated info object.
 // The application lock is expected to be held.
 func (sa *Application) HandleApplicationEventWithInfo(event applicationEvent, eventInfo string) error {
-	err := sa.stateMachine.Event(event.String(), sa, eventInfo)
+	err := sa.stateMachine.Event(context.Background(), event.String(), sa, eventInfo)
 	// handle the same state transition not nil error (limit of fsm).
 	if err != nil && err.Error() == noTransition {
 		return nil
@@ -515,6 +564,7 @@ func (sa *Application) removeAsksInternal(allocKey string) int {
 		deltaPendingResource = sa.pending
 		sa.pending = resources.NewResource()
 		sa.requests = make(map[string]*AllocationAsk)
+		sa.sortedRequests = nil
 		sa.askMaxPriority = configs.MinPriority
 		sa.queue.UpdateApplicationPriority(sa.ApplicationID, sa.askMaxPriority)
 	} else {
@@ -537,6 +587,7 @@ func (sa *Application) removeAsksInternal(allocKey string) int {
 			deltaPendingResource = resources.MultiplyBy(ask.GetAllocatedResource(), float64(ask.GetPendingAskRepeat()))
 			sa.pending = resources.Sub(sa.pending, deltaPendingResource)
 			delete(sa.requests, allocKey)
+			sa.sortRequests()
 			if priority := ask.GetPriority(); priority >= sa.askMaxPriority {
 				sa.updateAskMaxPriority()
 			}
@@ -561,7 +612,7 @@ func (sa *Application) removeAsksInternal(allocKey string) int {
 	log.Logger().Info("ask removed successfully from application",
 		zap.String("appID", sa.ApplicationID),
 		zap.String("ask", allocKey),
-		zap.String("pendingDelta", deltaPendingResource.String()))
+		zap.Stringer("pendingDelta", deltaPendingResource))
 
 	return toRelease
 }
@@ -618,7 +669,9 @@ func (sa *Application) AddAllocationAsk(ask *AllocationAsk) error {
 		zap.String("appID", sa.ApplicationID),
 		zap.String("ask", ask.GetAllocationKey()),
 		zap.Bool("placeholder", ask.IsPlaceholder()),
-		zap.String("pendingDelta", delta.String()))
+		zap.Stringer("pendingDelta", delta))
+
+	sa.sortRequests()
 
 	return nil
 }
@@ -655,6 +708,7 @@ func (sa *Application) UpdateAskRepeat(allocKey string, delta int32) (*resources
 	sa.Lock()
 	defer sa.Unlock()
 	if ask := sa.requests[allocKey]; ask != nil {
+
 		return sa.updateAskRepeatInternal(ask, delta)
 	}
 	return nil, fmt.Errorf("failed to locate ask with key %s", allocKey)
@@ -666,6 +720,11 @@ func (sa *Application) updateAskRepeatInternal(ask *AllocationAsk, delta int32)
 		return nil, fmt.Errorf("ask repaeat not updated resulting repeat less than zero for ask %s on app %s", ask.GetAllocationKey(), sa.ApplicationID)
 	}
 
+	if delta == 1 {
+		// adding request back, which was filtered out before
+		sa.sortRequests()
+	}
+
 	askPriority := ask.GetPriority()
 	if ask.GetPendingAskRepeat() == 0 {
 		// ask removed
@@ -775,8 +834,8 @@ func (sa *Application) unReserveInternal(node *Node, ask *AllocationAsk) (int, e
 	if resKey == "" {
 		log.Logger().Debug("unreserve reservation key create failed unexpectedly",
 			zap.String("appID", sa.ApplicationID),
-			zap.String("node", node.String()),
-			zap.String("ask", ask.String()))
+			zap.Stringer("node", node),
+			zap.Stringer("ask", ask))
 		return 0, fmt.Errorf("reservation key failed node or ask are nil for appID %s", sa.ApplicationID)
 	}
 	// unReserve the node before removing from the app
@@ -843,7 +902,7 @@ func (sa *Application) canAskReserve(ask *AllocationAsk) bool {
 // Sort the request for the app in order based on the priority of the request.
 // The sorted list only contains candidates that have an outstanding repeat.
 // No locking must be called while holding the lock
-func (sa *Application) sortRequests(ascending bool) {
+func (sa *Application) sortRequests() {
 	sa.sortedRequests = nil
 	for _, request := range sa.requests {
 		if request.GetPendingAskRepeat() == 0 {
@@ -853,25 +912,26 @@ func (sa *Application) sortRequests(ascending bool) {
 	}
 	// we might not have any requests
 	if len(sa.sortedRequests) > 0 {
-		sortAskByPriority(sa.sortedRequests, ascending)
+		sortAskByPriority(sa.sortedRequests)
 	}
 }
 
 func (sa *Application) getOutstandingRequests(headRoom *resources.Resource, total *[]*AllocationAsk) {
-	// make sure the request are sorted
-	sa.Lock()
-	sa.sortRequests(false)
-	sa.Unlock()
-
 	sa.RLock()
 	defer sa.RUnlock()
+	if sa.sortedRequests == nil {
+		return
+	}
 
 	for _, request := range sa.sortedRequests {
+		if request.GetPendingAskRepeat() == 0 {
+			continue
+		}
 		// ignore nil checks resource function calls are nil safe
 		if headRoom.FitInMaxUndef(request.GetAllocatedResource()) {
 			// if headroom is still enough for the resources
 			*total = append(*total, request)
-			headRoom.SubFrom(request.GetAllocatedResource())
+			headRoom.SubOnlyExisting(request.GetAllocatedResource())
 		}
 	}
 }
@@ -891,19 +951,36 @@ func (sa *Application) canReplace(request *AllocationAsk) bool {
 }
 
 // tryAllocate will perform a regular allocation of a pending request, includes placeholders.
-func (sa *Application) tryAllocate(headRoom *resources.Resource, nodeIterator func() NodeIterator, getNodeFn func(string) *Node) *Allocation {
+func (sa *Application) tryAllocate(headRoom *resources.Resource, preemptionDelay time.Duration, preemptAttemptsRemaining *int, nodeIterator func() NodeIterator, fullNodeIterator func() NodeIterator, getNodeFn func(string) *Node) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
-	// make sure the request are sorted
-	sa.sortRequests(false)
+	if sa.sortedRequests == nil {
+		return nil
+	}
 	// get all the requests from the app sorted in order
 	for _, request := range sa.sortedRequests {
+		if request.GetPendingAskRepeat() == 0 {
+			continue
+		}
 		// check if there is a replacement possible
 		if sa.canReplace(request) {
 			continue
 		}
-		// resource must fit in headroom otherwise skip the request
+
+		// resource must fit in headroom otherwise skip the request (unless preemption could help)
 		if !headRoom.FitInMaxUndef(request.GetAllocatedResource()) {
+			// attempt preemption
+			if *preemptAttemptsRemaining > 0 {
+				*preemptAttemptsRemaining--
+				fullIterator := fullNodeIterator()
+				if fullIterator != nil {
+					if alloc, ok := sa.tryPreemption(headRoom, preemptionDelay, request, fullIterator, false); ok {
+						// preemption occurred, and possibly reservation
+						return alloc
+					}
+				}
+			}
+
 			// post scheduling events via the event plugin
 			if eventCache := events.GetEventCache(); eventCache != nil {
 				message := fmt.Sprintf("Application %s does not fit into %s queue", request.GetApplicationID(), sa.queuePath)
@@ -952,19 +1029,30 @@ func (sa *Application) tryAllocate(headRoom *resources.Resource, nodeIterator fu
 				log.Logger().Debug("allocation on required node is completed",
 					zap.String("nodeID", node.NodeID),
 					zap.String("allocationKey", request.GetAllocationKey()),
-					zap.String("AllocationResult", alloc.GetResult().String()))
+					zap.Stringer("AllocationResult", alloc.GetResult()))
 				return alloc
 			}
-			return newReservedAllocation(Reserved, node.NodeID, request)
+			return newReservedAllocation(Reserved, node.NodeID, node.GetInstanceType(), request)
 		}
 
 		iterator := nodeIterator()
 		if iterator != nil {
-			alloc := sa.tryNodes(request, iterator)
-			// have a candidate return it
-			if alloc != nil {
+			if alloc := sa.tryNodes(request, iterator); alloc != nil {
+				// have a candidate return it
 				return alloc
 			}
+
+			// no nodes qualify, attempt preemption
+			if *preemptAttemptsRemaining > 0 {
+				*preemptAttemptsRemaining--
+				fullIterator := fullNodeIterator()
+				if fullIterator != nil {
+					if alloc, ok := sa.tryPreemption(headRoom, preemptionDelay, request, fullIterator, true); ok {
+						// preemption occurred, and possibly reservation
+						return alloc
+					}
+				}
+			}
 		}
 	}
 	// no requests fit, skip to next app
@@ -1018,16 +1106,15 @@ func (sa *Application) cancelReservations(reservations []*reservation) bool {
 }
 
 // tryPlaceholderAllocate tries to replace a placeholder that is allocated with a real allocation
+//
 //nolint:funlen
 func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator, getNodeFn func(string) *Node) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
 	// nothing to do if we have no placeholders allocated
-	if resources.IsZero(sa.allocatedPlaceholder) {
+	if resources.IsZero(sa.allocatedPlaceholder) || sa.sortedRequests == nil {
 		return nil
 	}
-	// make sure the request are sorted
-	sa.sortRequests(false)
 	// keep the first fits for later
 	var phFit *Allocation
 	var reqFit *AllocationAsk
@@ -1035,7 +1122,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 	for _, request := range sa.sortedRequests {
 		// skip placeholders they follow standard allocation
 		// this should also be part of a task group just make sure it is
-		if request.IsPlaceholder() || request.GetTaskGroup() == "" {
+		if request.IsPlaceholder() || request.GetTaskGroup() == "" || request.GetPendingAskRepeat() == 0 {
 			continue
 		}
 		// walk over the placeholders, allow for processing all as we can have multiple task groups
@@ -1054,9 +1141,9 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 			// All placeholders in the same task group are always the same size.
 			if delta.HasNegativeValue() {
 				log.Logger().Warn("releasing placeholder: real allocation is larger than placeholder",
-					zap.String("requested resource", request.GetAllocatedResource().String()),
+					zap.Stringer("requested resource", request.GetAllocatedResource()),
 					zap.String("placeholderID", ph.GetUUID()),
-					zap.String("placeholder resource", ph.GetAllocatedResource().String()))
+					zap.Stringer("placeholder resource", ph.GetAllocatedResource()))
 				// release the placeholder and tell the RM
 				ph.SetReleased(true)
 				sa.notifyRMAllocationReleased(sa.rmID, []*Allocation{ph}, si.TerminationType_TIMEOUT, "cancel placeholder: resource incompatible")
@@ -1083,7 +1170,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 			// got the node run same checks as for reservation (all but fits)
 			// resource usage should not change anyway between placeholder and real one at this point
 			if node != nil && node.preReserveConditions(request) {
-				alloc := NewAllocation(common.GetNewUUID(), node.NodeID, request)
+				alloc := NewAllocation(common.GetNewUUID(), node.NodeID, node.GetInstanceType(), request)
 				// double link to make it easier to find
 				// alloc (the real one) releases points to the placeholder in the releases list
 				alloc.SetRelease(ph)
@@ -1121,7 +1208,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 					zap.String("node", node.NodeID))
 				continue
 			}
-			if err := node.preAllocateCheck(reqFit.GetAllocatedResource(), reservationKey(nil, sa, reqFit)); err != nil {
+			if !node.preAllocateCheck(reqFit.GetAllocatedResource(), reservationKey(nil, sa, reqFit)) {
 				continue
 			}
 			// skip the node if conditions can not be satisfied
@@ -1129,7 +1216,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 				continue
 			}
 			// allocation worked: on a non placeholder node update result and return
-			alloc := NewAllocation(common.GetNewUUID(), node.NodeID, reqFit)
+			alloc := NewAllocation(common.GetNewUUID(), node.NodeID, node.GetInstanceType(), reqFit)
 			// double link to make it easier to find
 			// alloc (the real one) releases points to the placeholder in the releases list
 			alloc.SetRelease(phFit)
@@ -1143,8 +1230,8 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 			if !node.AddAllocation(alloc) {
 				log.Logger().Debug("Node update failed unexpectedly",
 					zap.String("applicationID", sa.ApplicationID),
-					zap.String("ask", reqFit.String()),
-					zap.String("placeholder", phFit.String()))
+					zap.Stringer("ask", reqFit),
+					zap.Stringer("placeholder", phFit))
 				return nil
 			}
 			_, err := sa.updateAskRepeatInternal(reqFit, -1)
@@ -1180,7 +1267,7 @@ func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIte
 				unreserveAsk = ask
 			}
 			// remove the reservation as this should not be reserved
-			alloc := newReservedAllocation(Unreserved, reserve.nodeID, unreserveAsk)
+			alloc := newReservedAllocation(Unreserved, reserve.nodeID, reserve.node.GetInstanceType(), unreserveAsk)
 			return alloc
 		}
 
@@ -1192,7 +1279,7 @@ func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIte
 		// Do we need a specific node?
 		if ask.GetRequiredNode() != "" {
 			if !reserve.node.CanAllocate(ask.GetAllocatedResource()) && !ask.HasTriggeredPreemption() {
-				sa.tryPreemption(reserve, ask)
+				sa.tryRequiredNodePreemption(reserve, ask)
 				continue
 			}
 		}
@@ -1224,7 +1311,23 @@ func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIte
 	return nil
 }
 
-func (sa *Application) tryPreemption(reserve *reservation, ask *AllocationAsk) bool {
+func (sa *Application) tryPreemption(headRoom *resources.Resource, preemptionDelay time.Duration, ask *AllocationAsk, iterator NodeIterator, nodesTried bool) (*Allocation, bool) {
+	preemptor := NewPreemptor(sa, headRoom, preemptionDelay, ask, iterator, nodesTried)
+
+	// validate prerequisites for preemption of an ask and mark ask for preemption if successful
+	if !preemptor.CheckPreconditions() {
+		return nil, false
+	}
+
+	// track time spent trying preemption
+	tryPreemptionStart := time.Now()
+	defer metrics.GetSchedulerMetrics().ObserveTryPreemptionLatency(tryPreemptionStart)
+
+	// attempt preemption
+	return preemptor.TryPreemption()
+}
+
+func (sa *Application) tryRequiredNodePreemption(reserve *reservation, ask *AllocationAsk) bool {
 	log.Logger().Info("Triggering preemption process for daemon set ask",
 		zap.String("ds allocation key", ask.GetAllocationKey()))
 
@@ -1240,6 +1343,9 @@ func (sa *Application) tryPreemption(reserve *reservation, ask *AllocationAsk) b
 			zap.String("ds allocation key", ask.GetAllocationKey()),
 			zap.Int("no.of victims", len(victims)))
 		for _, victim := range victims {
+			if victimQueue := sa.queue.FindQueueByAppID(victim.GetApplicationID()); victimQueue != nil {
+				victimQueue.IncPreemptingResource(victim.GetAllocatedResource())
+			}
 			victim.MarkPreempted()
 		}
 		ask.MarkTriggeredPreemption()
@@ -1350,7 +1456,7 @@ func (sa *Application) tryNodes(ask *AllocationAsk, iterator NodeIterator) *Allo
 				zap.Time("createTime", ask.GetCreateTime()),
 				zap.Duration("askAge", askAge),
 				zap.Duration("reservationDelay", reservationDelay))
-			score := ask.GetAllocatedResource().FitInScore(node.GetAvailableResource())
+			score := node.GetFitInScoreForAvailableResource(ask.GetAllocatedResource())
 			// Record the so-far best node to reserve
 			if score < scoreReserved {
 				scoreReserved = score
@@ -1372,7 +1478,7 @@ func (sa *Application) tryNodes(ask *AllocationAsk, iterator NodeIterator) *Allo
 			return nil
 		}
 		// return reservation allocation and mark it as a reservation
-		alloc := newReservedAllocation(Reserved, nodeToReserve.NodeID, ask)
+		alloc := newReservedAllocation(Reserved, nodeToReserve.NodeID, nodeToReserve.GetInstanceType(), ask)
 		return alloc
 	}
 	// ask does not fit, skip to next ask
@@ -1383,7 +1489,7 @@ func (sa *Application) tryNodes(ask *AllocationAsk, iterator NodeIterator) *Allo
 func (sa *Application) tryNode(node *Node, ask *AllocationAsk) *Allocation {
 	toAllocate := ask.GetAllocatedResource()
 	// create the key for the reservation
-	if err := node.preAllocateCheck(toAllocate, reservationKey(nil, sa, ask)); err != nil {
+	if !node.preAllocateCheck(toAllocate, reservationKey(nil, sa, ask)) {
 		// skip schedule onto node
 		return nil
 	}
@@ -1392,7 +1498,7 @@ func (sa *Application) tryNode(node *Node, ask *AllocationAsk) *Allocation {
 		return nil
 	}
 	// everything OK really allocate
-	alloc := NewAllocation(common.GetNewUUID(), node.NodeID, ask)
+	alloc := NewAllocation(common.GetNewUUID(), node.NodeID, node.GetInstanceType(), ask)
 	if node.AddAllocation(alloc) {
 		if err := sa.queue.IncAllocatedResource(alloc.GetAllocatedResource(), false); err != nil {
 			log.Logger().Warn("queue update failed unexpectedly",
@@ -1455,6 +1561,12 @@ func (sa *Application) UnSetQueue() {
 	sa.finishedTime = time.Now()
 }
 
+func (sa *Application) StartTime() time.Time {
+	sa.RLock()
+	defer sa.RUnlock()
+	return sa.startTime
+}
+
 func (sa *Application) FinishedTime() time.Time {
 	sa.RLock()
 	defer sa.RUnlock()
@@ -1552,11 +1664,6 @@ func (sa *Application) addAllocationInternal(info *Allocation) {
 		sa.allocatedResource = resources.Add(sa.allocatedResource, info.GetAllocatedResource())
 		sa.maxAllocatedResource = resources.ComponentWiseMax(sa.allocatedResource, sa.maxAllocatedResource)
 	}
-	priority := info.GetPriority()
-	if priority < sa.allocMinPriority {
-		sa.allocMinPriority = priority
-		sa.queue.UpdateApplicationPreemptionPriority(sa.ApplicationID, priority)
-	}
 	sa.allocations[info.GetUUID()] = info
 }
 
@@ -1584,6 +1691,13 @@ func (sa *Application) decUserResourceUsage(resource *resources.Resource, remove
 	}
 }
 
+// When the resource allocated with this allocation is to be removed,
+// have the usedResource to aggregate the resource used by this allocation
+func (sa *Application) updateUsedResource(info *Allocation) {
+	sa.usedResource.AggregateUsedResource(info.GetInstanceType(),
+		info.GetAllocatedResource(), info.GetBindTime())
+}
+
 func (sa *Application) ReplaceAllocation(uuid string) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
@@ -1593,7 +1707,7 @@ func (sa *Application) ReplaceAllocation(uuid string) *Allocation {
 	if ph == nil || ph.GetReleaseCount() == 0 {
 		log.Logger().Debug("Unexpected placeholder released",
 			zap.String("applicationID", sa.ApplicationID),
-			zap.String("placeholder", ph.String()))
+			zap.Stringer("placeholder", ph))
 		return nil
 	}
 	// weird issue we should never have more than 1 log it for debugging this error
@@ -1610,6 +1724,7 @@ func (sa *Application) ReplaceAllocation(uuid string) *Allocation {
 	alloc.SetPlaceholderUsed(true)
 	alloc.SetPlaceholderCreateTime(ph.GetCreateTime())
 	alloc.SetCreateTime(time.Now())
+	alloc.SetBindTime(time.Now())
 	sa.addAllocationInternal(alloc)
 	// order is important: clean up the allocation after adding it to the app
 	// we need the original Replaced allocation result.
@@ -1653,6 +1768,7 @@ func (sa *Application) removeAllocationInternal(uuid string, releaseType si.Term
 		// as and when every ph gets removed (for replacement), resource usage would be reduced.
 		// When real allocation happens as part of replacement, usage would be increased again with real alloc resource
 		sa.allocatedPlaceholder = resources.Sub(sa.allocatedPlaceholder, alloc.GetAllocatedResource())
+
 		// if all the placeholders are replaced, clear the placeholder timer
 		if resources.IsZero(sa.allocatedPlaceholder) {
 			sa.clearPlaceholderTimer()
@@ -1672,6 +1788,10 @@ func (sa *Application) removeAllocationInternal(uuid string, releaseType si.Term
 		sa.decUserResourceUsage(alloc.GetAllocatedResource(), removeApp)
 	} else {
 		sa.allocatedResource = resources.Sub(sa.allocatedResource, alloc.GetAllocatedResource())
+
+		// Aggregate the resources used by this alloc to the application's user resource tracker
+		sa.updateUsedResource(alloc)
+
 		// When the resource trackers are zero we should not expect anything to come in later.
 		if sa.hasZeroAllocations() {
 			removeApp = true
@@ -1684,14 +1804,11 @@ func (sa *Application) removeAllocationInternal(uuid string, releaseType si.Term
 		if err := sa.HandleApplicationEvent(event); err != nil {
 			log.Logger().Warn(eventWarning,
 				zap.String("currentState", sa.CurrentState()),
-				zap.String("event", event.String()),
+				zap.Stringer("event", event),
 				zap.Error(err))
 		}
 	}
 	delete(sa.allocations, uuid)
-	if sa.allocMinPriority == alloc.GetPriority() {
-		sa.updateAllocationMinPriority()
-	}
 	return alloc
 }
 
@@ -1710,18 +1827,6 @@ func (sa *Application) updateAskMaxPriority() {
 	sa.queue.UpdateApplicationPriority(sa.ApplicationID, value)
 }
 
-func (sa *Application) updateAllocationMinPriority() {
-	value := configs.MaxPriority
-	for _, v := range sa.allocations {
-		p := v.GetPriority()
-		if p < value {
-			value = p
-		}
-	}
-	sa.allocMinPriority = value
-	sa.queue.UpdateApplicationPreemptionPriority(sa.ApplicationID, value)
-}
-
 func (sa *Application) hasZeroAllocations() bool {
 	return resources.IsZero(sa.pending) && resources.IsZero(sa.allocatedResource)
 }
@@ -1735,7 +1840,10 @@ func (sa *Application) RemoveAllAllocations() []*Allocation {
 	allocationsToRelease := make([]*Allocation, 0)
 	for _, alloc := range sa.allocations {
 		allocationsToRelease = append(allocationsToRelease, alloc)
+		// Aggregate the resources used by this alloc to the application's user resource tracker
+		sa.updateUsedResource(alloc)
 	}
+
 	if resources.IsZero(sa.pending) {
 		sa.decUserResourceUsage(resources.Add(sa.allocatedResource, sa.allocatedPlaceholder), true)
 	}
@@ -1903,12 +2011,6 @@ func (sa *Application) GetAllPlaceholderData() []*PlaceholderData {
 	return placeholders
 }
 
-func (sa *Application) GetAllocationMinPriority() int32 {
-	sa.RLock()
-	defer sa.RUnlock()
-	return sa.allocMinPriority
-}
-
 func (sa *Application) GetAskMaxPriority() int32 {
 	sa.RLock()
 	defer sa.RUnlock()
@@ -1920,6 +2022,16 @@ func (sa *Application) cleanupAsks() {
 	sa.sortedRequests = nil
 }
 
+func (sa *Application) CleanupUsedResource() {
+	sa.usedResource = nil
+}
+
+func (sa *Application) LogAppSummary(rmID string) {
+	appSummary := sa.GetApplicationSummary(rmID)
+	appSummary.DoLogging()
+	appSummary.ResourceUsage = nil
+}
+
 // test only
 func SetCompletingTimeout(duration time.Duration) {
 	completingTimeout = duration
