@@ -33,6 +33,10 @@ import (
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
+const (
+	UnknownInstanceType = "UNKNOWN"
+)
+
 type Node struct {
 	// Fields for fast access These fields are considered read only.
 	// Values should only be set when creating a new node and never changed.
@@ -117,6 +121,20 @@ func (sn *Node) GetAttribute(key string) string {
 	return sn.attributes[key]
 }
 
+func (sn *Node) GetAttributes() map[string]string {
+	return sn.attributes
+}
+
+// Get InstanceType of this node.
+// This is a lock free call because all attributes are considered read only
+func (sn *Node) GetInstanceType() string {
+	itype := sn.GetAttribute(common.InstanceType)
+	if itype != "" {
+		return itype
+	}
+	return UnknownInstanceType
+}
+
 // GetReservationKeys Return an array of all reservation keys for the node.
 // This will return an empty array if there are no reservations.
 // Visible for tests
@@ -177,10 +195,10 @@ func (sn *Node) refreshAvailableResource() {
 	// check if any quantity is negative: a nil resource is all 0's
 	if !resources.StrictlyGreaterThanOrEquals(sn.availableResource, nil) {
 		log.Logger().Warn("Node update triggered over allocated node",
-			zap.String("available", sn.availableResource.String()),
-			zap.String("total", sn.totalResource.String()),
-			zap.String("occupied", sn.occupiedResource.String()),
-			zap.String("allocated", sn.allocatedResource.String()))
+			zap.Stringer("available", sn.availableResource),
+			zap.Stringer("total", sn.totalResource),
+			zap.Stringer("occupied", sn.occupiedResource),
+			zap.Stringer("allocated", sn.allocatedResource))
 	}
 }
 
@@ -237,6 +255,14 @@ func (sn *Node) GetAvailableResource() *resources.Resource {
 	return sn.availableResource.Clone()
 }
 
+// GetFitInScoreForAvailableResource calculates a fit in score for "res" based on the current
+// available resources, avoiding cloning. The caller must ensure that "res" cannot change while this method is running.
+func (sn *Node) GetFitInScoreForAvailableResource(res *resources.Resource) float64 {
+	sn.RLock()
+	defer sn.RUnlock()
+	return res.FitInScore(sn.availableResource)
+}
+
 // Get the utilized resource on this node.
 func (sn *Node) GetUtilizedResource() *resources.Resource {
 	total := sn.GetCapacity()
@@ -311,12 +337,14 @@ func (sn *Node) ReplaceAllocation(uuid string, replace *Allocation, delta *resou
 	replace.SetPlaceholderUsed(true)
 	sn.allocations[replace.GetUUID()] = replace
 	before := sn.allocatedResource.Clone()
+	// The allocatedResource and availableResource should be updated in the same way
 	sn.allocatedResource.AddTo(delta)
+	sn.availableResource.SubFrom(delta)
 	if !resources.FitIn(before, sn.allocatedResource) {
 		log.Logger().Warn("unexpected increase in node usage after placeholder replacement",
 			zap.String("placeholder uuid", uuid),
 			zap.String("allocation uuid", replace.GetUUID()),
-			zap.String("delta", delta.String()))
+			zap.Stringer("delta", delta))
 	}
 }
 
@@ -370,12 +398,12 @@ func (sn *Node) preConditions(ask *AllocationAsk, allocate bool) bool {
 
 // Check if the node should be considered as a possible node to allocate on.
 // This is a lock free call. No updates are made this only performs a pre allocate checks
-func (sn *Node) preAllocateCheck(res *resources.Resource, resKey string) error {
+func (sn *Node) preAllocateCheck(res *resources.Resource, resKey string) bool {
 	// cannot allocate zero or negative resource
 	if !resources.StrictlyGreaterThanZero(res) {
 		log.Logger().Debug("pre alloc check: requested resource is zero",
 			zap.String("nodeID", sn.NodeID))
-		return fmt.Errorf("pre alloc check: requested resource is zero: %s", sn.NodeID)
+		return false
 	}
 	// check if the node is reserved for this app/alloc
 	if sn.IsReserved() {
@@ -383,19 +411,14 @@ func (sn *Node) preAllocateCheck(res *resources.Resource, resKey string) error {
 			log.Logger().Debug("pre alloc check: node reserved for different app or ask",
 				zap.String("nodeID", sn.NodeID),
 				zap.String("resKey", resKey))
-			return fmt.Errorf("pre alloc check: node %s reserved for different app or ask: %s", sn.NodeID, resKey)
+			return false
 		}
 	}
 
-	// check if resources are available
-	available := sn.GetAvailableResource()
-	// check the request fits in what we have calculated
-	if !available.FitInMaxUndef(res) {
-		// requested resource is larger than currently available node resources
-		return fmt.Errorf("pre alloc check: requested resource %s is larger than currently available %s resource on %s", res.String(), available.String(), sn.NodeID)
-	}
-	// can allocate, based on resource size
-	return nil
+	sn.RLock()
+	defer sn.RUnlock()
+	// returns true/false based on if the request fits in what we have calculated
+	return sn.availableResource.FitInMaxUndef(res)
 }
 
 // Return if the node has been reserved by any application
@@ -452,7 +475,7 @@ func (sn *Node) Reserve(app *Application, ask *AllocationAsk) error {
 			zap.String("nodeID", sn.NodeID),
 			zap.String("appID", app.ApplicationID),
 			zap.String("ask", ask.GetAllocationKey()),
-			zap.String("allocationAsk", ask.GetAllocatedResource().String()))
+			zap.Stringer("allocationAsk", ask.GetAllocatedResource()))
 		return fmt.Errorf("reservation does not fit on node %s, appID %s, ask %s", sn.NodeID, app.ApplicationID, ask.GetAllocatedResource().String())
 	}
 	sn.reservations[appReservation.getKey()] = appReservation
