@@ -19,9 +19,10 @@
 package objects
 
 import (
+	"fmt"
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
@@ -114,31 +115,24 @@ func TestPreAllocateCheck(t *testing.T) {
 	}
 
 	// special cases
-	if err := node.preAllocateCheck(nil, ""); err == nil {
+	if node.preAllocateCheck(nil, "") {
 		t.Errorf("nil resource should not have fitted on node")
 	}
 	resNeg := resources.NewResourceFromMap(map[string]resources.Quantity{"first": -1})
-	if err := node.preAllocateCheck(resNeg, ""); err == nil {
+	if node.preAllocateCheck(resNeg, "") {
 		t.Errorf("negative resource should not have fitted on node")
 	}
 	// Check if we can allocate on scheduling node
 	resSmall := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 5})
 	resLarge := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 15})
-	err := node.preAllocateCheck(resNode, "")
-	assert.NilError(t, err, "node resource should have fitted on node")
-	err = node.preAllocateCheck(resSmall, "")
-	assert.NilError(t, err, "small resource should have fitted on node")
-	if err = node.preAllocateCheck(resLarge, ""); err == nil {
-		t.Errorf("too large resource should not have fitted on node: %v", err)
-	}
+	assert.Assert(t, node.preAllocateCheck(resNode, ""), "node resource should have fitted on node")
+	assert.Assert(t, node.preAllocateCheck(resSmall, ""), "small resource should have fitted on node")
+	assert.Assert(t, !node.preAllocateCheck(resLarge, ""), "too large resource should not have fitted on node")
 
 	// set allocated resource
 	node.AddAllocation(newAllocation(appID1, "UUID1", nodeID, "root.default", resSmall))
-	err = node.preAllocateCheck(resSmall, "")
-	assert.NilError(t, err, "small resource should have fitted in available allocation")
-	if err = node.preAllocateCheck(resNode, ""); err == nil {
-		t.Errorf("node resource should not have fitted in available allocation: %v", err)
-	}
+	assert.Assert(t, node.preAllocateCheck(resSmall, ""), "small resource should have fitted in available allocation")
+	assert.Assert(t, !node.preAllocateCheck(resNode, ""), "node resource should not have fitted in available allocation")
 
 	// check if we can allocate on a reserved node
 	q := map[string]resources.Quantity{"first": 0}
@@ -149,22 +143,14 @@ func TestPreAllocateCheck(t *testing.T) {
 	// standalone reservation unreserve returns false as app is not reserved
 	reserve := newReservation(node, app, ask, false)
 	node.reservations[reserve.getKey()] = reserve
-	if err = node.preAllocateCheck(resSmall, "app-2"); err == nil {
-		t.Errorf("node was reserved for different app but check passed: %v", err)
-	}
-	if err = node.preAllocateCheck(resSmall, "app-1|alloc-2"); err == nil {
-		t.Errorf("node was reserved for this app but not the alloc and check passed: %v", err)
-	}
-	err = node.preAllocateCheck(resSmall, appID1)
-	assert.NilError(t, err, "node was reserved for this app but check did not pass check")
-	err = node.preAllocateCheck(resSmall, "app-1|alloc-1")
-	assert.NilError(t, err, "node was reserved for this app/alloc but check did not pass check")
+	assert.Assert(t, !node.preAllocateCheck(resSmall, "app-2"), "node was reserved for different app but check passed")
+	assert.Assert(t, !node.preAllocateCheck(resSmall, "app-1|alloc-2"), "node was reserved for this app but not the alloc and check passed")
+	assert.Assert(t, node.preAllocateCheck(resSmall, appID1), "node was reserved for this app but check did not pass check")
+	assert.Assert(t, node.preAllocateCheck(resSmall, "app-1|alloc-1"), "node was reserved for this app/alloc but check did not pass check")
 
 	// Check if we can allocate on non scheduling node
 	node.SetSchedulable(false)
-	if err = node.preAllocateCheck(resSmall, ""); err == nil {
-		t.Errorf("node with scheduling set to false should not allow allocation: %v", err)
-	}
+	assert.Assert(t, !node.preAllocateCheck(resSmall, ""), "node with scheduling set to false should not allow allocation")
 }
 
 // Only test the CanAllocate code, the used logic in preAllocateCheck has its own test
@@ -292,9 +278,69 @@ func TestIsReservedForApp(t *testing.T) {
 }
 
 func TestAttributes(t *testing.T) {
+	type outputFormat struct {
+		hostname, rackname, partition string
+		attribites                    map[string]string
+	}
+	attribitesOfNode1 := map[string]string{common.NodePartition: "partition1", "something": "just a text"}
+	attribitesOfNode2 := map[string]string{common.HostName: "test", common.NodePartition: "partition2", "disk": "SSD", "GPU-type": "3090"}
+	var tests = []struct {
+		inputs   map[string]string
+		expected outputFormat
+	}{
+		{
+			attribitesOfNode1,
+			outputFormat{"", "", "partition1", attribitesOfNode1},
+		},
+		{
+			attribitesOfNode2,
+			outputFormat{"test", "", "partition2", attribitesOfNode2},
+		},
+	}
+	for index, tt := range tests {
+		testname := fmt.Sprintf("Attributes in the node %d", index)
+		t.Run(testname, func(t *testing.T) {
+			nodename := fmt.Sprintf("%s-%d", testNode, index)
+			node := NewNode(newProto(nodename, nil, nil, tt.inputs))
+			if node == nil || node.NodeID != nodename {
+				t.Error("node not returned correctly: node is nul or incorrect name")
+			}
+
+			if got, expect := node.Hostname, tt.expected.hostname; got != expect {
+				t.Errorf("node hostname: got %s, expected %s", got, expect)
+			}
+
+			if got, expect := node.Rackname, tt.expected.rackname; got != expect {
+				t.Errorf("node rackname: got %s, expected %s", got, expect)
+			}
+
+			if got, expect := node.Partition, tt.expected.partition; got != expect {
+				t.Errorf("node partition: got %s, expected %s", got, expect)
+			}
+
+			attribites := node.GetAttributes()
+			if got, expect := len(attribites), len(tt.expected.attribites); got != expect {
+				t.Errorf("length of attributes: got %d, expected %d", got, expect)
+			}
+
+			for key, expect := range tt.expected.attribites {
+				if got := node.GetAttribute(key); got != expect {
+					t.Errorf("Attribute %s: got %s, expect %s", key, got, expect)
+				}
+				if got := attribites[key]; got != expect {
+					t.Errorf("Attribute %s: got %s, expect %s", key, got, expect)
+				}
+			}
+		})
+	}
+}
+
+func TestGetInstanceType(t *testing.T) {
 	proto := newProto(testNode, nil, nil, map[string]string{
 		common.NodePartition: "partition1",
-		"something":          "just a text",
+		"label1":             "key1",
+		"label2":             "key2",
+		common.InstanceType:  "HighMem",
 	})
 
 	node := NewNode(proto)
@@ -306,10 +352,8 @@ func TestAttributes(t *testing.T) {
 	assert.Equal(t, "", node.Rackname)
 	assert.Equal(t, "partition1", node.Partition)
 
-	value := node.GetAttribute(common.NodePartition)
-	assert.Equal(t, "partition1", value, "node attributes not set, expected 'partition1' got '%v'", value)
-	value = node.GetAttribute("something")
-	assert.Equal(t, "just a text", value, "node attributes not set, expected 'just a text' got '%v'", value)
+	value := node.GetInstanceType()
+	assert.Equal(t, "HighMem", value, "node instanceType not set, expected 'HighMem' got '%v'", value)
 }
 
 func TestAddAllocation(t *testing.T) {
@@ -416,6 +460,7 @@ func TestNodeReplaceAllocation(t *testing.T) {
 	node.AddAllocation(ph)
 	assert.Assert(t, node.GetAllocation(phID) != nil, "failed to add placeholder allocation")
 	assert.Assert(t, resources.Equals(node.GetAllocatedResource(), half), "allocated resource not set correctly %v got %v", half, node.GetAllocatedResource())
+	assert.Assert(t, resources.Equals(node.GetAvailableResource(), half), "available resource not set correctly %v got %v", half, node.GetAvailableResource())
 
 	allocID := "real-1"
 	piece := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 25, "second": 50})
@@ -427,10 +472,12 @@ func TestNodeReplaceAllocation(t *testing.T) {
 	node.ReplaceAllocation(phID, alloc, delta)
 	assert.Assert(t, node.GetAllocation(allocID) != nil, "failed to replace allocation: allocation not returned")
 	assert.Assert(t, resources.Equals(node.GetAllocatedResource(), piece), "allocated resource not set correctly %v got %v", piece, node.GetAllocatedResource())
+	assert.Assert(t, resources.Equals(node.GetAvailableResource(), resources.Sub(node.GetCapacity(), piece)), "available resource not set correctly %v got %v", resources.Sub(node.GetCapacity(), piece), node.GetAvailableResource())
 
 	// clean up all should be zero
 	assert.Assert(t, node.RemoveAllocation(allocID) != nil, "allocation should have been removed but was not")
 	assert.Assert(t, resources.IsZero(node.GetAllocatedResource()), "allocated resource not updated correctly")
+	assert.Assert(t, resources.Equals(node.GetAvailableResource(), node.GetCapacity()), "available resource not set correctly %v got %v", node.GetCapacity(), node.GetAvailableResource())
 }
 
 func TestGetAllocation(t *testing.T) {
