@@ -24,7 +24,7 @@ import (
 	"testing"
 	"time"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
@@ -402,24 +402,16 @@ func TestSortAsks(t *testing.T) {
 	list[0], list[2] = list[2], list[0]
 	list[1], list[3] = list[3], list[1]
 	assertAskList(t, list, []int{2, 3, 0, 1}, "moved 1")
-	sortAskByPriority(list, true)
-	// asks should come back in order: 0, 1, 2, 3
-	assertAskList(t, list, []int{0, 1, 2, 3}, "ascending")
-	// move things around
-	list[0], list[2] = list[2], list[0]
-	list[1], list[3] = list[3], list[1]
-	assertAskList(t, list, []int{2, 3, 0, 1}, "moved 2")
-	sortAskByPriority(list, false)
+
+	sortAskByPriority(list)
 	// asks should come back in order: 3, 2, 1, 0
 	assertAskList(t, list, []int{3, 2, 1, 0}, "descending")
+
 	// make asks with same priority
 	// ask-3 and ask-1 both with prio 1 do not change order
 	// ask-3 must always be earlier in the list
 	list[0].priority = 1
-	sortAskByPriority(list, true)
-	// asks should come back in order: 0, 2, 3, 1
-	assertAskList(t, list, []int{0, 1, 3, 2}, "ascending same prio")
-	sortAskByPriority(list, false)
+	sortAskByPriority(list)
 	// asks should come back in order: 3, 2, 0, 1
 	assertAskList(t, list, []int{3, 1, 0, 2}, "descending same prio")
 }
@@ -456,3 +448,26 @@ func assertAppListLength(t *testing.T, list []*Application, apps []string, name
 		assert.Equal(t, apps[i], app.ApplicationID, "test name: %s", name)
 	}
 }
+
+func TestSortBySubmissionTime(t *testing.T) {
+	var list []*Application
+
+	// stable sort is used so equal values stay where they were
+	// let all resource have same resource but submissionTime is different
+	res := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": resources.Quantity(100)})
+	baseline := time.Now()
+	// setup to sort descending: all apps have pending resource
+	input := make(map[string]*Application, 4)
+	for i := 0; i < 4; i++ {
+		num := strconv.Itoa(i)
+		appID := "app-" + num
+		app := newApplication(appID, "partition", "queue")
+		app.pending = res
+		input[appID] = app
+		app.SubmissionTime = baseline.Add(-time.Minute * time.Duration(i))
+		input[appID] = app
+	}
+
+	list = sortApplications(input, policies.FifoSortPolicy, true, nil)
+	assertAppList(t, list, []int{3, 2, 1, 0}, "sort by submission time")
+}
