@@ -19,6 +19,7 @@
 package scheduler
 
 import (
+	"context"
 	"fmt"
 	"math"
 	"strconv"
@@ -56,7 +57,6 @@ type PartitionContext struct {
 	partitionManager       *partitionManager               // manager for this partition
 	stateMachine           *fsm.FSM                        // the state of the partition for scheduling
 	stateTime              time.Time                       // last time the state was updated (needed for cleanup)
-	isPreemptable          bool                            // can allocations be preempted
 	rules                  *[]configs.PlacementRule        // placement rules to be loaded by the scheduler
 	userGroupCache         *security.UserGroupCache        // user cache per partition
 	totalPartitionResource *resources.Resource             // Total node resources
@@ -120,9 +120,6 @@ func (pc *PartitionContext) initialPartitionFromConfig(conf configs.PartitionCon
 		zap.String("partitionName", pc.Name),
 		zap.String("rmID", pc.RmID))
 
-	// set preemption needed flag
-	pc.isPreemptable = conf.Preemption.Enabled
-
 	pc.rules = &conf.PlacementRules
 	// We need to pass in the locked version of the GetQueue function.
 	// Placing an application will not have a lock on the partition context.
@@ -144,7 +141,7 @@ func (pc *PartitionContext) updateNodeSortingPolicy(conf configs.PartitionConfig
 		log.Logger().Info(fmt.Sprintf("NodeSorting policy not set using '%s' as default", configuredPolicy))
 	} else {
 		log.Logger().Info("NodeSorting policy set from config",
-			zap.String("policyName", configuredPolicy.String()))
+			zap.Stringer("policyName", configuredPolicy))
 	}
 	pc.nodes.SetNodeSortingPolicy(objects.NewNodeSortingPolicy(conf.NodeSortPolicy.Type, conf.NodeSortPolicy.ResourceWeights))
 }
@@ -268,7 +265,7 @@ func (pc *PartitionContext) isStopped() bool {
 // Handle the state event for the partition.
 // The state machine handles the locking.
 func (pc *PartitionContext) handlePartitionEvent(event objects.ObjectEvent) error {
-	err := pc.stateMachine.Event(event.String(), pc.Name)
+	err := pc.stateMachine.Event(context.Background(), event.String(), pc.Name)
 	if err == nil {
 		pc.stateTime = time.Now()
 		return nil
@@ -557,7 +554,7 @@ func (pc *PartitionContext) AddNode(node *objects.Node, existingAllocations []*o
 					zap.Int("existingAllocations", len(existingAllocations)),
 					zap.Int("releasedAllocations", len(released)),
 					zap.Int("processingAlloc", current),
-					zap.String("allocation", alloc.String()),
+					zap.Stringer("allocation", alloc),
 					zap.Error(err))
 				// update failed metric, active metrics are tracked in add/remove from list
 				metrics.GetSchedulerMetrics().IncFailedNodes()
@@ -604,7 +601,7 @@ func (pc *PartitionContext) addNodeToList(node *objects.Node) error {
 	log.Logger().Info("Updated available resources from added node",
 		zap.String("partitionName", pc.Name),
 		zap.String("nodeID", node.NodeID),
-		zap.String("partitionResource", pc.totalPartitionResource.String()))
+		zap.Stringer("partitionResource", pc.totalPartitionResource))
 	return nil
 }
 
@@ -631,7 +628,7 @@ func (pc *PartitionContext) removeNodeFromList(nodeID string) *objects.Node {
 	log.Logger().Info("Updated available resources from removed node",
 		zap.String("partitionName", pc.Name),
 		zap.String("nodeID", node.NodeID),
-		zap.String("partitionResource", pc.totalPartitionResource.String()))
+		zap.Stringer("partitionResource", pc.totalPartitionResource))
 	return node
 }
 
@@ -712,9 +709,9 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*object
 						}
 						log.Logger().Warn("replacing placeholder: placeholder is larger than real allocation",
 							zap.String("allocationID", release.GetUUID()),
-							zap.String("requested resource", release.GetAllocatedResource().String()),
+							zap.Stringer("requested resource", release.GetAllocatedResource()),
 							zap.String("placeholderID", alloc.GetUUID()),
-							zap.String("placeholder resource", alloc.GetAllocatedResource().String()))
+							zap.Stringer("placeholder resource", alloc.GetAllocatedResource()))
 					}
 					// track what we confirm on the other node to confirm it in the shim and get is bound
 					confirmed = append(confirmed, release)
@@ -763,6 +760,10 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*object
 		} else {
 			metrics.GetQueueMetrics(app.GetQueuePath()).IncReleasedContainer()
 		}
+		// remove preempted resources
+		if alloc.IsPreempted() {
+			app.GetQueue().DecPreemptingResource(alloc.GetAllocatedResource())
+		}
 
 		// the allocation is removed so add it to the list that we return
 		released = append(released, alloc)
@@ -792,7 +793,7 @@ func (pc *PartitionContext) tryAllocate() *objects.Allocation {
 		return nil
 	}
 	// try allocating from the root down
-	alloc := pc.root.TryAllocate(pc.GetNodeIterator, pc.GetNode)
+	alloc := pc.root.TryAllocate(pc.GetNodeIterator, pc.GetFullNodeIterator, pc.GetNode)
 	if alloc != nil {
 		return pc.allocate(alloc)
 	}
@@ -888,7 +889,7 @@ func (pc *PartitionContext) allocate(alloc *objects.Allocation) *objects.Allocat
 		zap.String("appID", alloc.GetApplicationID()),
 		zap.String("allocationKey", alloc.GetAllocationKey()),
 		zap.String("uuid", alloc.GetUUID()),
-		zap.String("allocatedResource", alloc.GetAllocatedResource().String()),
+		zap.Stringer("allocatedResource", alloc.GetAllocatedResource()),
 		zap.Bool("placeholder", alloc.IsPlaceholder()),
 		zap.String("targetNode", alloc.GetNodeID()))
 	// pass the allocation back to the RM via the cluster context
@@ -952,6 +953,12 @@ func (pc *PartitionContext) GetNodeIterator() objects.NodeIterator {
 	return pc.nodes.GetNodeIterator()
 }
 
+// Create an ordered node iterator based on the node sort policy set for this partition.
+// The iterator is nil if there are no nodes available.
+func (pc *PartitionContext) GetFullNodeIterator() objects.NodeIterator {
+	return pc.nodes.GetFullNodeIterator()
+}
+
 // Updated the allocations counter for the partition
 func (pc *PartitionContext) updateAllocationCount(allocs int) {
 	pc.Lock()
@@ -973,12 +980,6 @@ func (pc *PartitionContext) GetAllocatedResource() *resources.Resource {
 	return pc.root.GetAllocatedResource()
 }
 
-func (pc *PartitionContext) GetTotalApplicationCount() int {
-	pc.RLock()
-	defer pc.RUnlock()
-	return len(pc.applications)
-}
-
 func (pc *PartitionContext) GetTotalAllocationCount() int {
 	pc.RLock()
 	defer pc.RUnlock()
@@ -1168,10 +1169,12 @@ func (pc *PartitionContext) convertUGI(ugi *si.UserGroupInformation) (security.U
 // which is a slice with 10 elements,
 // each element represents a range of resource usage,
 // such as
-//   0: 0%->10%
-//   1: 10% -> 20%
-//   ...
-//   9: 90% -> 100%
+//
+//	0: 0%->10%
+//	1: 10% -> 20%
+//	...
+//	9: 90% -> 100%
+//
 // the element value represents number of nodes fall into this bucket.
 // if slice[9] = 3, this means there are 3 nodes resource usage is in the range 80% to 90%.
 //
@@ -1180,9 +1183,11 @@ func (pc *PartitionContext) calculateNodesResourceUsage() map[string][]int {
 	nodesCopy := pc.GetNodes()
 	mapResult := make(map[string][]int)
 	for _, node := range nodesCopy {
-		for name, total := range node.GetCapacity().Resources {
-			if float64(total) > 0 {
-				resourceAllocated := float64(node.GetAllocatedResource().Resources[name])
+		capacity := node.GetCapacity()
+		allocated := node.GetAllocatedResource()
+		for name, total := range capacity.Resources {
+			if total > 0 {
+				resourceAllocated := float64(allocated.Resources[name])
 				// Consider over-allocated node as 100% utilized.
 				v := math.Min(resourceAllocated/float64(total), 1)
 				idx := int(math.Dim(math.Ceil(v*10), 1))
@@ -1216,7 +1221,7 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 		log.Logger().Info("Application not found while releasing allocation",
 			zap.String("appID", appID),
 			zap.String("allocationId", uuid),
-			zap.String("terminationType", release.TerminationType.String()))
+			zap.Stringer("terminationType", release.TerminationType))
 		return nil, nil
 	}
 	// temp store for allocations manipulated
@@ -1240,7 +1245,7 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 			log.Logger().Info("removing allocation from application",
 				zap.String("appID", appID),
 				zap.String("allocationId", uuid),
-				zap.String("terminationType", release.TerminationType.String()))
+				zap.Stringer("terminationType", release.TerminationType))
 			if alloc := app.RemoveAllocation(uuid, release.TerminationType); alloc != nil {
 				released = append(released, alloc)
 			}
@@ -1249,6 +1254,7 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 
 	// for each allocation to release, update the node and queue.
 	total := resources.NewResource()
+	totalPreempting := resources.NewResource()
 	for _, alloc := range released {
 		node := pc.GetNode(alloc.GetNodeID())
 		if node == nil {
@@ -1271,9 +1277,9 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 				total.SubFrom(delta)
 				log.Logger().Warn("replacing placeholder: placeholder is larger than real allocation",
 					zap.String("allocationID", confirmed.GetUUID()),
-					zap.String("requested resource", confirmed.GetAllocatedResource().String()),
+					zap.Stringer("requested resource", confirmed.GetAllocatedResource()),
 					zap.String("placeholderID", alloc.GetUUID()),
-					zap.String("placeholder resource", alloc.GetAllocatedResource().String()))
+					zap.Stringer("placeholder resource", alloc.GetAllocatedResource()))
 			}
 			// replacements could be on a different node and different size handle all cases
 			if confirmed.GetNodeID() == alloc.GetNodeID() {
@@ -1294,6 +1300,9 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 				zap.String("nodeID", alloc.GetNodeID()),
 				zap.String("allocationId", alloc.GetUUID()))
 		}
+		if alloc.IsPreempted() {
+			totalPreempting.AddTo(alloc.GetAllocatedResource())
+		}
 	}
 	if resources.StrictlyGreaterThanZero(total) {
 		queue := app.GetQueue()
@@ -1306,9 +1315,12 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 			metrics.GetQueueMetrics(queue.GetQueuePath()).IncReleasedContainer()
 		}
 	}
+	if resources.StrictlyGreaterThanZero(totalPreempting) {
+		app.GetQueue().DecPreemptingResource(totalPreempting)
+	}
+
 	// if confirmed is set we can assume there will just be one alloc in the released
 	// that allocation was already released by the shim, so clean up released
-
 	if confirmed != nil {
 		released = nil
 	}
@@ -1343,7 +1355,7 @@ func (pc *PartitionContext) removeAllocationAsk(release *si.AllocationAskRelease
 		log.Logger().Info("Invalid ask release requested by shim",
 			zap.String("appID", appID),
 			zap.String("ask", allocKey),
-			zap.String("terminationType", release.TerminationType.String()))
+			zap.Stringer("terminationType", release.TerminationType))
 		return
 	}
 	// remove the allocation asks from the app
@@ -1402,6 +1414,8 @@ func (pc *PartitionContext) moveTerminatedApp(appID string) {
 	log.Logger().Info("Removing terminated application from the application list",
 		zap.String("appID", appID),
 		zap.String("app status", app.CurrentState()))
+	app.LogAppSummary(pc.RmID)
+	app.CleanupUsedResource()
 	pc.Lock()
 	defer pc.Unlock()
 	delete(pc.applications, appID)
