@@ -25,7 +25,7 @@ import (
 	"time"
 
 	"go.uber.org/zap"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/entrypoint"
 	"github.com/apache/yunikorn-core/pkg/log"
