@@ -27,7 +27,7 @@ import (
 	"time"
 
 	"go.uber.org/zap"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
