@@ -38,7 +38,7 @@ type QueueTracker struct {
 }
 
 func newRootQueueTracker() *QueueTracker {
-	return newQueueTracker("root")
+	return newQueueTracker(configs.RootQueue)
 }
 
 func newQueueTracker(queueName string) *QueueTracker {
@@ -57,24 +57,15 @@ func (qt *QueueTracker) increaseTrackedResource(queuePath string, applicationID
 	log.Logger().Debug("Increasing resource usage",
 		zap.String("queue path", queuePath),
 		zap.String("application", applicationID),
-		zap.String("resource", usage.String()))
+		zap.Stringer("resource", usage))
 	if queuePath == "" || applicationID == "" || usage == nil {
 		return fmt.Errorf("mandatory parameters are missing. queuepath: %s, application id: %s, resource usage: %s",
 			queuePath, applicationID, usage.String())
 	}
 	qt.resourceUsage.AddTo(usage)
 	qt.runningApplications[applicationID] = true
-	idx := strings.Index(queuePath, configs.DOT)
-	childQueuePath := ""
-	if idx != -1 {
-		childQueuePath = queuePath[idx+1:]
-	}
 
-	childIndex := strings.Index(childQueuePath, configs.DOT)
-	immediateChildQueueName := childQueuePath
-	if childIndex != -1 {
-		immediateChildQueueName = childQueuePath[:childIndex]
-	}
+	childQueuePath, immediateChildQueueName := getChildQueuePath(queuePath)
 	if childQueuePath != "" {
 		if qt.childQueueTrackers[immediateChildQueueName] == nil {
 			qt.childQueueTrackers[immediateChildQueueName] = newQueueTracker(immediateChildQueueName)
@@ -87,43 +78,41 @@ func (qt *QueueTracker) increaseTrackedResource(queuePath string, applicationID
 	return nil
 }
 
-func (qt *QueueTracker) decreaseTrackedResource(queuePath string, applicationID string, usage *resources.Resource, removeApp bool) error {
+func (qt *QueueTracker) decreaseTrackedResource(queuePath string, applicationID string, usage *resources.Resource, removeApp bool) (bool, error) {
 	log.Logger().Debug("Decreasing resource usage",
 		zap.String("queue path", queuePath),
 		zap.String("application", applicationID),
-		zap.String("resource", usage.String()),
+		zap.Stringer("resource", usage),
 		zap.Bool("removeApp", removeApp))
 	if queuePath == "" || usage == nil {
-		return fmt.Errorf("mandatory parameters are missing. queuepath: %s, application id: %s, resource usage: %s",
+		return false, fmt.Errorf("mandatory parameters are missing. queuepath: %s, application id: %s, resource usage: %s",
 			queuePath, applicationID, usage.String())
 	}
 	qt.resourceUsage.SubFrom(usage)
 	if removeApp {
 		delete(qt.runningApplications, applicationID)
 	}
-	idx := strings.Index(queuePath, configs.DOT)
-	childQueuePath := ""
-	if idx != -1 {
-		childQueuePath = queuePath[idx+1:]
-	}
-	childIndex := strings.Index(childQueuePath, configs.DOT)
-	immediateChildQueueName := childQueuePath
-	if childIndex != -1 {
-		immediateChildQueueName = childQueuePath[:childIndex]
-	}
+
+	childQueuePath, immediateChildQueueName := getChildQueuePath(queuePath)
 	if childQueuePath != "" {
 		if qt.childQueueTrackers[immediateChildQueueName] != nil {
-			err := qt.childQueueTrackers[immediateChildQueueName].decreaseTrackedResource(childQueuePath, applicationID, usage, removeApp)
+			removeQT, err := qt.childQueueTrackers[immediateChildQueueName].decreaseTrackedResource(childQueuePath, applicationID, usage, removeApp)
 			if err != nil {
-				return err
+				return false, err
+			}
+			if removeQT {
+				delete(qt.childQueueTrackers, immediateChildQueueName)
 			}
 		} else {
 			log.Logger().Error("Child queueTracker tracker must be available in child queues map",
 				zap.String("child queueTracker name", immediateChildQueueName))
-			return fmt.Errorf("child queueTracker tracker for %s is missing in child queues map", immediateChildQueueName)
+			return false, fmt.Errorf("child queueTracker tracker for %s is missing in child queues map", immediateChildQueueName)
 		}
 	}
-	return nil
+
+	// Determine if the queue tracker should be removed
+	removeQT := len(qt.childQueueTrackers) == 0 && len(qt.runningApplications) == 0 && resources.IsZero(qt.resourceUsage)
+	return removeQT, nil
 }
 
 func (qt *QueueTracker) getResourceUsageDAOInfo(parentQueuePath string) *dao.ResourceUsageDAOInfo {
@@ -148,6 +137,18 @@ func (qt *QueueTracker) getResourceUsageDAOInfo(parentQueuePath string) *dao.Res
 	return usage
 }
 
-func (qt *QueueTracker) getRunningApplications() map[string]bool {
-	return qt.runningApplications
+func getChildQueuePath(queuePath string) (string, string) {
+	idx := strings.Index(queuePath, configs.DOT)
+	childQueuePath := ""
+	if idx != -1 {
+		childQueuePath = queuePath[idx+1:]
+	}
+
+	childIndex := strings.Index(childQueuePath, configs.DOT)
+	immediateChildQueueName := childQueuePath
+	if childIndex != -1 {
+		immediateChildQueueName = childQueuePath[:childIndex]
+	}
+
+	return childQueuePath, immediateChildQueueName
 }
