@@ -21,7 +21,7 @@ package ugm
 import (
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 )
@@ -79,7 +79,7 @@ func TestQTIncreaseTrackedResource(t *testing.T) {
 	assert.Equal(t, "map[mem:30000000 vcore:30000]", actualResources["root.parent.child1.child12"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:20000000 vcore:20000]", actualResources["root.parent.child2"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:20000000 vcore:20000]", actualResources["root.parent.child12"].String(), "wrong resource")
-	assert.Equal(t, 4, len(queueTracker.getRunningApplications()))
+	assert.Equal(t, 4, len(queueTracker.runningApplications))
 }
 
 func TestQTDecreaseTrackedResource(t *testing.T) {
@@ -96,7 +96,7 @@ func TestQTDecreaseTrackedResource(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v, error %t", queuePath1, TestApp1, usage1, err)
 	}
-	assert.Equal(t, 1, len(queueTracker.getRunningApplications()))
+	assert.Equal(t, 1, len(queueTracker.runningApplications))
 
 	usage2, err := resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
 	if err != nil {
@@ -108,7 +108,7 @@ func TestQTDecreaseTrackedResource(t *testing.T) {
 	}
 	actualResources := getQTResource(queueTracker)
 
-	assert.Equal(t, 2, len(queueTracker.getRunningApplications()))
+	assert.Equal(t, 2, len(queueTracker.runningApplications))
 	assert.Equal(t, "map[mem:90000000 vcore:90000]", actualResources["root"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:90000000 vcore:90000]", actualResources["root.parent"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:70000000 vcore:70000]", actualResources["root.parent.child1"].String(), "wrong resource")
@@ -122,32 +122,80 @@ func TestQTDecreaseTrackedResource(t *testing.T) {
 	err = queueTracker.increaseTrackedResource("", "", usage3)
 	assert.Error(t, err, "mandatory parameters are missing. queuepath: , application id: , resource usage: "+usage3.String())
 
-	err = queueTracker.decreaseTrackedResource(queuePath1, TestApp1, usage3, false)
+	removeQT, err := queueTracker.decreaseTrackedResource(queuePath1, TestApp1, usage3, false)
 	if err != nil {
 		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath1, TestApp1, usage3, err)
 	}
-	err = queueTracker.decreaseTrackedResource(queuePath2, TestApp2, usage3, false)
+	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
+
+	removeQT, err = queueTracker.decreaseTrackedResource(queuePath2, TestApp2, usage3, false)
 	if err != nil {
 		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath2, TestApp2, usage3, err)
 	}
 	actualResources1 := getQTResource(queueTracker)
 
+	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
 	assert.Equal(t, "map[mem:70000000 vcore:70000]", actualResources1["root"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:70000000 vcore:70000]", actualResources1["root.parent"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:60000000 vcore:60000]", actualResources1["root.parent.child1"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:10000000 vcore:10000]", actualResources1["root.parent.child2"].String(), "wrong resource")
+	assert.Equal(t, len(queueTracker.childQueueTrackers["parent"].childQueueTrackers), 2)
 
-	err = queueTracker.decreaseTrackedResource(queuePath1, TestApp1, usage1, true)
+	usage4, err := resources.NewResourceFromConf(map[string]string{"mem": "60M", "vcore": "60"})
+	if err != nil {
+		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
+	}
+	removeQT, err = queueTracker.decreaseTrackedResource(queuePath1, TestApp1, usage4, true)
 	if err != nil {
 		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath1, TestApp1, usage1, err)
 	}
-	assert.Equal(t, 1, len(queueTracker.getRunningApplications()))
+	assert.Equal(t, 1, len(queueTracker.runningApplications))
+	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
+	// Make sure childQueueTracker cleaned
+	assert.Equal(t, len(queueTracker.childQueueTrackers["parent"].childQueueTrackers), 1)
 
-	err = queueTracker.decreaseTrackedResource(queuePath2, TestApp2, usage2, true)
+	usage5, err := resources.NewResourceFromConf(map[string]string{"mem": "10M", "vcore": "10"})
+	if err != nil {
+		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage5)
+	}
+	removeQT, err = queueTracker.decreaseTrackedResource(queuePath2, TestApp2, usage5, true)
 	if err != nil {
 		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath2, TestApp2, usage2, err)
 	}
-	assert.Equal(t, 0, len(queueTracker.getRunningApplications()))
+	assert.Equal(t, 0, len(queueTracker.runningApplications))
+	// Make sure all childQueueTracker cleaned
+	assert.Equal(t, len(queueTracker.childQueueTrackers), 0)
+	assert.Equal(t, removeQT, true, "wrong remove queue tracker value")
+
+	// Test parent queueTracker has not zero usage, but child queueTrackers has all deleted
+	err = queueTracker.increaseTrackedResource(queuePath1, TestApp1, usage1)
+	if err != nil {
+		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v, error %t", queuePath1, TestApp1, usage1, err)
+	}
+	assert.Equal(t, 1, len(queueTracker.runningApplications))
+
+	usage2, err = resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
+	if err != nil {
+		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage2)
+	}
+	err = queueTracker.increaseTrackedResource("root.parent", TestApp2, usage2)
+	if err != nil {
+		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v, error %t", "root.parent", TestApp2, usage2, err)
+	}
+}
+
+func TestGetChildQueuePath(t *testing.T) {
+	childPath, immediateChildName := getChildQueuePath("root.parent.leaf")
+	assert.Equal(t, childPath, "parent.leaf")
+	assert.Equal(t, immediateChildName, "parent")
+
+	childPath, immediateChildName = getChildQueuePath("parent.leaf")
+	assert.Equal(t, childPath, "leaf")
+	assert.Equal(t, immediateChildName, "leaf")
+
+	childPath, immediateChildName = getChildQueuePath("leaf")
+	assert.Equal(t, childPath, "")
+	assert.Equal(t, immediateChildName, "")
 }
 
 func getQTResource(qt *QueueTracker) map[string]*resources.Resource {
