@@ -21,7 +21,7 @@ package scheduler
 import (
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
@@ -38,13 +38,16 @@ const (
 	appID2    = "app-2"
 	appID3    = "app-3"
 	nodeID1   = "node-1"
+	instType1 = "itype-1"
 	nodeID2   = "node-2"
+	instType2 = "itype-2"
 	defQueue  = "root.default"
 	rmID      = "testRM"
 	taskGroup = "tg-1"
 	phID      = "ph-1"
 	allocID   = "alloc-1"
 	allocID2  = "alloc-2"
+	allocID3  = "alloc-3"
 )
 
 func newBasePartition() (*PartitionContext, error) {
@@ -66,7 +69,6 @@ func newBasePartition() (*PartitionContext, error) {
 		},
 		PlacementRules: nil,
 		Limits:         nil,
-		Preemption:     configs.PartitionPreemptionConfig{},
 		NodeSortPolicy: configs.NodeSortingPolicy{},
 	}
 
@@ -102,7 +104,52 @@ func newConfiguredPartition() (*PartitionContext, error) {
 		},
 		PlacementRules: nil,
 		Limits:         nil,
-		Preemption:     configs.PartitionPreemptionConfig{},
+		NodeSortPolicy: configs.NodeSortingPolicy{},
+	}
+	return newPartitionContext(conf, rmID, nil)
+}
+
+func newPreemptionConfiguredPartition(parentLimit map[string]string, leafGuarantees map[string]string) (*PartitionContext, error) {
+	conf := configs.PartitionConfig{
+		Name: "test",
+		Queues: []configs.QueueConfig{
+			{
+				Name:      "root",
+				Parent:    true,
+				SubmitACL: "*",
+				Queues: []configs.QueueConfig{
+					{
+						Name:   "parent",
+						Parent: true,
+						Resources: configs.Resources{
+							Max: parentLimit,
+						},
+						Queues: []configs.QueueConfig{
+							{
+								Name:   "leaf1",
+								Parent: false,
+								Queues: nil,
+								Resources: configs.Resources{
+									Guaranteed: leafGuarantees,
+								},
+								Properties: map[string]string{configs.PreemptionDelay: "1ms"},
+							},
+							{
+								Name:   "leaf2",
+								Parent: false,
+								Queues: nil,
+								Resources: configs.Resources{
+									Guaranteed: leafGuarantees,
+								},
+								Properties: map[string]string{configs.PreemptionDelay: "1ms"},
+							},
+						},
+					},
+				},
+			},
+		},
+		PlacementRules: nil,
+		Limits:         nil,
 		NodeSortPolicy: configs.NodeSortingPolicy{},
 	}
 	return newPartitionContext(conf, rmID, nil)
@@ -130,7 +177,6 @@ func newLimitedPartition(resLimit map[string]string) (*PartitionContext, error)
 		},
 		PlacementRules: nil,
 		Limits:         nil,
-		Preemption:     configs.PartitionPreemptionConfig{},
 		NodeSortPolicy: configs.NodeSortingPolicy{},
 	}
 
@@ -157,7 +203,6 @@ func newPlacementPartition() (*PartitionContext, error) {
 			},
 		},
 		Limits:         nil,
-		Preemption:     configs.PartitionPreemptionConfig{},
 		NodeSortPolicy: configs.NodeSortingPolicy{},
 	}
 
@@ -210,6 +255,22 @@ func newApplicationTGTags(appID, partition, queueName string, task *resources.Re
 	return objects.NewApplication(siApp, user, nil, rmID)
 }
 
+func newApplicationTGTagsWithPhTimeout(appID, partition, queueName string, task *resources.Resource, tags map[string]string, phTimeout int64) *objects.Application {
+	user := security.UserGroup{
+		User:   "testuser",
+		Groups: []string{"testgroup"},
+	}
+	siApp := &si.AddApplicationRequest{
+		ApplicationID:                appID,
+		QueueName:                    queueName,
+		PartitionName:                partition,
+		PlaceholderAsk:               task.ToProto(),
+		Tags:                         tags,
+		ExecutionTimeoutMilliSeconds: phTimeout,
+	}
+	return objects.NewApplication(siApp, user, nil, rmID)
+}
+
 func newAllocationAskTG(allocKey, appID, taskGroup string, res *resources.Resource, placeHolder bool) *objects.AllocationAsk {
 	return newAllocationAskAll(allocKey, appID, taskGroup, res, 1, 1, placeHolder)
 }
@@ -239,6 +300,22 @@ func newAllocationAskAll(allocKey, appID, taskGroup string, res *resources.Resou
 	})
 }
 
+func newAllocationAskPreempt(allocKey, appID string, prio int32, res *resources.Resource) *objects.AllocationAsk {
+	return objects.NewAllocationAskFromSI(&si.AllocationAsk{
+		AllocationKey:  allocKey,
+		ApplicationID:  appID,
+		PartitionName:  "default",
+		ResourceAsk:    res.ToProto(),
+		MaxAllocations: 1,
+		Priority:       prio,
+		TaskGroupName:  taskGroup,
+		Placeholder:    false,
+		PreemptionPolicy: &si.PreemptionPolicy{
+			AllowPreemptSelf:  true,
+			AllowPreemptOther: true,
+		},
+	})
+}
 func newNodeWithResources(nodeID string, max, occupied *resources.Resource) *objects.Node {
 	proto := &si.NodeInfo{
 		NodeID: nodeID,
@@ -257,7 +334,9 @@ func newNodeMaxResource(nodeID string, max *resources.Resource) *objects.Node {
 
 // partition with an expected basic queue hierarchy
 // root -> parent -> leaf1
-//      -> leaf2
+//
+//	-> leaf2
+//
 // and 2 nodes: node-1 & node-2
 func createQueuesNodes(t *testing.T) *PartitionContext {
 	partition, err := newConfiguredPartition()
@@ -272,6 +351,26 @@ func createQueuesNodes(t *testing.T) *PartitionContext {
 	return partition
 }
 
+// partition with a sibling relationship for testing preemption
+// root -> parent -> {leaf1,leaf2}
+//
+//	parent max: 10 vcore, leaf guarantees: 5 vcore
+//
+// and 2 nodes: node-1 & node-2
+func createPreemptionQueuesNodes(t *testing.T) *PartitionContext {
+	parentLimit := map[string]string{"vcore": "10"}
+	leafGuarantees := map[string]string{"vcore": "5"}
+	partition, err := newPreemptionConfiguredPartition(parentLimit, leafGuarantees)
+	assert.NilError(t, err, "test partition create failed with error")
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "10"})
+	assert.NilError(t, err, "failed to create basic resource")
+	err = partition.AddNode(newNodeMaxResource("node-1", res), nil)
+	assert.NilError(t, err, "test node1 add failed unexpected")
+	err = partition.AddNode(newNodeMaxResource("node-2", res), nil)
+	assert.NilError(t, err, "test node2 add failed unexpected")
+	return partition
+}
+
 func getTestUserGroup() security.UserGroup {
 	return security.UserGroup{User: "testuser", Groups: []string{"testgroup"}}
 }
