@@ -19,6 +19,7 @@
 package webservice
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"net/http"
@@ -27,10 +28,11 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/gorilla/mux"
+	"github.com/julienschmidt/httprouter"
+
 	"github.com/prometheus/client_golang/prometheus/promhttp"
 	"gopkg.in/yaml.v2"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/configs"
@@ -198,6 +200,7 @@ const rmID = "rm-123"
 const policyGroup = "default-policy-group"
 const queueName = "root.default"
 const nodeID = "node-1"
+const instType = "itype-1"
 
 // setup To take care of setting up config, cluster, partitions etc
 func setup(t *testing.T, config string, partitionCount int) *scheduler.PartitionContext {
@@ -492,8 +495,8 @@ func TestGetClusterUtilJSON(t *testing.T) {
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 300, siCommon.CPU: 200})
 	ask1 := objects.NewAllocationAsk("alloc-1", appID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", appID, resAlloc2)
-	alloc1 := objects.NewAllocation("alloc-1-uuid", nodeID, ask1)
-	alloc2 := objects.NewAllocation("alloc-2-uuid", nodeID, ask2)
+	alloc1 := objects.NewAllocation("alloc-1-uuid", nodeID, instType, ask1)
+	alloc2 := objects.NewAllocation("alloc-2-uuid", nodeID, instType, ask2)
 	allocs := []*objects.Allocation{alloc1, alloc2}
 	err = partition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
@@ -555,10 +558,10 @@ func TestGetNodesUtilJSON(t *testing.T) {
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 300, siCommon.CPU: 500, "GPU": 5})
 	ask1 := objects.NewAllocationAsk("alloc-1", appID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", appID, resAlloc2)
-	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, ask1)}
+	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, instType, ask1)}
 	err = partition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
-	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, ask2)}
+	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, instType, ask2)}
 	err = partition.AddNode(node2, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
 
@@ -651,10 +654,10 @@ func TestPartitions(t *testing.T) {
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 200, siCommon.CPU: 300})
 	ask1 := objects.NewAllocationAsk("alloc-1", app6.ApplicationID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", app3.ApplicationID, resAlloc2)
-	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, ask1)}
+	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, instType, ask1)}
 	err := defaultPartition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
-	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, ask2)}
+	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, instType, ask2)}
 	err = defaultPartition.AddNode(node2, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
 
@@ -705,8 +708,7 @@ func TestMetricsNotEmpty(t *testing.T) {
 	req, err := http.NewRequest("GET", "/ws/v1/metrics", strings.NewReader(""))
 	assert.NilError(t, err, "Error while creating the request")
 	rr := httptest.NewRecorder()
-	mux := http.HandlerFunc(promhttp.Handler().ServeHTTP)
-	handler := loggingHandler(mux, "/ws/v1/metrics")
+	handler := http.HandlerFunc(promhttp.Handler().ServeHTTP)
 	handler.ServeHTTP(rr, req)
 	assert.Assert(t, len(rr.Body.Bytes()) > 0, "Metrics response should not be empty")
 }
@@ -718,10 +720,7 @@ func TestGetPartitionQueuesHandler(t *testing.T) {
 
 	var req *http.Request
 	req, err := http.NewRequest("GET", "/ws/v1/partition/default/queues", strings.NewReader(""))
-	vars := map[string]string{
-		"partition": partitionNameWithoutClusterID,
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID}}))
 	assert.NilError(t, err, "Get Queues for PartitionQueues Handler request failed")
 	resp := &MockResponseWriter{}
 	var partitionQueuesDao dao.PartitionQueueDAOInfo
@@ -750,14 +749,18 @@ func TestGetPartitionQueuesHandler(t *testing.T) {
 
 	// Partition not exists
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/queues", strings.NewReader(""))
-	vars = map[string]string{
-		"partition": "notexists",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{httprouter.Param{Key: "partition", Value: "notexists"}}))
 	assert.NilError(t, err, "Get Queues for PartitionQueues Handler request failed")
 	resp = &MockResponseWriter{}
 	getPartitionQueues(resp, req)
 	assertPartitionExists(t, resp)
+
+	// test params name missing
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/queues", strings.NewReader(""))
+	assert.NilError(t, err, "Get Queues for PartitionQueues Handler request failed")
+	resp = &MockResponseWriter{}
+	getPartitionQueues(resp, req)
+	assertParamsMissing(t, resp)
 }
 
 func TestGetClusterInfo(t *testing.T) {
@@ -789,21 +792,23 @@ func TestGetPartitionNodes(t *testing.T) {
 	assert.NilError(t, err, "add application to partition should not have failed")
 
 	// create test nodes
+	attributesOfnode1 := map[string]string{"Disk": "SSD"}
+	attributesOfnode2 := map[string]string{"Devices": "camera"}
 	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 1000, siCommon.CPU: 1000}).ToProto()
 	node1ID := "node-1"
-	node1 := objects.NewNode(&si.NodeInfo{NodeID: node1ID, SchedulableResource: nodeRes})
+	node1 := objects.NewNode(&si.NodeInfo{NodeID: node1ID, Attributes: attributesOfnode1, SchedulableResource: nodeRes})
 	node2ID := "node-2"
-	node2 := objects.NewNode(&si.NodeInfo{NodeID: node2ID, SchedulableResource: nodeRes})
+	node2 := objects.NewNode(&si.NodeInfo{NodeID: node2ID, Attributes: attributesOfnode2, SchedulableResource: nodeRes})
 
 	// create test allocations
 	resAlloc1 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 500, siCommon.CPU: 300})
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 300, siCommon.CPU: 500})
 	ask1 := objects.NewAllocationAsk("alloc-1", appID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", appID, resAlloc2)
-	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, ask1)}
+	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, instType, ask1)}
 	err = partition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
-	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, ask2)}
+	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, instType, ask2)}
 	err = partition.AddNode(node2, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
 
@@ -811,10 +816,7 @@ func TestGetPartitionNodes(t *testing.T) {
 
 	var req *http.Request
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/nodes", strings.NewReader(""))
-	vars := map[string]string{
-		"partition": partitionNameWithoutClusterID,
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID}}))
 	assert.NilError(t, err, "Get Nodes for PartitionNodes Handler request failed")
 	resp := &MockResponseWriter{}
 	var partitionNodesDao []*dao.NodeDAOInfo
@@ -834,25 +836,45 @@ func TestGetPartitionNodes(t *testing.T) {
 			assert.Equal(t, node.NodeID, node1ID)
 			assert.Equal(t, "alloc-1", node.Allocations[0].AllocationKey)
 			assert.Equal(t, "alloc-1-uuid", node.Allocations[0].UUID)
+			assert.DeepEqual(t, attributesOfnode1, node.Attributes)
 			assert.DeepEqual(t, map[string]int64{"memory": 50, "vcore": 30}, node.Utilized)
 		} else {
 			assert.Equal(t, node.NodeID, node2ID)
 			assert.Equal(t, "alloc-2", node.Allocations[0].AllocationKey)
 			assert.Equal(t, "alloc-2-uuid", node.Allocations[0].UUID)
+			assert.DeepEqual(t, attributesOfnode2, node.Attributes)
 			assert.DeepEqual(t, map[string]int64{"memory": 30, "vcore": 50}, node.Utilized)
 		}
 	}
 
-	var req1 *http.Request
-	req1, err = http.NewRequest("GET", "/ws/v1/partition/default/nodes", strings.NewReader(""))
-	vars1 := map[string]string{
-		"partition": "notexists",
-	}
-	req1 = mux.SetURLVars(req1, vars1)
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/nodes", strings.NewReader(""))
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{httprouter.Param{Key: "partition", Value: "notexists"}}))
 	assert.NilError(t, err, "Get Nodes for PartitionNodes Handler request failed")
 	resp1 := &MockResponseWriter{}
-	getPartitionNodes(resp1, req1)
+	getPartitionNodes(resp1, req)
 	assertPartitionExists(t, resp1)
+
+	// test params name missing
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/nodes", strings.NewReader(""))
+	assert.NilError(t, err, "Get Nodes for PartitionNodes Handler request failed")
+	resp = &MockResponseWriter{}
+	getPartitionNodes(resp, req)
+	assertParamsMissing(t, resp)
+
+	// Test specific node
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/node/node-1", strings.NewReader(""))
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{httprouter.Param{Key: "node", Value: "node-1"}}))
+	assert.NilError(t, err, "Get Node for PartitionNode Handler request failed")
+	resp = &MockResponseWriter{}
+	getPartitionNode(resp, req)
+
+	// Test node id is missing
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/node/node-1", strings.NewReader(""))
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{httprouter.Param{Key: "partition", Value: "default"}, httprouter.Param{Key: "node", Value: ""}}))
+	assert.NilError(t, err, "Get Node for PartitionNode Handler request failed")
+	resp = &MockResponseWriter{}
+	getPartitionNode(resp, req)
+	assertNodeIDExists(t, resp)
 }
 
 // addApp Add app to the given partition and assert the app count, state etc
@@ -902,11 +924,10 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 
 	var req *http.Request
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/applications", strings.NewReader(""))
-	vars := map[string]string{
-		"partition": partitionNameWithoutClusterID,
-		"queue":     "root.default",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID},
+		httprouter.Param{Key: "queue", Value: "root.default"},
+	}))
 	assert.NilError(t, err, "Get Queue Applications Handler request failed")
 	resp := &MockResponseWriter{}
 	var appsDao []*dao.ApplicationDAOInfo
@@ -932,11 +953,10 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	// test nonexistent partition
 	var req1 *http.Request
 	req1, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/applications", strings.NewReader(""))
-	vars1 := map[string]string{
-		"partition": "notexists",
-		"queue":     "root.default",
-	}
-	req1 = mux.SetURLVars(req1, vars1)
+	req1 = req1.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: "notexists"},
+		httprouter.Param{Key: "queue", Value: "root.default"},
+	}))
 	assert.NilError(t, err, "Get Queue Applications Handler request failed")
 	resp1 := &MockResponseWriter{}
 	getQueueApplications(resp1, req1)
@@ -945,11 +965,10 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	// test nonexistent queue
 	var req2 *http.Request
 	req2, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/applications", strings.NewReader(""))
-	vars2 := map[string]string{
-		"partition": partitionNameWithoutClusterID,
-		"queue":     "notexists",
-	}
-	req2 = mux.SetURLVars(req2, vars2)
+	req2 = req2.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID},
+		httprouter.Param{Key: "queue", Value: "notexists"},
+	}))
 	assert.NilError(t, err, "Get Queue Applications Handler request failed")
 	resp2 := &MockResponseWriter{}
 	getQueueApplications(resp2, req2)
@@ -958,11 +977,10 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	// test queue without applications
 	var req3 *http.Request
 	req3, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.noapps/applications", strings.NewReader(""))
-	vars3 := map[string]string{
-		"partition": partitionNameWithoutClusterID,
-		"queue":     "root.noapps",
-	}
-	req3 = mux.SetURLVars(req3, vars3)
+	req3 = req3.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID},
+		httprouter.Param{Key: "queue", Value: "root.noapps"},
+	}))
 	assert.NilError(t, err, "Get Queue Applications Handler request failed")
 	resp3 := &MockResponseWriter{}
 	var appsDao3 []*dao.ApplicationDAOInfo
@@ -970,6 +988,13 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	err = json.Unmarshal(resp3.outputBytes, &appsDao3)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body: %s", string(resp.outputBytes))
 	assert.Equal(t, len(appsDao3), 0)
+
+	// test missing params name
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/applications", strings.NewReader(""))
+	assert.NilError(t, err, "Get Queue Applications Handler request failed")
+	resp = &MockResponseWriter{}
+	getQueueApplications(resp, req)
+	assertParamsMissing(t, resp)
 }
 
 func TestGetApplicationHandler(t *testing.T) {
@@ -992,12 +1017,11 @@ func TestGetApplicationHandler(t *testing.T) {
 
 	var req *http.Request
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/application/app-1", strings.NewReader(""))
-	vars := map[string]string{
-		"partition":   partitionNameWithoutClusterID,
-		"queue":       "root.default",
-		"application": "app-1",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID},
+		httprouter.Param{Key: "queue", Value: "root.default"},
+		httprouter.Param{Key: "application", Value: "app-1"},
+	}))
 	assert.NilError(t, err, "Get Application Handler request failed")
 	resp := &MockResponseWriter{}
 	var appsDao *dao.ApplicationDAOInfo
@@ -1014,12 +1038,11 @@ func TestGetApplicationHandler(t *testing.T) {
 	// test nonexistent partition
 	var req1 *http.Request
 	req1, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/application/app-1", strings.NewReader(""))
-	vars1 := map[string]string{
-		"partition":   "notexists",
-		"queue":       "root.default",
-		"application": "app-1",
-	}
-	req1 = mux.SetURLVars(req1, vars1)
+	req1 = req1.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: "notexists"},
+		httprouter.Param{Key: "queue", Value: "root.default"},
+		httprouter.Param{Key: "application", Value: "app-1"},
+	}))
 	assert.NilError(t, err, "Get Application Handler request failed")
 	resp1 := &MockResponseWriter{}
 	getApplication(resp1, req1)
@@ -1028,12 +1051,11 @@ func TestGetApplicationHandler(t *testing.T) {
 	// test nonexistent queue
 	var req2 *http.Request
 	req2, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/application/app-1", strings.NewReader(""))
-	vars2 := map[string]string{
-		"partition":   partitionNameWithoutClusterID,
-		"queue":       "notexists",
-		"application": "app-1",
-	}
-	req2 = mux.SetURLVars(req2, vars2)
+	req2 = req2.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID},
+		httprouter.Param{Key: "queue", Value: "notexists"},
+		httprouter.Param{Key: "application", Value: "app-1"},
+	}))
 	assert.NilError(t, err, "Get Application Handler request failed")
 	resp2 := &MockResponseWriter{}
 	getApplication(resp2, req2)
@@ -1042,16 +1064,31 @@ func TestGetApplicationHandler(t *testing.T) {
 	// test nonexistent application
 	var req3 *http.Request
 	req3, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.noapps/application/app-1", strings.NewReader(""))
-	vars3 := map[string]string{
-		"partition":   partitionNameWithoutClusterID,
-		"queue":       "root.noapps",
-		"application": "app-1",
-	}
-	req3 = mux.SetURLVars(req3, vars3)
+	req3 = req3.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "partition", Value: partitionNameWithoutClusterID},
+		httprouter.Param{Key: "queue", Value: "root.noapps"},
+		httprouter.Param{Key: "application", Value: "app-1"},
+	}))
 	assert.NilError(t, err, "Get Application Handler request failed")
 	resp3 := &MockResponseWriter{}
 	getApplication(resp3, req3)
 	assertApplicationExists(t, resp3)
+
+	// test missing params name
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/application/app-1", strings.NewReader(""))
+	assert.NilError(t, err, "Get Application Handler request failed")
+	resp = &MockResponseWriter{}
+	getApplication(resp, req)
+	assertParamsMissing(t, resp)
+}
+
+func assertParamsMissing(t *testing.T, resp *MockResponseWriter) {
+	var errInfo dao.YAPIError
+	err := json.Unmarshal(resp.outputBytes, &errInfo)
+	assert.NilError(t, err, "failed to unmarshal applications dao response from response body")
+	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
+	assert.Equal(t, errInfo.Message, MissingParamsName, "JSON error message is incorrect")
+	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
 func assertPartitionExists(t *testing.T, resp *MockResponseWriter) {
@@ -1117,6 +1154,15 @@ func assertGroupNameExists(t *testing.T, resp *MockResponseWriter) {
 	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
+func assertNodeIDExists(t *testing.T, resp *MockResponseWriter) {
+	var errInfo dao.YAPIError
+	err := json.Unmarshal(resp.outputBytes, &errInfo)
+	assert.NilError(t, err, "failed to unmarshal node dao response from response body")
+	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
+	assert.Equal(t, errInfo.Message, "Node not found", "JSON error message is incorrect")
+	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
+}
+
 func TestValidateQueue(t *testing.T) {
 	err := validateQueue("root.test.test123")
 	assert.NilError(t, err, "Queue path is correct but stil throwing error.")
@@ -1142,7 +1188,6 @@ func TestFullStateDumpPath(t *testing.T) {
 	imHistory = history.NewInternalMetricsHistory(5)
 	req, err2 := http.NewRequest("GET", "/ws/v1/getfullstatedump", strings.NewReader(""))
 	assert.NilError(t, err2)
-	req = mux.SetURLVars(req, make(map[string]string))
 	resp := &MockResponseWriter{}
 
 	getFullStateDump(resp, req)
@@ -1172,11 +1217,9 @@ func TestSetLoggerLevel(t *testing.T) {
 	// invalid
 	req, err := http.NewRequest("PUT", "/ws/v1/loglevel", strings.NewReader(""))
 	assert.NilError(t, err)
-
-	vars := map[string]string{
-		"level": "invalid",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "level", Value: "invalid"},
+	}))
 	rr := httptest.NewRecorder()
 
 	handler := http.HandlerFunc(setLogLevel)
@@ -1187,8 +1230,9 @@ func TestSetLoggerLevel(t *testing.T) {
 	req, err = http.NewRequest("PUT", "/ws/v1/loglevel", strings.NewReader(""))
 	assert.NilError(t, err)
 
-	vars["level"] = "error"
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "level", Value: "error"},
+	}))
 	rr = httptest.NewRecorder()
 	handler.ServeHTTP(rr, req)
 	assert.Equal(t, rr.Code, http.StatusOK)
@@ -1198,6 +1242,9 @@ func TestSpecificUserAndGroupResourceUsage(t *testing.T) {
 	prepareUserAndGroupContext(t)
 	// Test user name is missing
 	req, err := http.NewRequest("GET", "/ws/v1/partition/default/usage/user/", strings.NewReader(""))
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "group", Value: "testgroup"},
+	}))
 	assert.NilError(t, err, "Get User Resource Usage Handler request failed")
 	resp := &MockResponseWriter{}
 	getUserResourceUsage(resp, req)
@@ -1205,11 +1252,10 @@ func TestSpecificUserAndGroupResourceUsage(t *testing.T) {
 
 	// Test group name is missing
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/usage/group/", strings.NewReader(""))
-	vars := map[string]string{
-		"user":  "testuser",
-		"group": "",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "user", Value: "testuser"},
+		httprouter.Param{Key: "group", Value: ""},
+	}))
 	assert.NilError(t, err, "Get Group Resource Usage Handler request failed")
 	resp = &MockResponseWriter{}
 	getGroupResourceUsage(resp, req)
@@ -1217,22 +1263,20 @@ func TestSpecificUserAndGroupResourceUsage(t *testing.T) {
 
 	// Test existed user query
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/usage/user/", strings.NewReader(""))
-	vars = map[string]string{
-		"user":  "testuser",
-		"group": "testgroup",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "user", Value: "testuser"},
+		httprouter.Param{Key: "group", Value: "testgroup"},
+	}))
 	assert.NilError(t, err, "Get User Resource Usage Handler request failed")
 	resp = &MockResponseWriter{}
 	getUserResourceUsage(resp, req)
 
 	// Test non-existing user query
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/usage/user/", strings.NewReader(""))
-	vars = map[string]string{
-		"user":  "testNonExistingUser",
-		"group": "testgroup",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "user", Value: "testNonExistingUser"},
+		httprouter.Param{Key: "group", Value: "testgroup"},
+	}))
 	assert.NilError(t, err, "Get User Resource Usage Handler request failed")
 	resp = &MockResponseWriter{}
 	getUserResourceUsage(resp, req)
@@ -1241,11 +1285,10 @@ func TestSpecificUserAndGroupResourceUsage(t *testing.T) {
 	// Test existed group query
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/usage/group/", strings.NewReader(""))
 	assert.NilError(t, err, "Get Group Resource Usage Handler request failed")
-	vars = map[string]string{
-		"user":  "testuser",
-		"group": "testgroup",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "user", Value: "testuser"},
+		httprouter.Param{Key: "group", Value: "testgroup"},
+	}))
 	var groupResourceUsageDao *dao.GroupResourceUsageDAOInfo
 	getGroupResourceUsage(resp, req)
 	err = json.Unmarshal(resp.outputBytes, &groupResourceUsageDao)
@@ -1256,13 +1299,19 @@ func TestSpecificUserAndGroupResourceUsage(t *testing.T) {
 	// Test non-existing group query
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/usage/group/", strings.NewReader(""))
 	assert.NilError(t, err, "Get Group Resource Usage Handler request failed")
-	vars = map[string]string{
-		"user":  "testuser",
-		"group": "testNonExistingGroup",
-	}
-	req = mux.SetURLVars(req, vars)
+	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
+		httprouter.Param{Key: "user", Value: "testuser"},
+		httprouter.Param{Key: "group", Value: "testNonExistingGroup"},
+	}))
 	getGroupResourceUsage(resp, req)
 	assertGroupExists(t, resp)
+
+	// Test params name missing
+	req, err = http.NewRequest("GET", "/ws/v1/partition/default/usage/group/", strings.NewReader(""))
+	assert.NilError(t, err, "Get Group Resource Usage Handler request failed")
+	resp = &MockResponseWriter{}
+	getGroupResourceUsage(resp, req)
+	assertParamsMissing(t, resp)
 }
 
 func TestUsersAndGroupsResourceUsage(t *testing.T) {
@@ -1336,7 +1385,7 @@ func prepareUserAndGroupContext(t *testing.T) {
 
 	// add an alloc
 	uuid := "uuid-1"
-	allocInfo := objects.NewAllocation(uuid, "node-1", ask)
+	allocInfo := objects.NewAllocation(uuid, "node-1", instType, ask)
 	app.AddAllocation(allocInfo)
 	assert.Assert(t, app.IsStarting(), "Application did not return starting state after alloc: %s", app.CurrentState())
 
