@@ -47,16 +47,12 @@ import (
 )
 
 const (
-	autoGenAppPrefix                = "yunikorn"
-	autoGenAppSuffix                = "autogen"
 	yunikornPod                     = "yunikorn"
-	defaultQueue                    = "root.default"
 	admissionReviewAPIVersion       = "admission.k8s.io/v1"
 	admissionReviewKind             = "AdmissionReview"
 	schedulerValidateConfURLPattern = "http://%s/ws/v1/validate-conf"
 	mutateURL                       = "/mutate"
 	validateConfURL                 = "/validate-conf"
-	pod                             = "Pod"
 )
 
 var (
@@ -68,6 +64,7 @@ var (
 type AdmissionController struct {
 	conf              *conf.AdmissionControllerConf
 	pcCache           *PriorityClassCache
+	nsCache           *NamespaceCache
 	annotationHandler *metadata.UserGroupAnnotationHandler
 	labelExtractor    metadata.LabelExtractor
 }
@@ -77,10 +74,11 @@ type ValidateConfResponse struct {
 	Reason  string `json:"reason"`
 }
 
-func InitAdmissionController(conf *conf.AdmissionControllerConf, pcCache *PriorityClassCache) *AdmissionController {
+func InitAdmissionController(conf *conf.AdmissionControllerConf, pcCache *PriorityClassCache, nsCache *NamespaceCache) *AdmissionController {
 	hook := &AdmissionController{
 		conf:              conf,
 		pcCache:           pcCache,
+		nsCache:           nsCache,
 		annotationHandler: metadata.NewUserGroupAnnotationHandler(conf),
 	}
 
@@ -140,7 +138,7 @@ func (c *AdmissionController) mutate(req *admissionv1.AdmissionRequest) *admissi
 		zap.Any("UserInfo", req.UserInfo))
 
 	if req.Operation == admissionv1.Update {
-		if req.Kind.Kind == pod {
+		if req.Kind.Kind == metadata.Pod {
 			return c.processPodUpdate(req, namespace)
 		}
 
@@ -148,7 +146,7 @@ func (c *AdmissionController) mutate(req *admissionv1.AdmissionRequest) *admissi
 		return admissionResponseBuilder(string(req.UID), true, "", nil)
 	}
 
-	if req.Kind.Kind == pod {
+	if req.Kind.Kind == metadata.Pod {
 		return c.processPod(req, namespace)
 	}
 
@@ -198,15 +196,13 @@ func (c *AdmissionController) processPod(req *admissionv1.AdmissionRequest, name
 	patch = updateSchedulerName(patch)
 
 	if c.shouldLabelNamespace(namespace) {
-		patch = updateLabels(namespace, &pod, patch)
+		patch = c.updateLabels(namespace, &pod, patch)
 		patch = c.updatePreemptionInfo(&pod, patch)
 	} else {
-		log.Logger().Info("skipping update of pod labels since namespace is set to no-label",
-			zap.String("podName", pod.Name),
-			zap.String("generateName", pod.GenerateName),
-			zap.String("namespace", namespace))
+		patch = disableYuniKorn(namespace, &pod, patch)
 	}
 	log.Logger().Info("generated patch",
+		zap.String("namespace", namespace),
 		zap.String("podName", pod.Name),
 		zap.String("generateName", pod.GenerateName),
 		zap.Any("patch", patch))
@@ -223,6 +219,10 @@ func (c *AdmissionController) processPod(req *admissionv1.AdmissionRequest, name
 func (c *AdmissionController) processWorkload(req *admissionv1.AdmissionRequest, namespace string) *admissionv1.AdmissionResponse {
 	var uid = string(req.UID)
 
+	if !c.shouldProcessWorkload(req) {
+		return admissionResponseBuilder(uid, true, "", nil)
+	}
+
 	var supported bool
 	var err error
 	var labels map[string]string
@@ -329,6 +329,22 @@ func (c *AdmissionController) shouldProcessAdmissionReview(namespace string, lab
 	return false
 }
 
+func (c *AdmissionController) shouldProcessWorkload(req *admissionv1.AdmissionRequest) bool {
+	// checking a special case: replicaset submitted by a K8s system user
+	// we must not touch the spec otherwise we'll end up having an infinite loop of replicaset creation
+	if req.Kind.Kind == metadata.ReplicaSet {
+		for _, sysUser := range c.conf.GetSystemUsers() {
+			if sysUser.MatchString(req.UserInfo.Username) {
+				log.Logger().Info("ReplicaSet was created by a system user, skipping it",
+					zap.String("uid", string(req.UID)))
+				return false
+			}
+		}
+	}
+
+	return true
+}
+
 func (c *AdmissionController) checkUserInfoAnnotation(getAnnotation func() (string, bool), userName string, groups []string, uid string) (*admissionv1.AdmissionResponse, bool) {
 	annotation, userInfoSet := getAnnotation()
 	if userInfoSet && !c.conf.GetBypassAuth() {
@@ -359,14 +375,6 @@ func updateSchedulerName(patch []common.PatchOperation) []common.PatchOperation
 	})
 }
 
-// generate appID based on the namespace value,
-// and the max length of the ID is 63 chars.
-func generateAppID(namespace string) string {
-	generatedID := fmt.Sprintf("%s-%s-%s", autoGenAppPrefix, namespace, autoGenAppSuffix)
-	appID := fmt.Sprintf("%.63s", generatedID)
-	return appID
-}
-
 func (c *AdmissionController) updatePreemptionInfo(pod *v1.Pod, patch []common.PatchOperation) []common.PatchOperation {
 	value := utils.GetPodAnnotationValue(pod, constants.AnnotationAllowPreemption)
 
@@ -379,7 +387,7 @@ func (c *AdmissionController) updatePreemptionInfo(pod *v1.Pod, patch []common.P
 	}
 
 	value = constants.False
-	if c.pcCache.IsPreemptSelfAllowed(pod.Spec.PriorityClassName) {
+	if c.pcCache.isPreemptSelfAllowed(pod.Spec.PriorityClassName) {
 		value = constants.True
 	}
 
@@ -397,7 +405,7 @@ func (c *AdmissionController) updatePreemptionInfo(pod *v1.Pod, patch []common.P
 		}
 	}
 
-	result := UpdatePodAnnotationForAdmissionController(pod, constants.AnnotationAllowPreemption, value)
+	result := updatePodAnnotation(pod, constants.AnnotationAllowPreemption, value)
 	patch = append(patch, common.PatchOperation{
 		Op:    "add",
 		Path:  "/metadata/annotations",
@@ -407,14 +415,14 @@ func (c *AdmissionController) updatePreemptionInfo(pod *v1.Pod, patch []common.P
 	return patch
 }
 
-func updateLabels(namespace string, pod *v1.Pod, patch []common.PatchOperation) []common.PatchOperation {
+func (c *AdmissionController) updateLabels(namespace string, pod *v1.Pod, patch []common.PatchOperation) []common.PatchOperation {
 	log.Logger().Info("updating pod labels",
 		zap.String("podName", pod.Name),
 		zap.String("generateName", pod.GenerateName),
 		zap.String("namespace", namespace),
 		zap.Any("labels", pod.Labels))
 
-	result := UpdatePodLabelForAdmissionController(pod, namespace)
+	result := updatePodLabel(pod, namespace, c.conf.GetGenerateUniqueAppIds(), c.conf.GetDefaultQueueName())
 
 	patch = append(patch, common.PatchOperation{
 		Op:    "add",
@@ -425,6 +433,23 @@ func updateLabels(namespace string, pod *v1.Pod, patch []common.PatchOperation)
 	return patch
 }
 
+func disableYuniKorn(namespace string, pod *v1.Pod, patch []common.PatchOperation) []common.PatchOperation {
+	log.Logger().Info("disabling yunikorn on pod since namespace is set to no-label",
+		zap.String("podName", pod.Name),
+		zap.String("generateName", pod.GenerateName),
+		zap.String("namespace", namespace))
+
+	result := updatePodAnnotation(pod, constants.AnnotationIgnoreApplication, constants.True)
+
+	patch = append(patch, common.PatchOperation{
+		Op:    "add",
+		Path:  "/metadata/annotations",
+		Value: result,
+	})
+
+	return patch
+}
+
 func (c *AdmissionController) validateConf(req *admissionv1.AdmissionRequest) *admissionv1.AdmissionResponse {
 	if req == nil {
 		log.Logger().Warn("empty request received")
@@ -505,11 +530,28 @@ func (c *AdmissionController) namespaceMatchesNoLabelList(namespace string) bool
 	return false
 }
 
+// shouldProcessNamespace returns true if the pod in the namespace must be redirected to the
+// YuniKorn scheduler by setting the schedulerName
+// First check is the namespace annotation (tri-state)
+// - if present (FALSE, TRUE) return the value as boolean
+// - if not present (UNSET) fallback to matching names based on regexp
 func (c *AdmissionController) shouldProcessNamespace(namespace string) bool {
+	process := c.nsCache.enableYuniKorn(namespace)
+	if process != UNSET {
+		return process == TRUE
+	}
 	return c.namespaceMatchesProcessList(namespace) && !c.namespaceMatchesBypassList(namespace)
 }
 
+// shouldLabelNamespace returns true if the pod in the namespace must be labeled
+// First check is the namespace annotation (tri-state)
+// - if present (FALSE, TRUE) return the value as boolean
+// - if not present (UNSET) fallback to matching names based on regexp
 func (c *AdmissionController) shouldLabelNamespace(namespace string) bool {
+	label := c.nsCache.generateAppID(namespace)
+	if label != UNSET {
+		return label == TRUE
+	}
 	return c.namespaceMatchesLabelList(namespace) && !c.namespaceMatchesNoLabelList(namespace)
 }
 
