@@ -48,10 +48,12 @@ const (
 	AMWebHookSchedulerServiceAddress = WebHookPrefix + "schedulerServiceAddress"
 
 	// filtering configuration
-	AMFilteringProcessNamespaces = FilteringPrefix + "processNamespaces"
-	AMFilteringBypassNamespaces  = FilteringPrefix + "bypassNamespaces"
-	AMFilteringLabelNamespaces   = FilteringPrefix + "labelNamespaces"
-	AMFilteringNoLabelNamespaces = FilteringPrefix + "noLabelNamespaces"
+	AMFilteringProcessNamespaces    = FilteringPrefix + "processNamespaces"
+	AMFilteringBypassNamespaces     = FilteringPrefix + "bypassNamespaces"
+	AMFilteringLabelNamespaces      = FilteringPrefix + "labelNamespaces"
+	AMFilteringNoLabelNamespaces    = FilteringPrefix + "noLabelNamespaces"
+	AMFilteringGenerateUniqueAppIds = FilteringPrefix + "generateUniqueAppId"
+	AMFilteringDefaultQueueName     = FilteringPrefix + "defaultQueue"
 
 	// access control configuration
 	AMAccessControlBypassAuth       = AccessControlPrefix + "bypassAuth"
@@ -67,10 +69,12 @@ const (
 	DefaultWebHookSchedulerServiceAddress = "yunikorn-service:9080"
 
 	// filtering defaults
-	DefaultFilteringProcessNamespaces = ""
-	DefaultFilteringBypassNamespaces  = "^kube-system$"
-	DefaultFilteringLabelNamespaces   = ""
-	DefaultFilteringNoLabelNamespaces = ""
+	DefaultFilteringProcessNamespaces    = ""
+	DefaultFilteringBypassNamespaces     = "^kube-system$"
+	DefaultFilteringLabelNamespaces      = ""
+	DefaultFilteringNoLabelNamespaces    = ""
+	DefaultFilteringGenerateUniqueAppIds = false
+	DefaultFilteringQueueName            = "root.default"
 
 	// access control defaults
 	DefaultAccessControlBypassAuth       = false
@@ -93,11 +97,13 @@ type AdmissionControllerConf struct {
 	bypassNamespaces        []*regexp.Regexp
 	labelNamespaces         []*regexp.Regexp
 	noLabelNamespaces       []*regexp.Regexp
+	generateUniqueAppIds    bool
 	bypassAuth              bool
 	trustControllers        bool
 	systemUsers             []*regexp.Regexp
 	externalUsers           []*regexp.Regexp
 	externalGroups          []*regexp.Regexp
+	defaultQueueName        string
 	configMaps              []*v1.ConfigMap
 
 	lock sync.RWMutex
@@ -176,6 +182,12 @@ func (acc *AdmissionControllerConf) GetNoLabelNamespaces() []*regexp.Regexp {
 	return acc.noLabelNamespaces
 }
 
+func (acc *AdmissionControllerConf) GetGenerateUniqueAppIds() bool {
+	acc.lock.RLock()
+	defer acc.lock.RUnlock()
+	return acc.generateUniqueAppIds
+}
+
 func (acc *AdmissionControllerConf) GetBypassAuth() bool {
 	acc.lock.RLock()
 	defer acc.lock.RUnlock()
@@ -206,6 +218,12 @@ func (acc *AdmissionControllerConf) GetExternalGroups() []*regexp.Regexp {
 	return acc.externalGroups
 }
 
+func (acc *AdmissionControllerConf) GetDefaultQueueName() string {
+	acc.lock.RLock()
+	defer acc.lock.RUnlock()
+	return acc.defaultQueueName
+}
+
 type configMapUpdateHandler struct {
 	conf *AdmissionControllerConf
 }
@@ -304,6 +322,7 @@ func (acc *AdmissionControllerConf) updateConfigMaps(configMaps []*v1.ConfigMap,
 	acc.bypassNamespaces = parseConfigRegexps(configs, AMFilteringBypassNamespaces, DefaultFilteringBypassNamespaces)
 	acc.labelNamespaces = parseConfigRegexps(configs, AMFilteringLabelNamespaces, DefaultFilteringLabelNamespaces)
 	acc.noLabelNamespaces = parseConfigRegexps(configs, AMFilteringNoLabelNamespaces, DefaultFilteringNoLabelNamespaces)
+	acc.generateUniqueAppIds = parseConfigBool(configs, AMFilteringGenerateUniqueAppIds, DefaultFilteringGenerateUniqueAppIds)
 
 	// access control
 	acc.bypassAuth = parseConfigBool(configs, AMAccessControlBypassAuth, DefaultAccessControlBypassAuth)
@@ -312,6 +331,9 @@ func (acc *AdmissionControllerConf) updateConfigMaps(configMaps []*v1.ConfigMap,
 	acc.externalUsers = parseConfigRegexps(configs, AMAccessControlExternalUsers, DefaultAccessControlExternalUsers)
 	acc.externalGroups = parseConfigRegexps(configs, AMAccessControlExternalGroups, DefaultAccessControlExternalGroups)
 
+	// labeling
+	acc.defaultQueueName = parseConfigString(configs, AMFilteringDefaultQueueName, DefaultFilteringQueueName)
+
 	acc.dumpConfigurationInternal()
 }
 
@@ -351,8 +373,8 @@ func parseConfigRegexps(config map[string]string, key string, defaultValue strin
 	value := parseConfigString(config, key, defaultValue)
 	result, err := parseRegexes(value)
 	if err != nil {
-		log.Logger().Error(fmt.Sprintf("Unable to parse regex values '%s' for configuration '%s', using default value '%s'",
-			value, key, defaultValue), zap.Error(err))
+		log.Logger().Error("Unable to parse regex values, using default",
+			zap.String("key", key), zap.String("value", value), zap.String("default", defaultValue), zap.Error(err))
 		result, err = parseRegexes(defaultValue)
 		if err != nil {
 			log.Logger().Fatal("BUG: can't parse default regex pattern", zap.Error(err))
@@ -365,8 +387,8 @@ func parseConfigBool(config map[string]string, key string, defaultValue bool) bo
 	value := parseConfigString(config, key, fmt.Sprintf("%t", defaultValue))
 	result, err := strconv.ParseBool(value)
 	if err != nil {
-		log.Logger().Error(fmt.Sprintf("Unable to parse bool value '%s' for configuration '%s', using default value '%t'",
-			value, key, defaultValue), zap.Error(err))
+		log.Logger().Error("Unable to parse bool value, using default",
+			zap.String("key", key), zap.String("value", value), zap.Bool("default", defaultValue), zap.Error(err))
 		result = defaultValue
 	}
 	return result
@@ -376,8 +398,8 @@ func parseConfigInt(config map[string]string, key string, defaultValue int) int
 	value := parseConfigString(config, key, fmt.Sprintf("%d", defaultValue))
 	result, err := strconv.ParseInt(value, 10, 31)
 	if err != nil {
-		log.Logger().Error(fmt.Sprintf("Unable to parse int value '%s' for configuration '%s', using default value '%d'",
-			value, key, defaultValue), zap.Error(err))
+		log.Logger().Error("Unable to parse int value, using default",
+			zap.String("key", key), zap.String("value", value), zap.Int("default", defaultValue), zap.Error(err))
 		return defaultValue
 	}
 	return int(result)
