@@ -19,7 +19,6 @@
 package general
 
 import (
-	"reflect"
 	"strconv"
 
 	v1 "k8s.io/api/core/v1"
@@ -104,15 +103,13 @@ func isStateAwareDisabled(pod *v1.Pod) bool {
 	return result
 }
 
-func getOwnerReferences(pod *v1.Pod) []metav1.OwnerReference {
-	if len(pod.OwnerReferences) > 0 {
-		return pod.OwnerReferences
-	}
+func getOwnerReference(pod *v1.Pod) []metav1.OwnerReference {
+	// Just return the originator pod as the owner of placeholder pods
 	controller := false
 	blockOwnerDeletion := true
 	ref := metav1.OwnerReference{
-		APIVersion:         v1.SchemeGroupVersion.String(),
-		Kind:               reflect.TypeOf(v1.Pod{}).Name(),
+		APIVersion:         "v1",
+		Kind:               "Pod",
 		Name:               pod.Name,
 		UID:                pod.UID,
 		Controller:         &controller,
@@ -126,13 +123,7 @@ func (os *Manager) filterPods(obj interface{}) bool {
 	switch obj.(type) {
 	case *v1.Pod:
 		pod := obj.(*v1.Pod)
-		if utils.GeneralPodFilter(pod) {
-			// only application ID is required
-			if _, err := utils.GetApplicationIDFromPod(pod); err == nil {
-				return true
-			}
-		}
-		return false
+		return utils.GetApplicationIDFromPod(pod) != ""
 	default:
 		return false
 	}
@@ -236,7 +227,7 @@ func (os *Manager) ListPods() ([]*v1.Pod, error) {
 		log.Logger().Debug("Looking at pod for recovery candidates", zap.String("podNamespace", pod.Namespace), zap.String("podName", pod.Name))
 		// general filter passes, and pod is assigned
 		// this means the pod is already scheduled by scheduler for an existing app
-		if utils.GeneralPodFilter(pod) && utils.IsAssignedPod(pod) {
+		if utils.GetApplicationIDFromPod(pod) != "" && utils.IsAssignedPod(pod) {
 			if meta, ok := getAppMetadata(pod, true); ok {
 				podsRecovered++
 				pods = append(pods, pod)
