@@ -21,7 +21,7 @@ package general
 import (
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 	v1 "k8s.io/api/core/v1"
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 
@@ -167,7 +167,9 @@ func TestOriginatorPod(t *testing.T) {
 	owner := apis.OwnerReference{
 		APIVersion: "v1",
 		UID:        "UID-POD-00002",
+		Kind:       "Pod",
 	}
+
 	refer := []apis.OwnerReference{
 		owner,
 	}
@@ -208,10 +210,12 @@ func TestOriginatorPod(t *testing.T) {
 	}
 	am.AddPod(&pod1)
 	assert.Equal(t, len(app.GetNewTasks()), 2)
-	task, err = app.GetTask("UID-POD-00002")
+	task, err = app.GetTask("UID-POD-00001")
 	assert.Assert(t, err == nil)
 
-	// app originator task should be pod 2
+	// app originator task should be pod 1
+	// even the pod 2 is the ownerreference for pod 1
+	// And pod 1 is first added to the AM service
 	assert.Equal(t, app.GetOriginatingTask().GetTaskID(), task.GetTaskID())
 }
 
@@ -448,11 +452,15 @@ func TestGetOwnerReferences(t *testing.T) {
 			UID:  "uid",
 		},
 	}
-	returnedOwnerRefs := getOwnerReferences(podWithOwnerRef)
-	assert.Assert(t, len(returnedOwnerRefs) == 1, "Only one owner reference is expected")
-	assert.DeepEqual(t, ownerRef, returnedOwnerRefs[0])
 
-	returnedOwnerRefs = getOwnerReferences(podWithNoOwnerRef)
+	returnedOwnerRefs := getOwnerReference(podWithOwnerRef)
+	assert.Assert(t, len(returnedOwnerRefs) == 1, "Only one owner reference is expected")
+	assert.Equal(t, returnedOwnerRefs[0].Name, podWithOwnerRef.Name, "Unexpected owner reference name")
+	assert.Equal(t, returnedOwnerRefs[0].UID, podWithOwnerRef.UID, "Unexpected owner reference UID")
+	assert.Equal(t, returnedOwnerRefs[0].Kind, "Pod", "Unexpected owner reference Kind")
+	assert.Equal(t, returnedOwnerRefs[0].APIVersion, v1.SchemeGroupVersion.String(), "Unexpected owner reference Kind")
+
+	returnedOwnerRefs = getOwnerReference(podWithNoOwnerRef)
 	assert.Assert(t, len(returnedOwnerRefs) == 1, "Only one owner reference is expected")
 	assert.Equal(t, returnedOwnerRefs[0].Name, podWithNoOwnerRef.Name, "Unexpected owner reference name")
 	assert.Equal(t, returnedOwnerRefs[0].UID, podWithNoOwnerRef.UID, "Unexpected owner reference UID")
@@ -609,8 +617,7 @@ func TestListApplication(t *testing.T) {
 	assert.NilError(t, err)
 	assert.Equal(t, len(pods), 3)
 	for _, pod := range pods {
-		name, err := utils.GetApplicationIDFromPod(pod)
-		assert.NilError(t, err, "Could not retrieve application id from pod")
+		name := utils.GetApplicationIDFromPod(pod)
 		expected := expectOutput[name]
 		description := descriptionMap[name]
 		assert.Assert(t, expected, description)
