@@ -27,8 +27,8 @@ import (
 	"testing"
 	"time"
 
-	"gotest.tools/assert"
-	is "gotest.tools/assert/cmp"
+	"gotest.tools/v3/assert"
+	is "gotest.tools/v3/assert/cmp"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -671,7 +671,8 @@ func TestSetTaskGroupsAndSchedulingPolicy(t *testing.T) {
 	// TG2: 1000Mi, 20 members -> total 20000Mi
 	// overall usage 5000Mi + 20000Mi = 25000Mi. This will also be the queue usage so the correct handling
 	// CPU is normal as it specifies milli cpu to start with
-	expectedPlaceholderAsk := common.NewResourceBuilder().AddResource(siCommon.Memory, 25000*1024*1024).AddResource(siCommon.CPU, 25000).Build()
+	// Pods should always be equal to the sum of the MinMember values
+	expectedPlaceholderAsk := common.NewResourceBuilder().AddResource("pods", 30).AddResource(siCommon.Memory, 25000*1024*1024).AddResource(siCommon.CPU, 25000).Build()
 	actualPlaceholderAsk := app.getPlaceholderAsk()
 	assert.DeepEqual(t, actualPlaceholderAsk, expectedPlaceholderAsk)
 }
@@ -848,6 +849,10 @@ func TestTryReservePostRestart(t *testing.T) {
 			Containers: containers,
 		},
 	})
+	task0.allocationUUID = string(task0.pod.UID)
+	task0.nodeName = "fake-host"
+	task0.sm.SetState(TaskStates().Allocated)
+
 	task1 := NewTask("task01", app, context, &v1.Pod{
 		TypeMeta: apis.TypeMeta{
 			Kind:       "Pod",
@@ -861,6 +866,7 @@ func TestTryReservePostRestart(t *testing.T) {
 			Containers: containers,
 		},
 	})
+
 	task2 := NewTask("task02", app, context, &v1.Pod{
 		TypeMeta: apis.TypeMeta{
 			Kind:       "Pod",
@@ -874,7 +880,7 @@ func TestTryReservePostRestart(t *testing.T) {
 			Containers: containers,
 		},
 	})
-	task0.setAllocated("fake-host", string(task0.pod.UID))
+
 	app.addTask(task0)
 	app.addTask(task1)
 	app.addTask(task2)
