@@ -61,8 +61,14 @@ type Context struct {
 	lock           *sync.RWMutex                  // lock
 }
 
-// Create a new context for the scheduler.
+// NewContext create a new context for the scheduler using a default (empty) configuration
+// VisibleForTesting
 func NewContext(apis client.APIProvider) *Context {
+	return NewContextWithBootstrapConfigMaps(apis, []*v1.ConfigMap{nil, nil})
+}
+
+// NewContextWithBootstrapConfigMaps creates a new context for the scheduler using configuration bootstrapped from Kubernetes ConfigMaps
+func NewContextWithBootstrapConfigMaps(apis client.APIProvider, bootstrapConfigMaps []*v1.ConfigMap) *Context {
 	// create the context note that order is important:
 	// volumebinder needs the informers
 	// the cache needs informers and volumebinder
@@ -72,7 +78,7 @@ func NewContext(apis client.APIProvider) *Context {
 		applications: make(map[string]*Application),
 		apiProvider:  apis,
 		namespace:    apis.GetAPIs().GetConf().Namespace,
-		configMaps:   []*v1.ConfigMap{nil, nil},
+		configMaps:   bootstrapConfigMaps,
 		lock:         &sync.RWMutex{},
 	}
 
@@ -83,12 +89,10 @@ func NewContext(apis client.APIProvider) *Context {
 	ctx.nodes = newSchedulerNodes(apis.GetAPIs().SchedulerAPI, ctx.schedulerCache)
 
 	// create the predicate manager
-	if !apis.IsTestingMode() {
-		sharedLister := support.NewSharedLister(ctx.schedulerCache)
-		clientSet := apis.GetAPIs().KubeClient.GetClientSet()
-		informerFactory := apis.GetAPIs().InformerFactory
-		ctx.predManager = predicates.NewPredicateManager(support.NewFrameworkHandle(sharedLister, informerFactory, clientSet))
-	}
+	sharedLister := support.NewSharedLister(ctx.schedulerCache)
+	clientSet := apis.GetAPIs().KubeClient.GetClientSet()
+	informerFactory := apis.GetAPIs().InformerFactory
+	ctx.predManager = predicates.NewPredicateManager(support.NewFrameworkHandle(sharedLister, informerFactory, clientSet))
 
 	return ctx
 }
@@ -277,11 +281,7 @@ func (ctx *Context) updatePodInCache(oldObj, newObj interface{}) {
 func (ctx *Context) filterPods(obj interface{}) bool {
 	switch obj := obj.(type) {
 	case *v1.Pod:
-		if utils.GeneralPodFilter(obj) {
-			_, err := utils.GetApplicationIDFromPod(obj)
-			return err == nil
-		}
-		return false
+		return utils.GetApplicationIDFromPod(obj) != ""
 	default:
 		return false
 	}
@@ -305,14 +305,13 @@ func (ctx *Context) addConfigMaps(obj interface{}) {
 	configmap := utils.Convert2ConfigMap(obj)
 	switch configmap.Name {
 	case constants.DefaultConfigMapName:
-		ctx.configMaps[0] = configmap
+		ctx.triggerReloadConfig(0, configmap)
 	case constants.ConfigMapName:
-		ctx.configMaps[1] = configmap
+		ctx.triggerReloadConfig(1, configmap)
 	default:
 		// ignore
 		return
 	}
-	ctx.triggerReloadConfig()
 }
 
 // when the configMap for the scheduler is updated, trigger hot-refresh
@@ -321,14 +320,13 @@ func (ctx *Context) updateConfigMaps(_, newObj interface{}) {
 	configmap := utils.Convert2ConfigMap(newObj)
 	switch configmap.Name {
 	case constants.DefaultConfigMapName:
-		ctx.configMaps[0] = configmap
+		ctx.triggerReloadConfig(0, configmap)
 	case constants.ConfigMapName:
-		ctx.configMaps[1] = configmap
+		ctx.triggerReloadConfig(1, configmap)
 	default:
 		// ignore
 		return
 	}
-	ctx.triggerReloadConfig()
 }
 
 // when the configMap for the scheduler is deleted, trigger refresh using default config
@@ -347,14 +345,13 @@ func (ctx *Context) deleteConfigMaps(obj interface{}) {
 
 	switch configmap.Name {
 	case constants.DefaultConfigMapName:
-		ctx.configMaps[0] = nil
+		ctx.triggerReloadConfig(0, nil)
 	case constants.ConfigMapName:
-		ctx.configMaps[1] = nil
+		ctx.triggerReloadConfig(1, nil)
 	default:
 		// ignore
 		return
 	}
-	ctx.triggerReloadConfig()
 }
 
 func (ctx *Context) filterPriorityClasses(obj interface{}) bool {
@@ -401,13 +398,17 @@ func (ctx *Context) deletePriorityClass(obj interface{}) {
 	}
 }
 
-func (ctx *Context) triggerReloadConfig() {
+func (ctx *Context) triggerReloadConfig(index int, configMap *v1.ConfigMap) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
 	conf := ctx.apiProvider.GetAPIs().GetConf()
 	if !conf.EnableConfigHotRefresh {
 		log.Logger().Info("hot-refresh disabled, skipping scheduler configuration update")
 		return
 	}
 
+	ctx.configMaps[index] = configMap
 	err := schedulerconf.UpdateConfigMaps(ctx.configMaps, false)
 	if err != nil {
 		log.Logger().Error("Unable to update configmap, ignoring changes", zap.Error(err))
@@ -434,11 +435,6 @@ func (ctx *Context) triggerReloadConfig() {
 
 // evaluate given predicates based on current context
 func (ctx *Context) IsPodFitNode(name, node string, allocate bool) error {
-	// simply skip if predicates are not enabled
-	if ctx.apiProvider.IsTestingMode() {
-		return nil
-	}
-
 	ctx.lock.RLock()
 	defer ctx.lock.RUnlock()
 	if pod, ok := ctx.schedulerCache.GetPod(name); ok {
@@ -454,6 +450,41 @@ func (ctx *Context) IsPodFitNode(name, node string, allocate bool) error {
 	return fmt.Errorf("predicates were not running because pod or node was not found in cache")
 }
 
+func (ctx *Context) IsPodFitNodeViaPreemption(name, node string, allocations []string, startIndex int) (index int, ok bool) {
+	// assume minimal pods need killing if running in testing mode
+	if ctx.apiProvider.IsTestingMode() {
+		return startIndex, ok
+	}
+
+	ctx.lock.RLock()
+	defer ctx.lock.RUnlock()
+	if pod, ok := ctx.schedulerCache.GetPod(name); ok {
+		// if pod exists in cache, try to run predicates
+		if targetNode := ctx.schedulerCache.GetNode(node); targetNode != nil {
+			// need to lock cache here as predicates need a stable view into the cache
+			ctx.schedulerCache.LockForReads()
+			defer ctx.schedulerCache.UnlockForReads()
+
+			// look up each victim in the scheduler cache
+			victims := make([]*v1.Pod, 0)
+			for _, uid := range allocations {
+				if victim, ok := ctx.schedulerCache.GetPod(uid); ok {
+					victims = append(victims, victim)
+				} else {
+					// if pod isn't found, add a placeholder so that the list is still the same size
+					victims = append(victims, nil)
+				}
+			}
+
+			// check predicates for a match
+			if index, ok := ctx.predManager.PreemptionPredicates(pod, targetNode, victims, startIndex); ok {
+				return index, ok
+			}
+		}
+	}
+	return -1, false
+}
+
 // call volume binder to bind pod volumes if necessary,
 // internally, volume binder maintains a cache (podBindingCache) for pod volumes,
 // and before calling this, they should have been updated by FindPodVolumes and AssumePodVolumes.
@@ -887,15 +918,12 @@ func (ctx *Context) getTask(appID string, taskID string) *Task {
 	return task
 }
 
-func (ctx *Context) SelectApplications(filter func(app *Application) bool) []*Application {
+func (ctx *Context) GetAllApplications() []*Application {
 	ctx.lock.RLock()
 	defer ctx.lock.RUnlock()
 
-	apps := make([]*Application, 0)
+	apps := make([]*Application, 0, len(ctx.applications))
 	for _, app := range ctx.applications {
-		if filter != nil && !filter(app) {
-			continue
-		}
 		apps = append(apps, app)
 	}
 
@@ -916,7 +944,7 @@ func (ctx *Context) PublishEvents(eventRecords []*si.EventRecord) {
 					log.Logger().Warn("task event is not published because task is not found",
 						zap.String("appID", appID),
 						zap.String("taskID", taskID),
-						zap.String("event", record.String()))
+						zap.Stringer("event", record))
 				}
 			case si.EventRecord_NODE:
 				nodeID := record.ObjectID
@@ -924,21 +952,21 @@ func (ctx *Context) PublishEvents(eventRecords []*si.EventRecord) {
 				if nodeInfo == nil {
 					log.Logger().Warn("node event is not published because nodeInfo is not found",
 						zap.String("nodeID", nodeID),
-						zap.String("event", record.String()))
+						zap.Stringer("event", record))
 					continue
 				}
 				node := nodeInfo.Node()
 				if node == nil {
 					log.Logger().Warn("node event is not published because node is not found",
 						zap.String("nodeID", nodeID),
-						zap.String("event", record.String()))
+						zap.Stringer("event", record))
 					continue
 				}
 				events.GetRecorder().Eventf(node.DeepCopy(), nil,
 					v1.EventTypeNormal, record.Reason, record.Reason, record.Message)
 			default:
 				log.Logger().Warn("Unsupported event type, currently only supports to publish request event records",
-					zap.String("type", record.Type.String()))
+					zap.Stringer("type", record.Type))
 			}
 		}
 	}
@@ -1009,7 +1037,7 @@ func (ctx *Context) HandleContainerStateUpdate(request *si.UpdateContainerSchedu
 			}
 		default:
 			log.Logger().Warn("no handler for container scheduling state",
-				zap.String("state", request.State.String()))
+				zap.Stringer("state", request.State))
 		}
 	}
 }
@@ -1095,3 +1123,8 @@ func (ctx *Context) GetStateDump() (string, error) {
 	}
 	return string(bytes), nil
 }
+
+// VisibleForTesting
+func (ctx *Context) GetSchedulerCache() *schedulercache.SchedulerCache {
+	return ctx.schedulerCache
+}
