@@ -25,9 +25,10 @@ import (
 	"testing"
 	"time"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 	v1 "k8s.io/api/core/v1"
 	schedulingv1 "k8s.io/api/scheduling/v1"
+	"k8s.io/apimachinery/pkg/api/resource"
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/tools/cache"
@@ -63,7 +64,7 @@ func initContextAndAPIProviderForTest() (*Context, *client.MockedAPIProvider) {
 	return context, apis
 }
 
-func newPodHelper(name, namespace, podUID, nodeName string, podPhase v1.PodPhase) *v1.Pod {
+func newPodHelper(name, namespace, podUID, nodeName string, appID string, podPhase v1.PodPhase) *v1.Pod {
 	return &v1.Pod{
 		TypeMeta: apis.TypeMeta{
 			Kind:       "Pod",
@@ -73,6 +74,7 @@ func newPodHelper(name, namespace, podUID, nodeName string, podPhase v1.PodPhase
 			Name:      name,
 			Namespace: namespace,
 			UID:       types.UID(podUID),
+			Labels:    map[string]string{constants.LabelApplicationID: appID},
 		},
 		Spec: v1.PodSpec{
 			NodeName:      nodeName,
@@ -84,6 +86,88 @@ func newPodHelper(name, namespace, podUID, nodeName string, podPhase v1.PodPhase
 	}
 }
 
+func TestAddNodes(t *testing.T) {
+	ctx := initContextForTest()
+
+	node := v1.Node{
+		ObjectMeta: apis.ObjectMeta{
+			Name:      "host0001",
+			Namespace: "default",
+			UID:       "uid_0001",
+		},
+	}
+
+	ctx.addNode(&node)
+	assert.Equal(t, true, ctx.schedulerCache.GetNode("host0001") != nil)
+	assert.Equal(t, true, ctx.nodes.getNode("host0001") != nil)
+}
+
+func TestUpdateNodes(t *testing.T) {
+	ctx := initContextForTest()
+
+	oldNodeResource := make(map[v1.ResourceName]resource.Quantity)
+	oldNodeResource[v1.ResourceName("memory")] = *resource.NewQuantity(1024*1000*1000, resource.DecimalSI)
+	oldNodeResource[v1.ResourceName("cpu")] = *resource.NewQuantity(2, resource.DecimalSI)
+	oldNode := v1.Node{
+		ObjectMeta: apis.ObjectMeta{
+			Name:      "host0001",
+			Namespace: "default",
+			UID:       "uid_0001",
+		},
+		Status: v1.NodeStatus{
+			Allocatable: oldNodeResource,
+		},
+	}
+
+	newNodeResource := make(map[v1.ResourceName]resource.Quantity)
+	newNodeResource[v1.ResourceName("memory")] = *resource.NewQuantity(2048*1000*1000, resource.DecimalSI)
+	newNodeResource[v1.ResourceName("cpu")] = *resource.NewQuantity(4, resource.DecimalSI)
+	newNode := v1.Node{
+		ObjectMeta: apis.ObjectMeta{
+			Name:      "host0001",
+			Namespace: "default",
+			UID:       "uid_0001",
+		},
+		Status: v1.NodeStatus{
+			Allocatable: newNodeResource,
+		},
+	}
+
+	ctx.addNode(&oldNode)
+	ctx.updateNode(&oldNode, &newNode)
+
+	assert.Equal(t, int64(2048*1000*1000), ctx.nodes.getNode("host0001").capacity.Resources[siCommon.Memory].Value)
+	assert.Equal(t, int64(4000), ctx.nodes.getNode("host0001").capacity.Resources[siCommon.CPU].Value)
+}
+
+func TestDeleteNodes(t *testing.T) {
+	ctx := initContextForTest()
+
+	node := v1.Node{
+		ObjectMeta: apis.ObjectMeta{
+			Name:      "host0001",
+			Namespace: "default",
+			UID:       "uid_0001",
+		},
+	}
+
+	ctx.addNode(&node)
+	assert.Equal(t, true, ctx.schedulerCache.GetNode("host0001") != nil)
+	assert.Equal(t, true, ctx.nodes.getNode("host0001") != nil)
+
+	ctx.deleteNode(&node)
+	assert.Equal(t, true, ctx.schedulerCache.GetNode("host0001") == nil)
+	assert.Equal(t, true, ctx.nodes.getNode("host0001") == nil)
+
+	ctx.addNode(&node)
+	assert.Equal(t, true, ctx.schedulerCache.GetNode("host0001") != nil)
+	assert.Equal(t, true, ctx.nodes.getNode("host0001") != nil)
+
+	ctx.deleteNode(cache.DeletedFinalStateUnknown{Key: "UID-00001", Obj: &node})
+	assert.Equal(t, true, ctx.schedulerCache.GetNode("host0001") == nil)
+	assert.Equal(t, true, ctx.nodes.getNode("host0001") == nil)
+}
+
 func TestAddApplications(t *testing.T) {
 	context := initContextForTest()
 
@@ -539,12 +623,12 @@ func TestRecoverTask(t *testing.T) {
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: appID,
 			TaskID:        taskUID1,
-			Pod:           newPodHelper("pod1", podNamespace, taskUID1, fakeNodeName, v1.PodRunning),
+			Pod:           newPodHelper("pod1", podNamespace, taskUID1, fakeNodeName, appID, v1.PodRunning),
 		},
 	})
 	assert.Assert(t, task != nil)
 	assert.Equal(t, task.GetTaskID(), taskUID1)
-	assert.Equal(t, task.GetTaskState(), TaskStates().Allocated)
+	assert.Equal(t, task.GetTaskState(), TaskStates().Bound)
 
 	// add a tasks to the existing application
 	// this task was already completed with state: Succeed
@@ -552,7 +636,7 @@ func TestRecoverTask(t *testing.T) {
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: appID,
 			TaskID:        taskUID2,
-			Pod:           newPodHelper("pod2", podNamespace, taskUID2, fakeNodeName, v1.PodSucceeded),
+			Pod:           newPodHelper("pod2", podNamespace, taskUID2, fakeNodeName, appID, v1.PodSucceeded),
 		},
 	})
 	assert.Assert(t, task != nil)
@@ -565,7 +649,7 @@ func TestRecoverTask(t *testing.T) {
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: appID,
 			TaskID:        taskUID3,
-			Pod:           newPodHelper("pod3", podNamespace, taskUID3, fakeNodeName, v1.PodFailed),
+			Pod:           newPodHelper("pod3", podNamespace, taskUID3, fakeNodeName, appID, v1.PodFailed),
 		},
 	})
 	assert.Assert(t, task != nil)
@@ -578,7 +662,7 @@ func TestRecoverTask(t *testing.T) {
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: appID,
 			TaskID:        taskUID4,
-			Pod:           newPodHelper("pod4", podNamespace, taskUID4, "", v1.PodPending),
+			Pod:           newPodHelper("pod4", podNamespace, taskUID4, "", appID, v1.PodPending),
 		},
 	})
 	assert.Assert(t, task != nil)
@@ -588,7 +672,7 @@ func TestRecoverTask(t *testing.T) {
 	// make sure the recovered task is added to the app
 	app, exist := context.applications[appID]
 	assert.Equal(t, exist, true)
-	assert.Equal(t, len(app.getTasks(TaskStates().Allocated)), 1)
+	assert.Equal(t, len(app.getTasks(TaskStates().Bound)), 1)
 	assert.Equal(t, len(app.getTasks(TaskStates().Completed)), 2)
 	assert.Equal(t, len(app.getTasks(TaskStates().New)), 1)
 
@@ -599,7 +683,7 @@ func TestRecoverTask(t *testing.T) {
 		expectedPodName        string
 		expectedNodeName       string
 	}{
-		{taskUID1, TaskStates().Allocated, taskUID1, "pod1", fakeNodeName},
+		{taskUID1, TaskStates().Bound, taskUID1, "pod1", fakeNodeName},
 		{taskUID2, TaskStates().Completed, taskUID2, "pod2", fakeNodeName},
 		{taskUID3, TaskStates().Completed, taskUID3, "pod3", fakeNodeName},
 		{taskUID4, TaskStates().New, "", "pod4", ""},
@@ -655,30 +739,30 @@ func TestTaskReleaseAfterRecovery(t *testing.T) {
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: appID,
 			TaskID:        pod1UID,
-			Pod:           newPodHelper(pod1Name, namespace, pod1UID, fakeNodeName, v1.PodRunning),
+			Pod:           newPodHelper(pod1Name, namespace, pod1UID, fakeNodeName, appID, v1.PodRunning),
 		},
 	})
 
 	assert.Assert(t, task0 != nil)
 	assert.Equal(t, task0.GetTaskID(), pod1UID)
-	assert.Equal(t, task0.GetTaskState(), TaskStates().Allocated)
+	assert.Equal(t, task0.GetTaskState(), TaskStates().Bound)
 
 	task1 := context.AddTask(&interfaces.AddTaskRequest{
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: appID,
 			TaskID:        pod2UID,
-			Pod:           newPodHelper(pod2Name, namespace, pod2UID, fakeNodeName, v1.PodRunning),
+			Pod:           newPodHelper(pod2Name, namespace, pod2UID, fakeNodeName, appID, v1.PodRunning),
 		},
 	})
 
 	assert.Assert(t, task1 != nil)
 	assert.Equal(t, task1.GetTaskID(), pod2UID)
-	assert.Equal(t, task1.GetTaskState(), TaskStates().Allocated)
+	assert.Equal(t, task1.GetTaskState(), TaskStates().Bound)
 
 	// app should have 2 tasks recovered
 	app, exist := context.applications[appID]
 	assert.Equal(t, exist, true)
-	assert.Equal(t, len(app.GetAllocatedTasks()), 2)
+	assert.Equal(t, len(app.GetBoundTasks()), 2)
 
 	// release one of the tasks
 	context.NotifyTaskComplete(appID, pod2UID)
@@ -697,7 +781,7 @@ func TestTaskReleaseAfterRecovery(t *testing.T) {
 	// expect to see:
 	//  - task0 is still there
 	//  - task1 gets released
-	assert.Equal(t, t0.GetTaskState(), TaskStates().Allocated)
+	assert.Equal(t, t0.GetTaskState(), TaskStates().Bound)
 	assert.Equal(t, t1.GetTaskState(), TaskStates().Completed)
 }
 
