@@ -19,6 +19,7 @@
 package cache
 
 import (
+	"context"
 	"sync"
 
 	"github.com/looplab/fsm"
@@ -36,7 +37,7 @@ import (
 type SchedulerNode struct {
 	name         string
 	uid          string
-	labels       string
+	labels       map[string]string
 	schedulable  bool
 	schedulerAPI api.SchedulerAPI
 	fsm          *fsm.FSM
@@ -50,7 +51,7 @@ type SchedulerNode struct {
 	lock *sync.RWMutex
 }
 
-func newSchedulerNode(nodeName string, nodeUID string, nodeLabels string,
+func newSchedulerNode(nodeName string, nodeUID string, nodeLabels map[string]string,
 	nodeResource *si.Resource, schedulerAPI api.SchedulerAPI, schedulable bool, ready bool) *SchedulerNode {
 	schedulerNode := &SchedulerNode{
 		name:         nodeName,
@@ -89,12 +90,12 @@ func (n *SchedulerNode) updateOccupiedResource(resource *si.Resource, opt update
 	case AddOccupiedResource:
 		log.Logger().Info("add node occupied resource",
 			zap.String("nodeID", n.name),
-			zap.String("occupied", resource.String()))
+			zap.Stringer("occupied", resource))
 		n.occupied = common.Add(n.occupied, resource)
 	case SubOccupiedResource:
 		log.Logger().Info("subtract node occupied resource",
 			zap.String("nodeID", n.name),
-			zap.String("occupied", resource.String()))
+			zap.Stringer("occupied", resource))
 		n.occupied = common.Sub(n.occupied, resource)
 	default:
 		// noop
@@ -107,7 +108,7 @@ func (n *SchedulerNode) setCapacity(capacity *si.Resource) {
 	defer n.lock.Unlock()
 	log.Logger().Debug("set node capacity",
 		zap.String("nodeID", n.name),
-		zap.String("capacity", capacity.String()))
+		zap.Stringer("capacity", capacity))
 	n.capacity = capacity
 }
 
@@ -186,7 +187,7 @@ func (n *SchedulerNode) handleRestoreNode() {
 func (n *SchedulerNode) handle(ev events.SchedulerNodeEvent) error {
 	n.lock.Lock()
 	defer n.lock.Unlock()
-	err := n.fsm.Event(ev.GetEvent(), n)
+	err := n.fsm.Event(context.Background(), ev.GetEvent(), n)
 	// handle the same state transition not nil error (limit of fsm).
 	if err != nil && err.Error() != "no transition" {
 		return err
