@@ -23,8 +23,8 @@ import (
 	"testing"
 	"time"
 
-	"gotest.tools/assert"
-	is "gotest.tools/assert/cmp"
+	"gotest.tools/v3/assert"
+	is "gotest.tools/v3/assert/cmp"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -118,7 +118,6 @@ func TestCreateAppPlaceholdersWithOwnReference(t *testing.T) {
 	pods := createAndCheckPlaceholderCreate(mockedAPIProvider, app, t)
 	for _, pod := range pods {
 		assert.Assert(t, len(pod.OwnerReferences) == 1, "The pod should have exactly one owner reference set")
-		assert.Assert(t, *pod.OwnerReferences[0].Controller == false, "The owner reference should not be a controller")
 		assert.Equal(t, pod.OwnerReferences[0].Name, ownRef.Name, "The owner reference name does not match")
 		assert.Equal(t, pod.OwnerReferences[0].UID, ownRef.UID, "The owner reference UID does not match")
 	}
