@@ -26,7 +26,7 @@ import (
 	"testing"
 
 	"github.com/looplab/fsm"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 	v1 "k8s.io/api/core/v1"
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
