@@ -40,6 +40,8 @@ import (
 
 type Type int
 
+var informerTypes = [...]string{"Pod", "Node", "ConfigMap", "Storage", "PV", "PVC", "Application", "PriorityClass"}
+
 const (
 	PodInformerHandlers Type = iota
 	NodeInformerHandlers
@@ -51,12 +53,16 @@ const (
 	PriorityClassInformerHandlers
 )
 
+func (t Type) String() string {
+	return informerTypes[t]
+}
+
 type APIProvider interface {
 	GetAPIs() *Clients
 	AddEventHandler(handlers *ResourceEventHandlers)
 	Start()
 	Stop()
-	WaitForSync() error
+	WaitForSync()
 	IsTestingMode() bool
 }
 
@@ -176,6 +182,7 @@ func (s *APIFactory) AddEventHandler(handlers *ResourceEventHandlers) {
 		h = fns
 	}
 
+	log.Logger().Info("registering event handler", zap.Stringer("type", handlers.Type))
 	s.addEventHandlers(handlers.Type, h, 0)
 }
 
@@ -209,22 +216,19 @@ func (s *APIFactory) addEventHandlers(
 	}
 }
 
-func (s *APIFactory) WaitForSync() error {
+func (s *APIFactory) WaitForSync() {
 	if s.testMode {
 		// skip this in test mode
-		return nil
+		return
 	}
-	return s.clients.WaitForSync(time.Second, 30*time.Second)
+	s.clients.WaitForSync()
 }
 
 func (s *APIFactory) Start() {
 	// launch clients
 	if !s.IsTestingMode() {
 		s.clients.Run(s.stopChan)
-		if err := s.clients.WaitForSync(time.Second, 30*time.Second); err != nil {
-			log.Logger().Warn("Failed to sync informers",
-				zap.Error(err))
-		}
+		s.clients.WaitForSync()
 	}
 }
 
