@@ -28,14 +28,24 @@ import (
 )
 
 var (
-	version string
-	date    string
+	version   string
+	date      string
+	goVersion string
+	arch      string
+	coreSHA   string
+	siSHA     string
+	shimSHA   string
 )
 
 func main() {
 	conf.BuildVersion = version
 	conf.BuildDate = date
 	conf.IsPluginVersion = true
+	conf.GoVersion = goVersion
+	conf.Arch = arch
+	conf.CoreSHA = coreSHA
+	conf.SiSHA = siSHA
+	conf.ShimSHA = shimSHA
 
 	command := app.NewSchedulerCommand(
 		app.WithPlugin(schedulerplugin.SchedulerPluginName, schedulerplugin.NewSchedulerPlugin))
