@@ -19,6 +19,7 @@
 package main
 
 import (
+	"fmt"
 	"os"
 	"os/signal"
 	"syscall"
@@ -35,16 +36,26 @@ import (
 )
 
 var (
-	version string
-	date    string
+	version   string
+	date      string
+	goVersion string
+	arch      string
+	coreSHA   string
+	siSHA     string
+	shimSHA   string
 )
 
 func main() {
 	conf.BuildVersion = version
 	conf.BuildDate = date
 	conf.IsPluginVersion = false
+	conf.GoVersion = goVersion
+	conf.Arch = arch
+	conf.CoreSHA = coreSHA
+	conf.SiSHA = siSHA
+	conf.ShimSHA = shimSHA
 
-	log.Logger().Info("Build info", zap.String("version", version), zap.String("date", date))
+	log.Logger().Info(fmt.Sprintf("Build info: version=%s date=%s isPluginVersion=%t goVersion=%s arch=%s coreSHA=%s siSHA=%s shimSHA=%s", version, date, false, goVersion, arch, coreSHA, siSHA, shimSHA))
 
 	configMaps, err := client.LoadBootstrapConfigMaps(conf.GetSchedulerNamespace())
 	if err != nil {
@@ -60,7 +71,7 @@ func main() {
 	serviceContext := entrypoint.StartAllServicesWithLogger(log.Logger(), log.GetZapConfigs())
 
 	if sa, ok := serviceContext.RMProxy.(api.SchedulerAPI); ok {
-		ss := shim.NewShimScheduler(sa, conf.GetSchedulerConf())
+		ss := shim.NewShimScheduler(sa, conf.GetSchedulerConf(), configMaps)
 		ss.Run()
 
 		signalChan := make(chan os.Signal, 1)
