@@ -49,22 +49,17 @@ func (w *ResourceBuilder) Build() *si.Resource {
 	return &si.Resource{Resources: w.resourceMap}
 }
 
-// Get the resources from a pod's containers and convert that into a internal resource.
-// A pod has two resource parts: Requests and Limits.
-// Based on the values a pod gets a QOS assigned, as per
-// https://kubernetes.io/docs/tasks/configure-pod-container/quality-service-pod/
-// QOS class Guaranteed and Burstable are supported. However Burstable is scheduled based on the request
-// values, limits are ignored in the current setup.
-// BestEffort pods are scheduled using a minimum resource of 1MB only.
+// GetPodResource from a pod's containers and convert that into an internal resource.
+// Scheduling only accounts for requests.
+// Convert the pod into a resource to allow for pod count checks in quotas and nodes.
 func GetPodResource(pod *v1.Pod) (resource *si.Resource) {
-	// var memory, vcore = int64(0), int64(0)
-	var podResource *si.Resource
-	// A QosBestEffort pod does not request any resources and thus cannot be
-	// scheduled. Handle a QosBestEffort pod by setting a tiny memory value.
+	podResource := &si.Resource{
+		Resources: map[string]*si.Quantity{"pods": {Value: 1}},
+	}
+
+	// A QosBestEffort pod does not request any resources, just a single pod
 	if qos.GetPodQOS(pod) == v1.PodQOSBestEffort {
-		resources := NewResourceBuilder()
-		resources.AddResource(siCommon.Memory, 1000000)
-		return resources.Build()
+		return podResource
 	}
 
 	for _, c := range pod.Spec.Containers {
@@ -79,6 +74,22 @@ func GetPodResource(pod *v1.Pod) (resource *si.Resource) {
 		checkInitContainerRequest(pod, podResource)
 	}
 
+	// K8s pod EnableOverHead from:
+	// alpha: v1.16
+	// beta: v1.18
+	// Enables PodOverhead, for accounting pod overheads which are specific to a given RuntimeClass
+
+	// If Overhead is being utilized, add to the total requests for the pod
+	if pod.Spec.Overhead != nil {
+		podOverHeadResource := getResource(pod.Spec.Overhead)
+		podResource = Add(podResource, podOverHeadResource)
+		// Logging the overall pod size and pod overhead
+		log.Logger().Debug("Pod overhead specified, overall pod size adjusted",
+			zap.String("taskID", string(pod.UID)),
+			zap.Stringer("overallSize", podResource),
+			zap.Stringer("overheadSize", podOverHeadResource))
+	}
+
 	return podResource
 }
 
@@ -173,6 +184,7 @@ func GetResource(resMap map[string]string) *si.Resource {
 
 func GetTGResource(resMap map[string]resource.Quantity, members int64) *si.Resource {
 	result := NewResourceBuilder()
+	result.AddResource("pods", members)
 	for resName, resValue := range resMap {
 		switch resName {
 		case v1.ResourceCPU.String():
