@@ -149,7 +149,7 @@ func CreateReleaseAllocationRequestForTask(appID, allocUUID, partition, terminat
 }
 
 // CreateUpdateRequestForNewNode builds a NodeRequest for new node addition and restoring existing node
-func CreateUpdateRequestForNewNode(nodeID string, nodeLabels string, capacity *si.Resource, occupied *si.Resource,
+func CreateUpdateRequestForNewNode(nodeID string, nodeLabels map[string]string, capacity *si.Resource, occupied *si.Resource,
 	existingAllocations []*si.Allocation, ready bool) si.NodeRequest {
 	// Use node's name as the NodeID, this is because when bind pod to node,
 	// name of node is required but uid is optional.
@@ -158,15 +158,22 @@ func CreateUpdateRequestForNewNode(nodeID string, nodeLabels string, capacity *s
 		SchedulableResource: capacity,
 		OccupiedResource:    occupied,
 		Attributes: map[string]string{
-			constants.DefaultNodeAttributeHostNameKey:   nodeID,
-			constants.DefaultNodeAttributeRackNameKey:   constants.DefaultRackName,
-			constants.DefaultNodeAttributeNodeLabelsKey: nodeLabels,
-			common.NodeReadyAttribute:                   strconv.FormatBool(ready),
+			constants.DefaultNodeAttributeHostNameKey: nodeID,
+			constants.DefaultNodeAttributeRackNameKey: constants.DefaultRackName,
+			common.NodeReadyAttribute:                 strconv.FormatBool(ready),
 		},
 		ExistingAllocations: existingAllocations,
 		Action:              si.NodeInfo_CREATE,
 	}
 
+	// Add nodeLabels key value to Attributes map
+	for k, v := range nodeLabels {
+		nodeInfo.Attributes[k] = v
+	}
+
+	// Add instanceType to Attributes map
+	nodeInfo.Attributes[common.InstanceType] = nodeLabels[conf.GetSchedulerConf().InstanceTypeNodeLabelKey]
+
 	nodes := make([]*si.NodeInfo, 1)
 	nodes[0] = nodeInfo
 	request := si.NodeRequest{
