@@ -19,6 +19,7 @@
 package shim
 
 import (
+	"context"
 	"os"
 	"strconv"
 	"sync"
@@ -26,6 +27,7 @@ import (
 
 	"github.com/looplab/fsm"
 	"go.uber.org/zap"
+	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/informers"
 
@@ -61,22 +63,22 @@ var (
 	outstandingAppLogTimeout = 2 * time.Minute
 )
 
-func NewShimScheduler(scheduler api.SchedulerAPI, configs *conf.SchedulerConf) *KubernetesShim {
+func NewShimScheduler(scheduler api.SchedulerAPI, configs *conf.SchedulerConf, bootstrapConfigMaps []*v1.ConfigMap) *KubernetesShim {
 	kubeClient := client.NewKubeClient(configs.KubeConfig)
 
 	// we have disabled re-sync to keep ourselves up-to-date
 	informerFactory := informers.NewSharedInformerFactory(kubeClient.GetClientSet(), 0)
 
 	apiFactory := client.NewAPIFactory(scheduler, informerFactory, configs, false)
-	context := cache.NewContext(apiFactory)
+	context := cache.NewContextWithBootstrapConfigMaps(apiFactory, bootstrapConfigMaps)
 	rmCallback := callback.NewAsyncRMCallback(context)
 	appManager := appmgmt.NewAMService(context, apiFactory)
 	return newShimSchedulerInternal(context, apiFactory, appManager, rmCallback)
 }
 
-func NewShimSchedulerForPlugin(scheduler api.SchedulerAPI, informerFactory informers.SharedInformerFactory, configs *conf.SchedulerConf) *KubernetesShim {
+func NewShimSchedulerForPlugin(scheduler api.SchedulerAPI, informerFactory informers.SharedInformerFactory, configs *conf.SchedulerConf, bootstrapConfigMaps []*v1.ConfigMap) *KubernetesShim {
 	apiFactory := client.NewAPIFactory(scheduler, informerFactory, configs, false)
-	context := cache.NewContext(apiFactory)
+	context := cache.NewContextWithBootstrapConfigMaps(apiFactory, bootstrapConfigMaps)
 	context.SetPluginMode(true)
 	rmCallback := callback.NewAsyncRMCallback(context)
 	appManager := appmgmt.NewAMService(context, apiFactory)
@@ -160,7 +162,7 @@ func (ss *KubernetesShim) recoverSchedulerState() {
 		// this step, we collect all the existing allocated pods from api-server,
 		// identify the scheduling identity (aka applicationInfo) from the pod,
 		// and then add these applications to the scheduler.
-		if err := ss.appManager.WaitForRecovery(30 * time.Second); err != nil {
+		if err := ss.appManager.WaitForRecovery(); err != nil {
 			// failed
 			log.Logger().Error("scheduler recovery failed", zap.Error(err))
 			dispatcher.Dispatch(ShimSchedulerEvent{
@@ -213,6 +215,11 @@ func (ss *KubernetesShim) registerShimLayer() error {
 	buildInfoMap["buildVersion"] = conf.BuildVersion
 	buildInfoMap["buildDate"] = conf.BuildDate
 	buildInfoMap["isPluginVersion"] = strconv.FormatBool(conf.IsPluginVersion)
+	buildInfoMap["goVersion"] = conf.GoVersion
+	buildInfoMap["arch"] = conf.Arch
+	buildInfoMap["coreSHA"] = conf.CoreSHA
+	buildInfoMap["siSHA"] = conf.SiSHA
+	buildInfoMap["shimSHA"] = conf.ShimSHA
 
 	configMaps, err := ss.context.LoadConfigMaps()
 	if err != nil {
@@ -254,7 +261,7 @@ func (ss *KubernetesShim) GetSchedulerState() string {
 func (ss *KubernetesShim) handle(se events.SchedulerEvent) error {
 	ss.lock.Lock()
 	defer ss.lock.Unlock()
-	err := ss.stateMachine.Event(se.GetEvent(), ss)
+	err := ss.stateMachine.Event(context.Background(), se.GetEvent(), ss)
 	if err != nil && err.Error() == "no transition" {
 		return err
 	}
@@ -269,7 +276,7 @@ func (ss *KubernetesShim) canHandle(se events.SchedulerEvent) bool {
 
 // each schedule iteration, we scan all apps and triggers app state transition
 func (ss *KubernetesShim) schedule() {
-	apps := ss.context.SelectApplications(nil)
+	apps := ss.context.GetAllApplications()
 	for _, app := range apps {
 		if app.Schedule() {
 			ss.setOutstandingAppsFound(true)
