@@ -20,14 +20,17 @@ package shim
 
 import (
 	"fmt"
+	"sync/atomic"
 	"testing"
 	"time"
 
 	"go.uber.org/zap"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 	v1 "k8s.io/api/core/v1"
+	schedv1 "k8s.io/api/scheduling/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
 
 	"github.com/apache/yunikorn-core/pkg/entrypoint"
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt"
@@ -36,6 +39,8 @@ import (
 	"github.com/apache/yunikorn-k8shim/pkg/callback"
 	"github.com/apache/yunikorn-k8shim/pkg/client"
 	"github.com/apache/yunikorn-k8shim/pkg/common"
+	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
+	"github.com/apache/yunikorn-k8shim/pkg/common/events"
 	"github.com/apache/yunikorn-k8shim/pkg/common/utils"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
 	"github.com/apache/yunikorn-k8shim/pkg/log"
@@ -53,6 +58,7 @@ type MockScheduler struct {
 	coreContext *entrypoint.ServiceContext
 	apiProvider *client.MockedAPIProvider
 	stopChan    chan struct{}
+	started     atomic.Bool
 }
 
 func (fc *MockScheduler) init() {
@@ -62,6 +68,7 @@ func (fc *MockScheduler) init() {
 	fc.rmProxy = serviceContext.RMProxy
 	mockedAPIProvider := client.NewMockedAPIProvider(false)
 	mockedAPIProvider.GetAPIs().SchedulerAPI = fc.rmProxy
+	events.SetRecorder(events.NewMockedRecorder())
 
 	context := cache.NewContext(mockedAPIProvider)
 	rmCallback := callback.NewAsyncRMCallback(context)
@@ -75,7 +82,9 @@ func (fc *MockScheduler) init() {
 }
 
 func (fc *MockScheduler) start() {
+	fc.apiProvider.RunEventHandler() // must be called first
 	fc.scheduler.Run()
+	fc.started.Store(true)
 }
 
 func (fc *MockScheduler) updateConfig(queues string) error {
@@ -86,17 +95,44 @@ func (fc *MockScheduler) updateConfig(queues string) error {
 	})
 }
 
-func (fc *MockScheduler) addNode(nodeName, nodeLabels string, memory, cpu int64) error {
+// Deprecated: this method only updates the core without the shim. Prefer MockScheduler.AddNode(*v1.Node) instead.
+func (fc *MockScheduler) addNode(nodeName string, nodeLabels map[string]string, memory, cpu, pods int64) error {
+	cache := fc.context.GetSchedulerCache()
+	zero := resource.Scale(0)
+	// add node to the cache so that predicates can run properly
+	cache.AddNode(&v1.Node{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:   nodeName,
+			Labels: nodeLabels,
+		},
+		Status: v1.NodeStatus{
+			Allocatable: map[v1.ResourceName]resource.Quantity{
+				v1.ResourcePods:   *resource.NewScaledQuantity(pods, zero),
+				v1.ResourceMemory: *resource.NewScaledQuantity(memory, zero),
+				v1.ResourceCPU:    *resource.NewScaledQuantity(cpu, zero),
+			},
+			Capacity: map[v1.ResourceName]resource.Quantity{
+				v1.ResourcePods:   *resource.NewScaledQuantity(pods, zero),
+				v1.ResourceMemory: *resource.NewScaledQuantity(memory, zero),
+				v1.ResourceCPU:    *resource.NewScaledQuantity(cpu, zero),
+			},
+		},
+	})
+
 	nodeResource := common.NewResourceBuilder().
 		AddResource(siCommon.Memory, memory).
 		AddResource(siCommon.CPU, cpu).
+		AddResource("pods", pods).
 		Build()
 	request := common.CreateUpdateRequestForNewNode(nodeName, nodeLabels, nodeResource, nil, nil, true)
 	fmt.Printf("report new nodes to scheduler, request: %s", request.String())
 	return fc.apiProvider.GetAPIs().SchedulerAPI.UpdateNode(&request)
 }
 
+// Deprecated: this method only updates the core without the shim. Prefer MockScheduler.AddPod(*v1.Pod) instead.
 func (fc *MockScheduler) addTask(appID string, taskID string, ask *si.Resource) {
+	cache := fc.context.GetSchedulerCache()
+	// add pod to the cache so that predicates can run properly
 	resources := make(map[v1.ResourceName]resource.Quantity)
 	for k, v := range ask.Resources {
 		resources[v1.ResourceName(k)] = *resource.NewQuantity(v.Value, resource.DecimalSI)
@@ -108,18 +144,29 @@ func (fc *MockScheduler) addTask(appID string, taskID string, ask *si.Resource)
 			Requests: resources,
 		},
 	})
+	pod := &v1.Pod{
+		ObjectMeta: metav1.ObjectMeta{
+			UID:  types.UID(taskID),
+			Name: taskID,
+			Annotations: map[string]string{
+				constants.AnnotationApplicationID: appID,
+			},
+			Labels: map[string]string{
+				constants.LabelApplicationID: appID,
+			},
+		},
+		Spec: v1.PodSpec{
+			SchedulerName: constants.SchedulerName,
+			Containers:    containers,
+		},
+	}
+	cache.AddPod(pod)
+
 	fc.context.AddTask(&interfaces.AddTaskRequest{
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: appID,
 			TaskID:        taskID,
-			Pod: &v1.Pod{
-				ObjectMeta: metav1.ObjectMeta{
-					Name: taskID,
-				},
-				Spec: v1.PodSpec{
-					Containers: containers,
-				},
-			},
+			Pod:           pod,
 		},
 	})
 }
@@ -142,27 +189,27 @@ func (fc *MockScheduler) waitForSchedulerState(t *testing.T, expectedState strin
 }
 
 func (fc *MockScheduler) waitAndAssertApplicationState(t *testing.T, appID, expectedState string) {
-	appList := fc.context.SelectApplications(func(app *cache.Application) bool {
-		return app.GetApplicationID() == appID
-	})
-	assert.Equal(t, len(appList), 1)
-	assert.Equal(t, appList[0].GetApplicationID(), appID)
+	app := fc.context.GetApplication(appID)
+	assert.Equal(t, app != nil, true)
+	assert.Equal(t, app.GetApplicationID(), appID)
 	deadline := time.Now().Add(10 * time.Second)
 	for {
-		if appList[0].GetApplicationState() == expectedState {
+		if app.GetApplicationState() == expectedState {
 			break
 		}
 		log.Logger().Info("waiting for app state",
 			zap.String("expected", expectedState),
-			zap.String("actual", appList[0].GetApplicationState()))
+			zap.String("actual", app.GetApplicationState()))
 		time.Sleep(time.Second)
 		if time.Now().After(deadline) {
 			t.Errorf("application %s doesn't reach expected state in given time, expecting: %s, actual: %s",
-				appID, expectedState, appList[0].GetApplicationState())
+				appID, expectedState, app.GetApplicationState())
 		}
 	}
 }
 
+// Deprecated: this method adds an application directly to the Context, and it skips relevant
+// code paths. Prefer MockScheduler.AddPod(*v1.Pod) instead.
 func (fc *MockScheduler) addApplication(appId string, queue string) {
 	fc.context.AddApplication(&interfaces.AddApplicationRequest{
 		Metadata: interfaces.ApplicationMetadata{
@@ -178,19 +225,12 @@ func (fc *MockScheduler) removeApplication(appId string) error {
 	return fc.context.RemoveApplication(appId)
 }
 
-func (fc *MockScheduler) newApplication(appID, queueName string) *cache.Application {
-	app := cache.NewApplication(appID, queueName, "testuser", []string{"dev"}, map[string]string{}, fc.apiProvider.GetAPIs().SchedulerAPI)
-	return app
-}
-
 func (fc *MockScheduler) waitAndAssertTaskState(t *testing.T, appID, taskID, expectedState string) {
-	appList := fc.context.SelectApplications(func(app *cache.Application) bool {
-		return app.GetApplicationID() == appID
-	})
-	assert.Equal(t, len(appList), 1)
-	assert.Equal(t, appList[0].GetApplicationID(), appID)
+	app := fc.context.GetApplication(appID)
+	assert.Equal(t, app != nil, true)
+	assert.Equal(t, app.GetApplicationID(), appID)
 
-	task, err := appList[0].GetTask(taskID)
+	task, err := app.GetTask(taskID)
 	assert.NilError(t, err, "Task retrieval failed")
 	deadline := time.Now().Add(10 * time.Second)
 	for {
@@ -230,4 +270,76 @@ func (fc *MockScheduler) waitAndVerifySchedulerAllocations(
 func (fc *MockScheduler) stop() {
 	close(fc.stopChan)
 	fc.scheduler.Stop()
+	fc.apiProvider.Stop()
+	fc.started.Store(false)
+}
+
+func (fc *MockScheduler) AddPod(pod *v1.Pod) {
+	fc.ensureStarted()
+	fc.apiProvider.AddPod(pod)
+}
+
+func (fc *MockScheduler) UpdatePod(oldObj *v1.Pod, newObj *v1.Pod) {
+	fc.ensureStarted()
+	fc.apiProvider.UpdatePod(oldObj, newObj)
+}
+
+func (fc *MockScheduler) DeletePod(obj *v1.Pod) {
+	fc.ensureStarted()
+	fc.apiProvider.DeletePod(obj)
+}
+
+func (fc *MockScheduler) AddNode(obj *v1.Node) {
+	fc.ensureStarted()
+	fc.apiProvider.AddNode(obj)
+}
+
+func (fc *MockScheduler) DeleteNode(obj *v1.Node) {
+	fc.ensureStarted()
+	fc.apiProvider.DeleteNode(obj)
+}
+
+func (fc *MockScheduler) UpdateNode(oldObj *v1.Node, newObj *v1.Node) {
+	fc.ensureStarted()
+	fc.apiProvider.UpdateNode(oldObj, newObj)
+}
+
+func (fc *MockScheduler) AddConfigMap(obj *v1.ConfigMap) {
+	fc.ensureStarted()
+	fc.apiProvider.AddConfigMap(obj)
+}
+
+func (fc *MockScheduler) DeleteConfigMap(obj *v1.ConfigMap) {
+	fc.ensureStarted()
+	fc.apiProvider.DeleteConfigMap(obj)
+}
+
+func (fc *MockScheduler) UpdateConfigMap(oldObj *v1.ConfigMap, newObj *v1.ConfigMap) {
+	fc.ensureStarted()
+	fc.apiProvider.UpdateConfigMap(oldObj, newObj)
+}
+
+func (fc *MockScheduler) AddPriorityClass(obj *schedv1.PriorityClass) {
+	fc.ensureStarted()
+	fc.apiProvider.AddPriorityClass(obj)
+}
+
+func (fc *MockScheduler) DeletePriorityClass(obj *schedv1.PriorityClass) {
+	fc.ensureStarted()
+	fc.apiProvider.DeletePriorityClass(obj)
+}
+
+func (fc *MockScheduler) UpdatePriorityClass(oldObj *schedv1.PriorityClass, newObj *schedv1.PriorityClass) {
+	fc.ensureStarted()
+	fc.apiProvider.UpdatePriorityClass(oldObj, newObj)
+}
+
+func (fc *MockScheduler) GetPodBindStats() client.BindStats {
+	return fc.apiProvider.GetPodBindStats()
+}
+
+func (fc *MockScheduler) ensureStarted() {
+	if !fc.started.Load() {
+		panic("mock scheduler is not started - call start() first")
+	}
 }
