@@ -24,7 +24,7 @@ import (
 	"time"
 
 	"go.uber.org/zap"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 	v1 "k8s.io/api/core/v1"
 
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt"
@@ -38,8 +38,6 @@ import (
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
-const nodeLabels = "{\"label1\":\"key1\",\"label2\":\"key2\"}"
-
 func TestApplicationScheduling(t *testing.T) {
 	configData := `
 partitions:
@@ -67,11 +65,15 @@ partitions:
 	cluster.waitForSchedulerState(t, SchedulerStates().Running)
 	err := cluster.updateConfig(configData)
 	assert.NilError(t, err, "update config failed")
+	nodeLabels := map[string]string{
+		"label1": "key1",
+		"label2": "key2",
+	}
 
 	// register nodes
-	err = cluster.addNode("test.host.01", nodeLabels, 100000000, 10)
+	err = cluster.addNode("test.host.01", nodeLabels, 100000000, 10, 10)
 	assert.NilError(t, err, "add node failed")
-	err = cluster.addNode("test.host.02", nodeLabels, 100000000, 10)
+	err = cluster.addNode("test.host.02", nodeLabels, 100000000, 10, 10)
 	assert.NilError(t, err, "add node failed")
 
 	// create app and tasks
@@ -118,10 +120,15 @@ partitions:
 	err := cluster.updateConfig(configData)
 	assert.NilError(t, err, "update config failed")
 
+	nodeLabels := map[string]string{
+		"label1": "key1",
+		"label2": "key2",
+	}
+
 	// register nodes
-	err = cluster.addNode("test.host.01", nodeLabels, 100000000, 10)
+	err = cluster.addNode("test.host.01", nodeLabels, 100000000, 10, 10)
 	assert.NilError(t, err)
-	err = cluster.addNode("test.host.02", nodeLabels, 100000000, 10)
+	err = cluster.addNode("test.host.02", nodeLabels, 100000000, 10, 10)
 	assert.NilError(t, err)
 
 	// add app to context
@@ -212,10 +219,14 @@ partitions:
 	err := cluster.updateConfig(configData)
 	assert.NilError(t, err, "update config failed")
 
+	nodeLabels := map[string]string{
+		"label1": "key1",
+		"label2": "key2",
+	}
 	// register nodes
-	err = cluster.addNode("test.host.01", nodeLabels, 100000000, 10)
+	err = cluster.addNode("test.host.01", nodeLabels, 100000000, 10, 10)
 	assert.NilError(t, err, "add node failed")
-	err = cluster.addNode("test.host.02", nodeLabels, 100000000, 10)
+	err = cluster.addNode("test.host.02", nodeLabels, 100000000, 10, 10)
 	assert.NilError(t, err, "add node failed")
 
 	// create app and tasks
