@@ -100,6 +100,24 @@ function install_kind() {
   check_cmd "${KIND}"
 }
 
+function install_spark() {
+  CURRENT=$(pwd)
+  SPARK_VERSION="3.3.1"
+  DOWNLOAD="${CURRENT}/_spark"
+  SPARK_SUBMIT_CMD="${DOWNLOAD}/spark-${SPARK_VERSION}-bin-hadoop3/bin/spark-submit"
+  if [[ ! -d "${DOWNLOAD}" || ! -f "${SPARK_SUBMIT_CMD}" ]]; then
+    rm -rf "${DOWNLOAD}"
+    mkdir "${DOWNLOAD}" \
+      && cd "${DOWNLOAD}" \
+      && curl "https://archive.apache.org/dist/spark/spark-${SPARK_VERSION}/spark-${SPARK_VERSION}-bin-hadoop3.tgz" | tar xzf - \
+      && chmod +x "spark-${SPARK_VERSION}-bin-hadoop3"
+    exit_on_error "install spark failed"
+  fi
+  export SPARK_HOME="${DOWNLOAD}/spark-${SPARK_VERSION}-bin-hadoop3"
+  export SPARK_PYTHON_IMAGE="docker.io/apache/spark-py:v${SPARK_VERSION}"
+  cd "${CURRENT}" || exit
+}
+
 function install_helm() {
   FORCE_HELM_INSTALL=false
   if ! command -v helm &> /dev/null
@@ -124,7 +142,7 @@ function install_helm() {
 }
 
 function install_cluster() {
-  echo "step 1/9: checking required configuration"
+  echo "step 1/10: checking required configuration"
   if [ ! -r "${KIND_CONFIG}" ]; then
     exit_on_error "kind config not found: ${KIND_CONFIG}"
   fi
@@ -138,7 +156,7 @@ function install_cluster() {
   fi
 
   # build docker images from latest code, so that we can install yunikorn with these latest images
-  echo "step 2/9: building docker images from latest code"
+  echo "step 2/10: building docker images from latest code"
   check_docker
   QUIET="--quiet" REGISTRY=local VERSION=latest make image
   exit_on_error "build docker images failed"
@@ -146,24 +164,29 @@ function install_cluster() {
   exit_on_error "build test web images failed"
 
   # install ginkgo and gomega for e2e tests.
-  echo "step 3/9: installing Ginkgo & Gomega at $(go env GOPATH)/bin"
-  go install github.com/onsi/ginkgo/ginkgo
+  echo "step 3/10: installing Ginkgo & Gomega at $(go env GOPATH)/bin"
+  go get github.com/onsi/ginkgo/v2@v2.9.0
+  go install github.com/onsi/ginkgo/v2/ginkgo
   go install github.com/onsi/gomega
   check_cmd "ginkgo"
 
-  echo "step 4/9: installing helm-v3"
+  echo "step 4/10: installing helm-v3"
   install_helm
 
   # install kubectl
-  echo "step 5/9: installing kubectl"
+  echo "step 5/10: installing kubectl"
   install_kubectl
 
   # install KIND
-  echo "step 6/9: installing kind"
+  echo "step 6/10: installing kind"
   install_kind
 
+  # install Spark
+  echo "step 7/10: installing spark"
+  install_spark
+
   # create K8s cluster
-  echo "step 7/9: installing K8s cluster using kind"
+  echo "step 8/10: installing K8s cluster using kind"
   "${KIND}" create cluster --name "${CLUSTER_NAME}" --image "${CLUSTER_VERSION}" --config="${KIND_CONFIG}"
   exit_on_error "install K8s cluster failed"
   kubectl cluster-info --context kind-"${CLUSTER_NAME}"
@@ -174,7 +197,7 @@ function install_cluster() {
   kubectl describe nodes
 
   # pre-load yunikorn docker images to kind
-  echo "step 8/9: pre-load yunikorn images"
+  echo "step 9/10: pre-load yunikorn images"
   "${KIND}" load docker-image "local/yunikorn:${SCHEDULER_IMAGE}" --name "${CLUSTER_NAME}"
   exit_on_error "pre-load scheduler image failed: ${SCHEDULER_IMAGE}"
   "${KIND}" load docker-image "local/yunikorn:${ADMISSION_IMAGE}" --name "${CLUSTER_NAME}"
@@ -182,7 +205,7 @@ function install_cluster() {
   "${KIND}" load docker-image "local/yunikorn:${WEBTEST_IMAGE}" --name "${CLUSTER_NAME}"
   exit_on_error "pre-load web image failed: ${WEBTEST_IMAGE}"
 
-  echo "step 9/9: installing yunikorn"
+  echo "step 10/10: installing yunikorn"
   helm install yunikorn "${CHART_PATH}" --namespace yunikorn \
     --set image.repository=local/yunikorn \
     --set image.tag="${SCHEDULER_IMAGE}" \
