@@ -32,11 +32,22 @@ import (
 	"time"
 
 	"github.com/google/uuid"
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
 )
 
+var MasterTaints = map[string]struct{}{
+	"node-role.kubernetes.io/control-plane": {},
+	"node-role.kubernetes.io/master":        {},
+}
+
+var ComputeNodeLabels = map[string]struct{}{
+	"compute": {},
+}
+
+var RoleNodeLabel = "role"
+
 func GetAbsPath(p string) (string, error) {
 	path, err := filepath.Abs(p)
 	return path, err
@@ -44,6 +55,7 @@ func GetAbsPath(p string) (string, error) {
 
 // GetTestName returns the test Name in a single string without spaces or /
 func GetTestName() string {
+	//nolint
 	testDesc := ginkgo.CurrentGinkgoTestDescription()
 	name := strings.Replace(testDesc.FullTestText, " ", "_", -1)
 	name = strings.Trim(name, "*")
@@ -68,6 +80,17 @@ func CreateReportDirectory() (string, error) {
 	return testPath, err
 }
 
+// CreateDirectoryForJunitReports and returns the directory path
+// If the directory cannot be created it'll return an error
+func CreateJUnitReportDir() error {
+	dir := configmanager.YuniKornTestConfig.LogDir
+	if _, err := os.Stat(dir); err == nil {
+		return nil
+	}
+	err := os.MkdirAll(dir, os.ModePerm)
+	return err
+}
+
 // CreateLogFile creates the ReportDirectory if it is not present, writes the
 // given testdata to the given filename.
 func CreateLogFile(filename string, data []byte) error {
@@ -117,10 +140,10 @@ func SliceExists(slice interface{}, item interface{}) (bool, error) {
 }
 
 // Takes 2 restClient responses with queue information, and checks if they are the same by
-// 		1. Validates queue name is same
-//		2. Validates qA and qB have same number of sub-queues
-// 		3a. If same queue name and 0 sub-queues, return true
-//		3b. Otherwise, call CompareQueueMap on each child pairing of qA and qB
+//  1. Validates queue name is same
+//  2. Validates qA and qB have same number of sub-queues
+//     3a. If same queue name and 0 sub-queues, return true
+//     3b. Otherwise, call CompareQueueMap on each child pairing of qA and qB
 func CompareQueueMap(a map[string]interface{}, b map[string]interface{}) (bool, error) {
 	// Validate queue name
 	if a["queuename"] != b["queuename"] {
