@@ -42,6 +42,7 @@ type SleepPodConfig struct {
 	RequiredNode string
 	Optedout     bool
 	UID          types.UID
+	Labels       map[string]string
 }
 
 // TestPodConfig template for  sleepPods
@@ -89,16 +90,21 @@ func InitSleepPod(conf SleepPodConfig) (*v1.Pod, error) {
 		}
 	}
 
-	if conf.UID != "" {
-		owner := metav1.OwnerReference{APIVersion: "v1", Kind: "ReplicaSet", Name: "ReplicaSetJob", UID: conf.UID}
-		owners = []metav1.OwnerReference{owner}
-	}
-
 	optedOut := "true"
 	if !conf.Optedout {
 		optedOut = "false"
 	}
 
+	labels := map[string]string{
+		"app":                                  "sleep",
+		"applicationId":                        conf.AppID,
+		"yunikorn.apache.org/allow-preemption": optedOut,
+	}
+	if len(conf.Labels) > 0 {
+		for k, v := range conf.Labels {
+			labels[k] = v
+		}
+	}
 	var secs int64 = 0
 	testPodConfig := TestPodConfig{
 		Name:                       conf.Name,
@@ -106,11 +112,7 @@ func InitSleepPod(conf SleepPodConfig) (*v1.Pod, error) {
 		RestartPolicy:              v1.RestartPolicyNever,
 		DeletionGracePeriodSeconds: &secs,
 		Command:                    []string{"sleep", strconv.Itoa(conf.Time)},
-		Labels: map[string]string{
-			"app":                                  "sleep",
-			"applicationId":                        conf.AppID,
-			"yunikorn.apache.org/allow-preemption": optedOut,
-		},
+		Labels:                     labels,
 		Resources: &v1.ResourceRequirements{
 			Requests: v1.ResourceList{
 				"cpu":    resource.MustParse(strconv.FormatInt(conf.CPU, 10) + "m"),
