@@ -22,6 +22,9 @@ import (
 	"fmt"
 	"time"
 
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+
+	"github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
@@ -142,14 +145,14 @@ var _ = Describe("", func() {
 		tgPlaceHolders := yunikorn.GetPlaceholderNames(podConf.Annotations, podConf.Labels["applicationId"])
 		for _, phNames := range tgPlaceHolders {
 			for _, ph := range phNames {
-				phTermErr := kClient.WaitForPodTerminated(ns, ph, time.Minute)
+				phTermErr := kClient.WaitForPodTerminated(ns, ph, 3*time.Minute)
 				Ω(phTermErr).NotTo(HaveOccurred())
 			}
 		}
 
 		// Check real gang members now running
 		By("Wait for all gang members running")
-		jobRunErr := kClient.WaitForJobPods(ns, realJobConf.Name, int(realJobConf.Parallelism), 30*time.Second)
+		jobRunErr := kClient.WaitForJobPods(ns, realJobConf.Name, int(realJobConf.Parallelism), 3*time.Minute)
 		Ω(jobRunErr).NotTo(HaveOccurred())
 
 		By("Verify appStatus = Running")
@@ -231,6 +234,8 @@ var _ = Describe("", func() {
 		for tg, phNames := range tgPlaceHolders {
 			taskGroupNodes[tg] = map[string]int{}
 			for _, name := range phNames {
+				podRunErr := kClient.WaitForPodRunning(ns, name, time.Second*120)
+				Ω(podRunErr).NotTo(HaveOccurred())
 				ph, phErr := kClient.GetPod(name, ns)
 				Ω(phErr).NotTo(HaveOccurred())
 				taskGroupNodes[tg][ph.Spec.NodeName]++
@@ -259,7 +264,7 @@ var _ = Describe("", func() {
 		By("Wait for all placeholders terminated")
 		for _, phNames := range tgPlaceHolders {
 			for _, ph := range phNames {
-				phTermErr := kClient.WaitForPodTerminated(ns, ph, time.Minute)
+				phTermErr := kClient.WaitForPodTerminated(ns, ph, 3*time.Minute)
 				Ω(phTermErr).NotTo(HaveOccurred())
 			}
 		}
@@ -273,7 +278,7 @@ var _ = Describe("", func() {
 			Ω(len(jobPods.Items)).Should(BeNumerically("==", tg.MinMember))
 			realPodNodes[tg.Name] = map[string]int{}
 			for _, pod := range jobPods.Items {
-				podRunErr := kClient.WaitForPodRunning(ns, pod.Name, time.Minute)
+				podRunErr := kClient.WaitForPodRunning(ns, pod.Name, time.Minute*5)
 				Ω(podRunErr).NotTo(HaveOccurred())
 				pod, getErr := kClient.GetPod(pod.Name, ns)
 				Ω(getErr).NotTo(HaveOccurred())
@@ -877,7 +882,7 @@ var _ = Describe("", func() {
 		By("Verify placeholders deleted")
 		for _, placeholders := range phNames {
 			for _, ph := range placeholders {
-				deleteErr = kClient.WaitForPodTerminated(fifoQName, ph, 30*time.Second)
+				deleteErr = kClient.WaitForPodTerminated(ns, ph, 30*time.Second)
 				Ω(deleteErr).NotTo(HaveOccurred(), "Placeholder %s still running", ph)
 			}
 		}
@@ -888,7 +893,194 @@ var _ = Describe("", func() {
 		Ω(len(appInfo.Allocations)).To(BeNumerically("==", 0))
 	})
 
-	DescribeTable("", func(annotations k8s.PodAnnotation) {
+	// Test to verify originator deletion will trigger placeholders cleanup
+	// 1. Create an originator pod
+	// 2. Not set pod ownerreference
+	// 3. Delete originator pod to trigger placeholders deletion
+	// 4. Verify placeholders deleted
+	// 5. Verify app allocation is empty
+	It("Verify_OriginatorDeletion_Trigger_Placeholders_Cleanup", func() {
+		// case 1: originator pod without ownerreference
+		podResources := map[string]resource.Quantity{
+			"cpu":    resource.MustParse("10m"),
+			"memory": resource.MustParse("10M"),
+		}
+		podConf := k8s.TestPodConfig{
+			Name: "gang-driver-pod" + common.RandSeq(5),
+			Labels: map[string]string{
+				"app":           "sleep-" + common.RandSeq(5),
+				"applicationId": "appid-" + common.RandSeq(5),
+			},
+			Annotations: &k8s.PodAnnotation{
+				TaskGroups: []v1alpha1.TaskGroup{
+					{
+						Name:         groupA + "-" + common.RandSeq(5),
+						MinMember:    int32(3),
+						MinResource:  podResources,
+						NodeSelector: map[string]string{"kubernetes.io/hostname": "unsatisfiable"},
+					},
+					{
+						Name:        groupB + "-" + common.RandSeq(5),
+						MinMember:   int32(3),
+						MinResource: podResources,
+					},
+				},
+			},
+			Resources: &v1.ResourceRequirements{
+				Requests: v1.ResourceList{
+					"cpu":    podResources["cpu"],
+					"memory": podResources["memory"],
+				},
+			},
+		}
+
+		podTest, err := k8s.InitTestPod(podConf)
+		Ω(err).NotTo(HaveOccurred())
+		taskGroupsMap, annErr := k8s.PodAnnotationToMap(podConf.Annotations)
+		Ω(annErr).NotTo(HaveOccurred())
+		By(fmt.Sprintf("Deploy pod %s with task-groups: %+v", podTest.Name, taskGroupsMap[k8s.TaskGroups]))
+		originator, err := kClient.CreatePod(podTest, ns)
+		Ω(err).NotTo(HaveOccurred())
+
+		By("Verify appState = Accepted")
+		timeoutErr := restClient.WaitForAppStateTransition(defaultPartition, nsQueue, podConf.Labels["applicationId"], yunikorn.States().Application.Accepted, 20)
+		Ω(timeoutErr).NotTo(HaveOccurred())
+
+		By("Wait for placeholders running")
+		phNames := yunikorn.GetPlaceholderNames(podConf.Annotations, podConf.Labels["applicationId"])
+		tgBNames := phNames[podConf.Annotations.TaskGroups[1].Name]
+		for _, ph := range tgBNames {
+			runErr := kClient.WaitForPodRunning(ns, ph, 30*time.Second)
+			Ω(runErr).NotTo(HaveOccurred())
+		}
+
+		By("Delete originator pod")
+		deleteErr := kClient.DeletePod(originator.Name, ns)
+		Ω(deleteErr).NotTo(HaveOccurred())
+
+		By("Verify placeholders deleted")
+		for _, placeholders := range phNames {
+			for _, ph := range placeholders {
+				deleteErr = kClient.WaitForPodTerminated(ns, ph, 30*time.Second)
+				Ω(deleteErr).NotTo(HaveOccurred(), "Placeholder %s still running", ph)
+			}
+		}
+
+		By("Verify app allocation is empty")
+		appInfo, restErr := restClient.GetAppInfo(defaultPartition, nsQueue, podConf.Labels["applicationId"])
+		Ω(restErr).NotTo(HaveOccurred())
+		Ω(len(appInfo.Allocations)).To(BeNumerically("==", 0))
+
+	})
+
+	// Test to verify originator deletion will trigger placeholders cleanup
+	// 1. Create an originator pod
+	// 2. Set pod ownerreference with ownerreference, take configmap for example
+	// 3. Delete originator pod to trigger placeholders deletion
+	// 4. Verify placeholders deleted
+	// 5. Verify app allocation is empty
+	It("Verify_OriginatorDeletionWithOwnerreference_Trigger_Placeholders_Cleanup", func() {
+		// case 2: originator pod with ownerreference
+		podResources := map[string]resource.Quantity{
+			"cpu":    resource.MustParse("10m"),
+			"memory": resource.MustParse("10M"),
+		}
+		podConf := k8s.TestPodConfig{
+			Name: "gang-driver-pod" + common.RandSeq(5),
+			Labels: map[string]string{
+				"app":           "sleep-" + common.RandSeq(5),
+				"applicationId": "appid-" + common.RandSeq(5),
+			},
+			Annotations: &k8s.PodAnnotation{
+				TaskGroups: []v1alpha1.TaskGroup{
+					{
+						Name:         groupA + "-" + common.RandSeq(5),
+						MinMember:    int32(3),
+						MinResource:  podResources,
+						NodeSelector: map[string]string{"kubernetes.io/hostname": "unsatisfiable"},
+					},
+					{
+						Name:        groupB + "-" + common.RandSeq(5),
+						MinMember:   int32(3),
+						MinResource: podResources,
+					},
+				},
+			},
+			Resources: &v1.ResourceRequirements{
+				Requests: v1.ResourceList{
+					"cpu":    podResources["cpu"],
+					"memory": podResources["memory"],
+				},
+			},
+		}
+
+		// create a configmap as ownerreference
+		testConfigmap := &v1.ConfigMap{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "test-cm",
+				UID:  "test-cm-uid",
+			},
+			Data: map[string]string{
+				"test": "test",
+			},
+		}
+		defer func() {
+			err := kClient.DeleteConfigMap(testConfigmap.Name, ns)
+			Ω(err).NotTo(HaveOccurred())
+		}()
+
+		testConfigmap, err := kClient.CreateConfigMap(testConfigmap, ns)
+		Ω(err).NotTo(HaveOccurred())
+
+		podTest, err := k8s.InitTestPod(podConf)
+		Ω(err).NotTo(HaveOccurred())
+
+		podTest.OwnerReferences = []metav1.OwnerReference{
+			{
+				APIVersion: "v1",
+				Kind:       "ConfigMap",
+				Name:       testConfigmap.Name,
+				UID:        testConfigmap.UID,
+			},
+		}
+
+		taskGroupsMap, annErr := k8s.PodAnnotationToMap(podConf.Annotations)
+		Ω(annErr).NotTo(HaveOccurred())
+		By(fmt.Sprintf("Deploy pod %s with task-groups: %+v", podTest.Name, taskGroupsMap[k8s.TaskGroups]))
+		originator, err := kClient.CreatePod(podTest, ns)
+		Ω(err).NotTo(HaveOccurred())
+
+		By("Verify appState = Accepted")
+		timeoutErr := restClient.WaitForAppStateTransition(defaultPartition, nsQueue, podConf.Labels["applicationId"], yunikorn.States().Application.Accepted, 20)
+		Ω(timeoutErr).NotTo(HaveOccurred())
+
+		By("Wait for placeholders running")
+		phNames := yunikorn.GetPlaceholderNames(podConf.Annotations, podConf.Labels["applicationId"])
+		tgBNames := phNames[podConf.Annotations.TaskGroups[1].Name]
+		for _, ph := range tgBNames {
+			runErr := kClient.WaitForPodRunning(ns, ph, 30*time.Second)
+			Ω(runErr).NotTo(HaveOccurred())
+		}
+
+		By("Delete originator pod")
+		deleteErr := kClient.DeletePod(originator.Name, ns)
+		Ω(deleteErr).NotTo(HaveOccurred())
+
+		By("Verify placeholders deleted")
+		for _, placeholders := range phNames {
+			for _, ph := range placeholders {
+				deleteErr = kClient.WaitForPodTerminated(ns, ph, 30*time.Second)
+				Ω(deleteErr).NotTo(HaveOccurred(), "Placeholder %s still running", ph)
+			}
+		}
+
+		By("Verify app allocation is empty")
+		appInfo, restErr := restClient.GetAppInfo(defaultPartition, nsQueue, podConf.Labels["applicationId"])
+		Ω(restErr).NotTo(HaveOccurred())
+		Ω(len(appInfo.Allocations)).To(BeNumerically("==", 0))
+	})
+
+	ginkgo.DescribeTable("", func(annotations k8s.PodAnnotation) {
 		podConf := k8s.TestPodConfig{
 			Labels: map[string]string{
 				"app":           "sleep-" + common.RandSeq(5),
@@ -937,7 +1129,7 @@ var _ = Describe("", func() {
 		Ω(getErr).NotTo(HaveOccurred())
 		Ω(len(appPods.Items)).To(BeNumerically("==", jobConf.Parallelism))
 	},
-		Entry("Verify_TG_With_Duplicate_Group", k8s.PodAnnotation{
+		ginkgo.Entry("Verify_TG_With_Duplicate_Group", k8s.PodAnnotation{
 			TaskGroups: []v1alpha1.TaskGroup{
 				{
 					Name:      "groupdup",
@@ -965,7 +1157,7 @@ var _ = Describe("", func() {
 				},
 			},
 		}),
-		Entry("Verify_TG_With_Invalid_Chars", k8s.PodAnnotation{
+		ginkgo.Entry("Verify_TG_With_Invalid_Chars", k8s.PodAnnotation{
 			TaskGroups: []v1alpha1.TaskGroup{
 				{
 					Name:      "GROUPCAPS",
@@ -977,7 +1169,7 @@ var _ = Describe("", func() {
 				},
 			},
 		}),
-		Entry("Verify_TG_With_Invalid_MinMember", k8s.PodAnnotation{
+		ginkgo.Entry("Verify_TG_With_Invalid_MinMember", k8s.PodAnnotation{
 			TaskGroups: []v1alpha1.TaskGroup{
 				{
 					Name:      groupA,
@@ -991,10 +1183,10 @@ var _ = Describe("", func() {
 		}),
 	)
 	AfterEach(func() {
-		testDescription := CurrentGinkgoTestDescription()
-		if testDescription.Failed {
-			tests.LogTestClusterInfoWrapper(testDescription.TestText, []string{ns})
-			tests.LogYunikornContainer(testDescription.TestText)
+		testDescription := ginkgo.CurrentSpecReport()
+		if testDescription.Failed() {
+			tests.LogTestClusterInfoWrapper(testDescription.FailureMessage(), []string{ns})
+			tests.LogYunikornContainer(testDescription.FailureMessage())
 		}
 		By("Tear down namespace: " + ns)
 		err := kClient.TearDownNamespace(ns)
