@@ -22,115 +22,141 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 	v1 "k8s.io/api/core/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/types"
 
 	tests "github.com/apache/yunikorn-k8shim/test/e2e"
-	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/common"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/k8s"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/yunikorn"
 )
 
-var _ = ginkgo.Describe("SimplePreemptor", func() {
-	var kClient k8s.KubeCtl
-	var restClient yunikorn.RClient
-	var ns *v1.Namespace
-	var oldConfigMap *v1.ConfigMap
-	var dev = "dev" + common.RandSeq(5)
+var kClient k8s.KubeCtl
+var restClient yunikorn.RClient
+var ns *v1.Namespace
+var dev = "dev" + common.RandSeq(5)
+var oldConfigMap = new(v1.ConfigMap)
+var annotation = "ann-" + common.RandSeq(10)
 
-	// Nodes
-	Worker1 := "yk8s-worker"
-	Worker2 := "yk8s-worker2"
-	Worker1Res := resource.NewQuantity(8*1000*1000, resource.DecimalSI)
-	Worker2Res := resource.NewQuantity(8*1000*1000, resource.DecimalSI)
-	var sleepPodMemLimit1 int64
-	var sleepPodMemLimit2 int64
+// Nodes
+var Worker1 = ""
+var Worker2 = ""
+var Worker1Res = resource.NewQuantity(8*1000*1000, resource.DecimalSI)
+var Worker2Res = resource.NewQuantity(8*1000*1000, resource.DecimalSI)
+var sleepPodMemLimit1 int64
+var sleepPodMemLimit2 int64
+var taintKey = "e2e_test_simple_preemptor"
+var nodesToTaint []string
 
-	ginkgo.BeforeSuite(func() {
-		// Initializing kubectl client
-		kClient = k8s.KubeCtl{}
-		Ω(kClient.SetClient()).To(gomega.BeNil())
-		// Initializing rest client
-		restClient = yunikorn.RClient{}
-		Ω(restClient).NotTo(gomega.BeNil())
+var _ = ginkgo.BeforeSuite(func() {
+	// Initializing kubectl client
+	kClient = k8s.KubeCtl{}
+	Ω(kClient.SetClient()).To(gomega.BeNil())
+	// Initializing rest client
+	restClient = yunikorn.RClient{}
+	Ω(restClient).NotTo(gomega.BeNil())
 
-		yunikorn.EnsureYuniKornConfigsPresent()
+	annotation = "ann-" + common.RandSeq(10)
+	yunikorn.EnsureYuniKornConfigsPresent()
+	yunikorn.UpdateConfigMapWrapper(oldConfigMap, "", annotation)
 
-		ginkgo.By("Port-forward the scheduler pod")
-		var err = kClient.PortForwardYkSchedulerPod()
-		Ω(err).NotTo(gomega.HaveOccurred())
+	ginkgo.By("Port-forward the scheduler pod")
+	var err = kClient.PortForwardYkSchedulerPod()
+	Ω(err).NotTo(gomega.HaveOccurred())
 
-		ginkgo.By("Enable basic scheduling config over config maps")
-		var c, configErr = kClient.GetConfigMaps(configmanager.YuniKornTestConfig.YkNamespace,
-			configmanager.DefaultYuniKornConfigMap)
-		Ω(configErr).NotTo(gomega.HaveOccurred())
-		Ω(c).NotTo(gomega.BeNil())
+	ginkgo.By("create development namespace")
+	ns, err = kClient.CreateNamespace(dev, nil)
+	gomega.Ω(err).NotTo(gomega.HaveOccurred())
+	gomega.Ω(ns.Status.Phase).To(gomega.Equal(v1.NamespaceActive))
 
-		oldConfigMap = c.DeepCopy()
-		Ω(c).Should(gomega.BeEquivalentTo(oldConfigMap))
+	var nodes *v1.NodeList
+	nodes, err = kClient.GetNodes()
+	Ω(err).NotTo(gomega.HaveOccurred())
+	Ω(len(nodes.Items)).NotTo(gomega.BeZero(), "Events cant be empty")
 
-		// Define basic configMap
-		sc := common.CreateBasicConfigMap()
-		configStr, yamlErr := common.ToYAML(sc)
-		Ω(yamlErr).NotTo(gomega.HaveOccurred())
-
-		c.Data[configmanager.DefaultPolicyGroup] = configStr
-		var d, err3 = kClient.UpdateConfigMap(c, configmanager.YuniKornTestConfig.YkNamespace)
-		Ω(err3).NotTo(gomega.HaveOccurred())
-		Ω(d).NotTo(gomega.BeNil())
-
-		ginkgo.By("create development namespace")
-		ns, err = kClient.CreateNamespace(dev, nil)
-		gomega.Ω(err).NotTo(gomega.HaveOccurred())
-		gomega.Ω(ns.Status.Phase).To(gomega.Equal(v1.NamespaceActive))
-
-		var nodes *v1.NodeList
-		nodes, err = kClient.GetNodes()
-		Ω(err).NotTo(gomega.HaveOccurred())
-		Ω(len(nodes.Items)).NotTo(gomega.BeZero(), "Events cant be empty")
-
-		// Extract node allocatable resources
-		for _, node := range nodes.Items {
-			if node.Name == Worker1 {
-				Worker1Res = node.Status.Allocatable.Memory()
-			} else if node.Name == Worker2 {
-				Worker2Res = node.Status.Allocatable.Memory()
-			}
+	// Extract node allocatable resources
+	for _, node := range nodes.Items {
+		// skip master if it's marked as such
+		node := node
+		if k8s.IsMasterNode(&node) || !k8s.IsComputeNode(&node) {
+			continue
 		}
+		if Worker1 == "" {
+			Worker1 = node.Name
+			Worker1Res = node.Status.Allocatable.Memory()
+		} else if Worker2 == "" {
+			Worker2 = node.Name
+			Worker2Res = node.Status.Allocatable.Memory()
+		} else {
+			nodesToTaint = append(nodesToTaint, node.Name)
+		}
+	}
 
-		var pods *v1.PodList
-		totalPodQuantity1 := *resource.NewQuantity(0, resource.DecimalSI)
-		totalPodQuantity2 := *resource.NewQuantity(0, resource.DecimalSI)
-		pods, err = kClient.GetPods("yunikorn")
-		if err == nil {
-			for _, pod := range pods.Items {
-				for _, c := range pod.Spec.Containers {
-					if pod.Spec.NodeName == Worker1 {
-						totalPodQuantity1.Add(*resource.NewQuantity(c.Resources.Requests.Memory().Value(), resource.DecimalSI))
-					} else if pod.Spec.NodeName == Worker2 {
-						totalPodQuantity2.Add(*resource.NewQuantity(c.Resources.Requests.Memory().Value(), resource.DecimalSI))
-					}
+	ginkgo.By("Tainting some nodes..")
+	for _, nodeName := range nodesToTaint {
+		ginkgo.By("Tainting node " + nodeName)
+		err = kClient.TaintNode(nodeName, taintKey, "value", v1.TaintEffectNoSchedule)
+		Ω(err).NotTo(gomega.HaveOccurred())
+	}
+
+	var pods *v1.PodList
+	totalPodQuantity1 := *resource.NewQuantity(0, resource.DecimalSI)
+	totalPodQuantity2 := *resource.NewQuantity(0, resource.DecimalSI)
+	pods, err = kClient.GetPods("yunikorn")
+	if err == nil {
+		for _, pod := range pods.Items {
+			for _, c := range pod.Spec.Containers {
+				if pod.Spec.NodeName == Worker1 {
+					totalPodQuantity1.Add(*resource.NewQuantity(c.Resources.Requests.Memory().Value(), resource.DecimalSI))
+				} else if pod.Spec.NodeName == Worker2 {
+					totalPodQuantity2.Add(*resource.NewQuantity(c.Resources.Requests.Memory().Value(), resource.DecimalSI))
 				}
 			}
 		}
-		Worker1Res.Sub(totalPodQuantity1)
-		sleepPodMemLimit1 = int64(float64(Worker1Res.Value())/3.5) / (1000 * 1000)
-		Worker2Res.Sub(totalPodQuantity2)
-		sleepPodMemLimit2 = int64(float64(Worker2Res.Value())/3.5) / (1000 * 1000)
-	})
+	}
+	Worker1Res.Sub(totalPodQuantity1)
+	sleepPodMemLimit1 = int64(float64(Worker1Res.Value())/3.5) / (1000 * 1000)
+	Worker2Res.Sub(totalPodQuantity2)
+	sleepPodMemLimit2 = int64(float64(Worker2Res.Value())/3.5) / (1000 * 1000)
+})
 
+var _ = ginkgo.AfterSuite(func() {
+
+	ginkgo.By("Untainting some nodes")
+	for _, nodeName := range nodesToTaint {
+		ginkgo.By("Untainting node " + nodeName)
+		err := kClient.UntaintNode(nodeName, taintKey)
+		Ω(err).NotTo(gomega.HaveOccurred(), "Could not remove taint from node "+nodeName)
+	}
+
+	ginkgo.By("Check Yunikorn's health")
+	checks, err := yunikorn.GetFailedHealthChecks()
+	Ω(err).NotTo(gomega.HaveOccurred())
+	Ω(checks).To(gomega.Equal(""), checks)
+
+	testDescription := ginkgo.CurrentSpecReport()
+	if testDescription.Failed() {
+		tests.LogTestClusterInfoWrapper(testDescription.FailureMessage(), []string{ns.Name})
+		tests.LogYunikornContainer(testDescription.FailureMessage())
+	}
+	ginkgo.By("Tearing down namespace: " + ns.Name)
+	err = kClient.TearDownNamespace(ns.Name)
+	Ω(err).NotTo(gomega.HaveOccurred())
+
+	yunikorn.RestoreConfigMapWrapper(oldConfigMap, annotation)
+})
+
+var _ = ginkgo.Describe("SimplePreemptor", func() {
 	ginkgo.It("Verify_basic_simple_preemption. Use case: Only one pod is running and same pod has been selected as victim", func() {
 
 		// Define sleepPod
-		sleepPodConfigs := k8s.SleepPodConfig{Name: "sleepjob", NS: dev, Mem: sleepPodMemLimit1 * 2, Time: 600, RequiredNode: "yk8s-worker"}
+		sleepPodConfigs := k8s.SleepPodConfig{Name: "sleepjob", NS: dev, Mem: sleepPodMemLimit1 * 2, Time: 600, RequiredNode: Worker1}
 		sleepPod2Configs := k8s.SleepPodConfig{Name: "sleepjob2", NS: dev, Mem: sleepPodMemLimit2 * 2, Time: 600}
-		sleepPod3Configs := k8s.SleepPodConfig{Name: "sleepjob3", NS: dev, RequiredNode: "yk8s-worker2", Mem: sleepPodMemLimit2 * 2, Time: 600}
+		sleepPod3Configs := k8s.SleepPodConfig{Name: "sleepjob3", NS: dev, RequiredNode: Worker2, Mem: sleepPodMemLimit2 * 2, Time: 600}
 
 		for _, config := range []k8s.SleepPodConfig{sleepPodConfigs, sleepPod2Configs, sleepPod3Configs} {
 			ginkgo.By("Deploy the sleep pod " + config.Name + " to the development namespace")
@@ -153,15 +179,15 @@ var _ = ginkgo.Describe("SimplePreemptor", func() {
 	ginkgo.It("Verify_simple_preemption. Use case: When 3 sleep pods (2 opted out, regular) are running, regular pod should be victim to free up resources for 4th sleep pod", func() {
 
 		// Define sleepPod
-		sleepPodConfigs := k8s.SleepPodConfig{Name: "sleepjob", NS: dev, Mem: sleepPodMemLimit1, Time: 600, RequiredNode: "yk8s-worker"}
-		sleepPod2Configs := k8s.SleepPodConfig{Name: "sleepjob2", NS: dev, Mem: sleepPodMemLimit1, Time: 600, RequiredNode: "yk8s-worker"}
-		sleepPod3Configs := k8s.SleepPodConfig{Name: "sleepjob3", NS: dev, Mem: sleepPodMemLimit1, Time: 600, RequiredNode: "yk8s-worker"}
+		sleepPodConfigs := k8s.SleepPodConfig{Name: "sleepjob", NS: dev, Mem: sleepPodMemLimit1, Time: 600, RequiredNode: Worker1}
+		sleepPod2Configs := k8s.SleepPodConfig{Name: "sleepjob2", NS: dev, Mem: sleepPodMemLimit1, Time: 600, RequiredNode: Worker1}
+		sleepPod3Configs := k8s.SleepPodConfig{Name: "sleepjob3", NS: dev, Mem: sleepPodMemLimit1, Time: 600, RequiredNode: Worker1, AppID: "test01"}
 
-		// random uid in pod spec to ensure this Non-opted pod doesn't become originator
-		sleepPod4Configs := k8s.SleepPodConfig{Name: "sleepjob4", NS: dev, Mem: sleepPodMemLimit2, Time: 600, Optedout: true, UID: types.UID("ddd")}
+		// add the same AppID to sleepPod3Configs so that sleepPod4Configs is not the originator pod
+		sleepPod4Configs := k8s.SleepPodConfig{Name: "sleepjob4", NS: dev, Mem: sleepPodMemLimit2, Time: 600, Optedout: true, AppID: "test01"}
 		sleepPod5Configs := k8s.SleepPodConfig{Name: "sleepjob5", NS: dev, Mem: sleepPodMemLimit2, Time: 600, Optedout: false}
 		sleepPod6Configs := k8s.SleepPodConfig{Name: "sleepjob6", NS: dev, Mem: sleepPodMemLimit2, Time: 600, Optedout: false}
-		sleepPod7Configs := k8s.SleepPodConfig{Name: "sleepjob7", NS: dev, Mem: sleepPodMemLimit2, Time: 600, RequiredNode: "yk8s-worker2"}
+		sleepPod7Configs := k8s.SleepPodConfig{Name: "sleepjob7", NS: dev, Mem: sleepPodMemLimit2, Time: 600, RequiredNode: Worker2}
 
 		for _, config := range []k8s.SleepPodConfig{sleepPodConfigs, sleepPod2Configs,
 			sleepPod3Configs, sleepPod4Configs, sleepPod5Configs, sleepPod6Configs} {
@@ -191,7 +217,7 @@ var _ = ginkgo.Describe("SimplePreemptor", func() {
 			600)
 		gomega.Ω(err).NotTo(gomega.HaveOccurred())
 
-		// assert sleeppod6 is killed
+		// assert sleeppod4 is killed
 		err = kClient.WaitForPodEvent(dev, "sleepjob4", "Killing", 1200)
 		gomega.Ω(err).NotTo(gomega.HaveOccurred())
 
@@ -222,30 +248,4 @@ var _ = ginkgo.Describe("SimplePreemptor", func() {
 			fmt.Fprintf(ginkgo.GinkgoWriter, "Failed to get pods from namespace %s - reason is %s\n", ns.Name, err.Error())
 		}
 	})
-
-	ginkgo.AfterSuite(func() {
-		ginkgo.By("Check Yunikorn's health")
-		checks, err := yunikorn.GetFailedHealthChecks()
-		Ω(err).NotTo(gomega.HaveOccurred())
-		Ω(checks).To(gomega.Equal(""), checks)
-
-		testDescription := ginkgo.CurrentGinkgoTestDescription()
-		if testDescription.Failed {
-			tests.LogTestClusterInfoWrapper(testDescription.TestText, []string{ns.Name})
-			tests.LogYunikornContainer(testDescription.TestText)
-		}
-		ginkgo.By("Tearing down namespace: " + ns.Name)
-		err = kClient.TearDownNamespace(ns.Name)
-		Ω(err).NotTo(gomega.HaveOccurred())
-
-		ginkgo.By("Restoring the old config maps")
-		var c, err1 = kClient.GetConfigMaps(configmanager.YuniKornTestConfig.YkNamespace,
-			configmanager.DefaultYuniKornConfigMap)
-		Ω(err1).NotTo(gomega.HaveOccurred())
-		Ω(c).NotTo(gomega.BeNil())
-		c.Data = oldConfigMap.Data
-		var e, err3 = kClient.UpdateConfigMap(c, configmanager.YuniKornTestConfig.YkNamespace)
-		Ω(err3).NotTo(gomega.HaveOccurred())
-		Ω(e).NotTo(gomega.BeNil())
-	})
 })
