@@ -26,9 +26,9 @@ import (
 
 	v1 "k8s.io/api/core/v1"
 
-	"github.com/onsi/ginkgo/reporters"
+	"github.com/onsi/ginkgo/v2/reporters"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
@@ -53,10 +53,18 @@ var _ = AfterSuite(func() {
 })
 
 func TestStateAwareAppScheduling(t *testing.T) {
-	RegisterFailHandler(Fail)
-	junitReporter := reporters.NewJUnitReporter(
-		filepath.Join(configmanager.YuniKornTestConfig.LogDir, "StateAwareAppScheduling_junit.xml"))
-	ginkgo.RunSpecsWithDefaultAndCustomReporters(t, "TestStateAwareAppScheduling", []ginkgo.Reporter{junitReporter})
+	ginkgo.ReportAfterSuite("TestStateAwareAppScheduling", func(report ginkgo.Report) {
+		err := common.CreateJUnitReportDir()
+		Ω(err).NotTo(gomega.HaveOccurred())
+		err = reporters.GenerateJUnitReportWithConfig(
+			report,
+			filepath.Join(configmanager.YuniKornTestConfig.LogDir, "TEST-stateaware_app_scheduling_junit.xml"),
+			reporters.JunitReportConfig{OmitSpecLabels: true},
+		)
+		Ω(err).NotTo(HaveOccurred())
+	})
+	gomega.RegisterFailHandler(ginkgo.Fail)
+	ginkgo.RunSpecs(t, "TestStateAwareAppScheduling", ginkgo.Label("TestStateAwareAppScheduling"))
 }
 
 // Declarations for Ginkgo DSL
