@@ -19,13 +19,12 @@
 import { Component, OnInit } from '@angular/core';
 import { NgxSpinnerService } from 'ngx-spinner';
 import { finalize } from 'rxjs/operators';
-
 import { SchedulerService } from '@app/services/scheduler/scheduler.service';
-import { ClusterInfo } from '@app/models/cluster-info.model';
+import { BuildInfo, ClusterInfo } from '@app/models/cluster-info.model';
 import { DonutDataItem } from '@app/models/donut-data.model';
 import { AreaDataItem } from '@app/models/area-data.model';
 import { HistoryInfo } from '@app/models/history-info.model';
-import { Partition } from '@app/models/partition-info.model';
+import { Applications, Partition } from '@app/models/partition-info.model';
 import { EventBusService, EventMap } from '@app/services/event-bus/event-bus.service';
 import { NOT_AVAILABLE } from '@app/utils/constants';
 
@@ -39,11 +38,15 @@ export class DashboardComponent implements OnInit {
   partitionList: Partition[] = [];
   nodeSortPolicy = '';
   partitionName = '';
+  totalNodes = '';
+  totalApplications = '';
+  totalContainers = '';
   appStatusData: DonutDataItem[] = [];
   containerStatusData: DonutDataItem[] = [];
   appHistoryData: AreaDataItem[] = [];
   containerHistoryData: AreaDataItem[] = [];
   clusterInfo: ClusterInfo = this.getEmptyClusterInfo();
+  buildInfo: BuildInfo = this.getEmptyBuildInfo();
   initialAppHistory: HistoryInfo[] = [];
   initialContainerHistory: HistoryInfo[] = [];
 
@@ -69,8 +72,9 @@ export class DashboardComponent implements OnInit {
         if (list && list[0]) {
           this.clusterInfo = list[0];
           this.clusterInfo.clusterStatus = 'Active';
-          this.updateAppStatusData(this.clusterInfo);
-          this.updateContainerStatusData(this.clusterInfo);
+          if (list[0].rmBuildInformation && list[0].rmBuildInformation[0]) {
+            this.buildInfo = list[0].rmBuildInformation[0];
+          }
         }
       });
 
@@ -83,9 +87,17 @@ export class DashboardComponent implements OnInit {
           : NOT_AVAILABLE;
 
         this.partitionName = list[0].name || NOT_AVAILABLE;
+        this.totalNodes = String(list[0].totalNodes);
+        this.totalApplications = String(list[0].applications.total);
+        this.totalContainers = String(list[0].totalContainers);
+        this.updateAppStatusData(list[0].applications);
+        this.updateContainerStatusData(list[0]);
       } else {
         this.nodeSortPolicy = NOT_AVAILABLE;
         this.partitionName = NOT_AVAILABLE;
+        this.totalNodes = NOT_AVAILABLE;
+        this.totalApplications = NOT_AVAILABLE;
+        this.totalContainers = NOT_AVAILABLE;
       }
     });
 
@@ -100,27 +112,31 @@ export class DashboardComponent implements OnInit {
     });
 
     this.eventBus.getEvent(EventMap.LayoutChangedEvent).subscribe(() => {
-      this.updateAppStatusData(this.clusterInfo);
-      this.updateContainerStatusData(this.clusterInfo);
+      this.updateAppStatusData(this.partitionList[0].applications);
+      this.updateContainerStatusData(this.partitionList[0]);
       this.appHistoryData = this.getAreaChartData(this.initialAppHistory);
       this.containerHistoryData = this.getAreaChartData(this.initialContainerHistory);
     });
   }
 
-  updateAppStatusData(info: ClusterInfo) {
-    this.appStatusData = [
-      new DonutDataItem('Failed', +info.failedApplications, '#cc6164'),
-      new DonutDataItem('Pending', +info.pendingApplications, '#facc54'),
-      new DonutDataItem('Running', +info.runningApplications, '#26bbf0'),
-      new DonutDataItem('Completed', +info.completedApplications, '#60cea5'),
-    ];
+  updateAppStatusData(applications: Applications) {
+    this.appStatusData = []
+    if (applications.New) this.appStatusData.push(new DonutDataItem('New', applications.New, '#facc54'))
+    if (applications.Accepted) this.appStatusData.push(new DonutDataItem('Accepted', applications.Accepted, '#facc54'))
+    if (applications.Starting) this.appStatusData.push(new DonutDataItem('Starting', applications.Starting, '#26bbf0'))
+    if (applications.Running) this.appStatusData.push(new DonutDataItem('Running', applications.Running, '#26bbf0'))
+    if (applications.Rejected) this.appStatusData.push(new DonutDataItem('Rejected', applications.Rejected, '#cc6164'))
+    if (applications.Completing) this.appStatusData.push(new DonutDataItem('Completing', applications.Completing, '#60cea5'))
+    if (applications.Completed) this.appStatusData.push(new DonutDataItem('Completed', applications.Completed, '#60cea5'))
+    if (applications.Failing) this.appStatusData.push(new DonutDataItem('Failing', applications.Failing, '#cc6164'))
+    if (applications.Failed) this.appStatusData.push(new DonutDataItem('Failed', applications.Failed, '#cc6164'))
+    if (applications.Expired) this.appStatusData.push(new DonutDataItem('Expired', applications.Expired, '#cc6164'))
+    if (applications.Resuming) this.appStatusData.push(new DonutDataItem('Resuming', applications.Resuming, '#facc54'))
   }
 
-  updateContainerStatusData(info: ClusterInfo) {
+  updateContainerStatusData(info: Partition) {
     this.containerStatusData = [
-      new DonutDataItem('Failed', +info.failedContainers, '#cc6164'),
-      new DonutDataItem('Pending', +info.pendingContainers, '#facc54'),
-      new DonutDataItem('Running', +info.runningContainers, '#26bbf0'),
+      new DonutDataItem('Running', +info.totalContainers, '#26bbf0'),
     ];
   }
 
@@ -130,20 +146,20 @@ export class DashboardComponent implements OnInit {
 
   getEmptyClusterInfo(): ClusterInfo {
     return {
+      startTime: 0,
+      rmBuildInformation: [],
+      partition: NOT_AVAILABLE,
       clusterName: NOT_AVAILABLE,
       clusterStatus: NOT_AVAILABLE,
-      totalApplications: NOT_AVAILABLE,
-      failedApplications: NOT_AVAILABLE,
-      pendingApplications: NOT_AVAILABLE,
-      runningApplications: NOT_AVAILABLE,
-      completedApplications: NOT_AVAILABLE,
-      totalContainers: NOT_AVAILABLE,
-      failedContainers: NOT_AVAILABLE,
-      pendingContainers: NOT_AVAILABLE,
-      runningContainers: NOT_AVAILABLE,
-      activeNodes: NOT_AVAILABLE,
-      totalNodes: NOT_AVAILABLE,
-      failedNodes: NOT_AVAILABLE,
+    };
+  }
+
+  getEmptyBuildInfo(): BuildInfo {
+    return {
+      buildDate: NOT_AVAILABLE,
+      buildVersion: NOT_AVAILABLE,
+      isPluginVersion: NOT_AVAILABLE,
+      rmId: NOT_AVAILABLE,
     };
   }
 }
